/* This file is part of
* ======================================================
* 
*           LyX, the High Level Word Processor
* 	 
*	    Copyright (C) 1995 Matthias Ettrich
*
*======================================================*/

#include"inset.h"

extern Widget toplevel;


char Inset::Deletable() {
  return 1;
}


char Inset::Editable() {
  return 0;
}

char Inset::AutoDelete() {
  return 0;
}

void Inset::Edit() {
}

LyXFont Inset::ConvertFont(LyXFont font) {
   return font;
}


/* Quotes. Used for the various quotes. German, English, French, all either
* double or single */ 
   
InsetQuotes::InsetQuotes(char* string) {
   kind = new char[4];
   contents = new char[2];
   ParseString(string);
}

InsetQuotes::~InsetQuotes() {
   delete(kind);
   delete(contents);
}

void InsetQuotes::ParseString(char* string) {
 if (string[0])
     kind[0]=string[0];
   else
     kind[0]=' ';
   if (string[1])
     kind[1]=string[1];
   else
     kind[1]=' ';
   if (string[2])
     kind[2]=string[2];
   else
     kind[2]=' ';
   kind[3] = '\0';
   
   switch (kind[0]) {
    case 'g':			       /* german quotes  */
      if (kind[1]=='l')
      	contents[0]=',';
      else
      	contents[0]='`';
      break;
    case 'f':			       /* french quotes  */
      if (kind[1]=='l')
      	contents[0]='<';
      else
      	contents[0]='>';
      break;
    default:			       /* english quotes  */
      if (kind[1]=='l')
      	contents[0]='`';
      else
      	contents[0]='\'';
      break;
   }
   
   /* single or double quotes */ 
   if (kind[2]=='s')
     contents[1]='\0';
   else
     contents[1]=contents[0];
  
}

int InsetQuotes::Ascent(LyXFont font) {
   if (contents[1])
     return Maximum(LyXAscent(font, contents[0]),
		    LyXAscent(font, contents[1]));
   else
     return LyXAscent(font, contents[0]);
}

int InsetQuotes::Descent(LyXFont font) {
   if (contents[1])
     return Maximum(LyXDescent(font, contents[0]),
		    LyXDescent(font, contents[1]));
   else
     return LyXDescent(font, contents[0]);
}

int InsetQuotes::Width(LyXFont font) {
   int w = LyXWidth(font, contents[0]);
   
   if (contents[1])
     w += LyXWidth(font, ',');       /* double quotes are wider*/
   
   return w;
}

InsetQuotes::InsetQuotes() {
   kind = new char[4];
   contents = new char[2];
   ParseString("eld");
}

LyXFont InsetQuotes::ConvertFont(LyXFont font) {
   /* quotes-insets cannot be latex of any kind */
   font.latex = LYX_NO_LATEX;
   return font;
}

void InsetQuotes::Draw(LyXFont font, unsigned long  pm, int baseline, float &x) {

   /* draw it! */ 
   XDrawString(fl_display,
	       pm,
	       LyXGetGC(font),
	       (int) x, baseline,
	       contents, 1);
   
   if (contents[1]) {	       /* double quotes*/
      x += LyXWidth(font, ',');
      /* draw the second! */ 
      XDrawString(fl_display,
		  pm,
		  LyXGetGC(font),
		  (int) x, baseline,
		  contents+1, 1);      /* a little pointer hack!  */
   }
   
   /* calculate the width and set the new x position */ 
   x +=  LyXWidth(font, contents[0]);
}


void InsetQuotes::Write(FILE *file) {
   fprintf(file, "Quotes %s", kind);
}

void InsetQuotes::Read(FILE *file) {
   char string[4];
   if (!feof(file))
     string[0] = fgetc(file);
   else
     string[0] = ' ';
   if (!feof(file))
     string[1] = fgetc(file);
   else
     string[1] = ' ';
   if (!feof(file))
     string[2] = fgetc(file);
   else
     string[2] = ' ';
   string[3]='\0';
   
   ParseString(string);
}

void InsetQuotes::Latex(FILE *file){
   switch (kind[0]) {
    case 'g':			       /* german quotes  */
      fprintf(file, "\\%c%cq", kind[0], kind[1]);
      if (kind[2]!='s')
      	fprintf(file, "q");
      fprintf(file,"{}");
      break;
    case 'f':			       /* french quotes  */
      fprintf(file, "\\%c%cq", kind[0], kind[1]);
      if (kind[2]!='s')
      	fprintf(file, "q");
      fprintf(file,"{}");
      break;
    default:			       /* english quotes  */
      fprintf(file, "%c", contents[0]);
      if (kind[2]!='s')
      	fprintf(file, "%c", contents[0]);
      break;
   }
}


Inset* InsetQuotes::Clone(){
  InsetQuotes *result;
  result = new InsetQuotes(kind);
  return result;
}






/* Latex. Used to insert Latex-Code automatically */

InsetLatex::InsetLatex(char* string) {
  contents = new char[20];
  int i;
  for (i=0; string[i]; i++)
    contents[i]=string[i];
  contents[i]=string[i];
}

InsetLatex::~InsetLatex() {
   delete(contents);
}

int InsetLatex::Ascent(LyXFont font) {
   int max=0;
   int i;
   for (i=0; contents[i];i++) {
     if (LyXAscent(font, contents[i])> max)
       max = LyXAscent(font, contents[i]);
   }
   return max;
}

int InsetLatex::Descent(LyXFont font) {
   int max=0;
   int i;
   for (i=0; contents[i];i++) {
     if (LyXDescent(font, contents[i])> max)
       max = LyXDescent(font, contents[i]);
   }
   return max;
}

int InsetLatex::Width(LyXFont font) {
  int n;
  for (n=0; contents[n];n++);	/* get the size */

   int w = LyXTextWidth(font, contents, n );
   
   w +=2;
   
   return w;
}

InsetLatex::InsetLatex() {
   contents = new char[20];
}



void InsetLatex::Draw(LyXFont font, unsigned long  pm, int baseline, float &x) {
   int n;
   for (n=0; contents[n];n++);	/* get the size */
   
   /* Latex-insets are always LaTeX, so just correct the font */ 
   if (font.latex == LYX_NO_LATEX)
      font.latex = LYX_LATEX;
   
   /* draw it! */ 
   
   x += 1;
   XFillRectangle(fl_display,
		  pm,
		 LyXGetLightedGC(),
		 (int) x, baseline- Ascent(font),
		 Width(font)-2, (Ascent(font)+Descent(font)));
		 
  XDrawString(fl_display,
	      pm,
	      LyXGetGC(font),
	      (int) x, baseline,
	      contents, n);
  
  
  x +=  Width(font) - 1;

}


void InsetLatex::Write(FILE *file) {
   fprintf(file, "Latex %s\n", contents);
}

void InsetLatex::Read(FILE *file) {
  int i = 0;
  char c='\0';

  while (!feof(file) && (c = fgetc(file)) != '\n') {
    contents[i]=c;
    i++;
  }
  
  contents[i]='\0';

}

void InsetLatex::Latex(FILE *file){
  fprintf(file, "%s", contents);

};

char InsetLatex::Deletable() {
  return 0;
}


Inset* InsetLatex::Clone(){
  InsetLatex *result = new InsetLatex(contents);
  return result;
}




/* FormulaLatex. Used to insert FormulaLatex-Code automatically */

InsetFormulaLatex::InsetFormulaLatex(char* string) {
  contents = new char[20];
  int i;
  for (i=0; string[i]; i++)
    contents[i]=string[i];
  contents[i]=string[i];
}

InsetFormulaLatex::~InsetFormulaLatex() {
   delete(contents);
}

int InsetFormulaLatex::Ascent(LyXFont font) {
   int max=0;
   int i;
   for (i=0; contents[i];i++) {
     if (LyXAscent(font, contents[i])> max)
       max = LyXAscent(font, contents[i]);
   }
   return max;
}

int InsetFormulaLatex::Descent(LyXFont font) {
   int max=0;
   int i;
   for (i=0; contents[i];i++) {
     if (LyXDescent(font, contents[i])> max)
       max = LyXDescent(font, contents[i]);
   }
   return max;
}


int InsetFormulaLatex::Width(LyXFont font) {
  int n;
  for (n=0; contents[n];n++);	/* get the size */

   int w = LyXTextWidth(font, contents, n );
   
   w +=2;
   
   return w;
}

InsetFormulaLatex::InsetFormulaLatex() {
   contents = new char[20];
}

LyXFont InsetFormulaLatex::ConvertFont(LyXFont font) {
   /* FormulaLatex-insets are always formulas*/ 
   font.latex = LYX_FORMULA_LATEX;
   return font;
}

void InsetFormulaLatex::Draw(LyXFont font, unsigned long  pm, int baseline, float &x) {
  int n;
  for (n=0; contents[n];n++);	/* get the size */

  /* draw it! */ 
  
  x += 1;
  XFillRectangle(fl_display,
		 pm,
		 LyXGetLightedGC(),
		 (int) x, baseline- Ascent(font),
		 Width(font)-2, (Ascent(font)+Descent(font)));
		 
  XDrawString(fl_display,
	      pm,
	      LyXGetGC(font),
	      (int) x, baseline,
	      contents, n);
  
  
  x +=  Width(font) - 1;
}


void InsetFormulaLatex::Write(FILE *file) {
   fprintf(file, "FormulaLatex %s\n", contents);
}

void InsetFormulaLatex::Read(FILE *file) {
  int i = 0;
  char c='\0';

  while (!feof(file) && (c = fgetc(file)) != '\n') {
    contents[i]=c;
    i++;
  }
  
  contents[i]='\0';

}

void InsetFormulaLatex::Latex(FILE *file){
  fprintf(file, "%s", contents);

};

char InsetFormulaLatex::Deletable() {
  return 0;
}

Inset* InsetFormulaLatex::Clone(){
  InsetFormulaLatex *result = new InsetFormulaLatex(contents);
  return result;
}






/* LatexDel. Used to insert Latex-Code automatically */

InsetLatexDel::InsetLatexDel(char* string) {
  contents = new char[20];
  int i;
  for (i=0; string[i]; i++)
    contents[i]=string[i];
  contents[i]=string[i];
}


InsetLatexDel::~InsetLatexDel() {
   delete(contents);
}


int InsetLatexDel::Ascent(LyXFont font) {
   int max=0;
   int i;
   for (i=0; contents[i];i++) {
     if (LyXAscent(font, contents[i])> max)
       max = LyXAscent(font, contents[i]);
   }
   return max;
}

int InsetLatexDel::Descent(LyXFont font) {
   int max=0;
   int i;
   for (i=0; contents[i];i++) {
     if (LyXDescent(font, contents[i])> max)
       max = LyXDescent(font, contents[i]);
   }
   return max;
}

int InsetLatexDel::Width(LyXFont font) {
  int n;
  for (n=0; contents[n];n++);	/* get the size */

   int w = LyXTextWidth(font, contents, n );
   
   w +=2;
   
   return w;
}

InsetLatexDel::InsetLatexDel() {
   contents = new char[20];
}

void InsetLatexDel::Draw(LyXFont font, unsigned long  pm, int baseline, float &x) {
  int n;
  for (n=0; contents[n];n++);	/* get the size */
   
   /* Latex-insets are always LaTeX, so just correct the font */ 
   if (font.latex == LYX_NO_LATEX)
      font.latex = LYX_LATEX;

   /* draw it! */ 
  
  x += 1;
  XFillRectangle(fl_display,
		 pm,
		 LyXGetLightedGC(),
		 (int) x, baseline- Ascent(font),
		 Width(font)-2, (Ascent(font)+Descent(font)));
		 
  XDrawString(fl_display,
	      pm,
	      LyXGetGC(font),
	      (int) x, baseline,
	      contents, n);
  
  
  x +=  Width(font) - 1;

}


void InsetLatexDel::Write(FILE *file) {
   fprintf(file, "LatexDel %s\n", contents);
}

void InsetLatexDel::Read(FILE *file) {
  int i = 0;
  char c='\0';

  while (!feof(file) && (c = fgetc(file)) != '\n') {
    contents[i]=c;
    i++;
  }
  
  contents[i]='\0';

}

void InsetLatexDel::Latex(FILE *file){
  fprintf(file, "%s", contents);

};

char InsetLatexDel::Deletable() {
  return 1;
}

Inset* InsetLatexDel::Clone(){
  InsetLatexDel *result = new InsetLatexDel(contents);
  return result;
}






/* Error, used for the LaTeX-Error Messages */

InsetError::InsetError(char* string) {
  contents = new char[300];
  int i;
  for (i=0; string[i]; i++)
    contents[i]=string[i];
  contents[i]=string[i];
}


InsetError::~InsetError() {
   delete(contents);
}


int InsetError::Ascent(LyXFont font) {
   return LyXAscent(font, 'E')+3;
}

int InsetError::Descent(LyXFont font) {
   return LyXDescent(font, 'E')+3;
}

int InsetError::Width(LyXFont font) {
   int w = LyXTextWidth(font, "Error", 5 );
   
   w +=6;
   
   return w;
}

InsetError::InsetError() {
   contents = new char[100];
}

void InsetError::Draw(LyXFont font, unsigned long  pm, int baseline, float &x) {
   
   /* Latex-insets are always LaTeX, so just correct the font */ 
   
   font.latex = LYX_LATEX;
   
   
   /* draw it! */ 
   
   x += 1;
   XFillRectangle(fl_display,
		  pm,
		  LyXGetLightedGC(),
		  (int) x, baseline- Ascent(font),
		  Width(font)-2, (Ascent(font)+Descent(font)));
   
   XDrawRectangle(fl_display,
		  pm,
		  LyXGetCopyGC(),
		  (int) x-1, baseline- Ascent(font)-1,
		  Width(font)-1, (Ascent(font)+Descent(font))+1);
   
   XDrawString(fl_display,
	       pm,
	       LyXGetGC(font),
	       (int) x+3, baseline,
	       "Error", 5);
   
   
   x +=  Width(font) - 1;

}


void InsetError::Write(FILE *file) {
}

void InsetError::Read(FILE *file) {
}

void InsetError::Latex(FILE *file){
};

char InsetError::AutoDelete() {
  return 1;
}

char InsetError::Editable() {
  return 1;
}


static void CloseErrorCB(FL_OBJECT *ob, long data) {
   FL_FORM *form = (FL_FORM*) data;
   fl_hide_form(form);
}

void InsetError::Edit() {
   static   FL_FORM *form = NULL;
   static    FL_OBJECT *strobj = NULL;
   
   if (!form) {
      FL_OBJECT *obj;
      form = fl_bgn_form(FL_UP_BOX,400,140);
      strobj = fl_add_box(FL_FRAME_BOX,10,50,380,80,"");
      fl_set_object_color(strobj,FL_MCOL,FL_MCOL);
      obj = fl_add_button(FL_RETURN_BUTTON,160,10,120,30,"OK");
      fl_set_object_callback(obj, CloseErrorCB, (long) form);
      fl_end_form();
   }
   fl_set_object_label(strobj, contents);
   fl_show_form(form,FL_PLACE_CENTER | FL_FREE_SIZE, FL_FULLBORDER, "LaTeX-Error");
}
   
Inset* InsetError::Clone(){
      InsetError *result = new InsetError(contents);
   return result;
}

