/* This file is part of
* ======================================================
* 
*           LyX, the High Level Word Processor
* 	 
*	    Copyright (C) 1995 Matthias Ettrich
*
*======================================================*/

#include"xdefinitions.h"


/* Quotes. Used for the various quotes. German, English, French, all either
* double or single */ 

class InsetQuotes: public Inset {
 private:
   char* contents;
   char* kind;
   
   void ParseString(char* string);
   
 public:
   ~InsetQuotes();
   
   /* the construktor works like this:
    * 
    * fls <- french single quote left 
    * grd <- german double quote right 
    * 
    * etc. */ 
   
   InsetQuotes(char* string);
   InsetQuotes();
   int Ascent(LyXFont font);
   int Descent(LyXFont font);
   int Width(LyXFont font);
   void Draw(LyXFont font, unsigned long  pm, int baseline, float &x);
   LyXFont ConvertFont(LyXFont font);
   void Write(FILE *file);
   void Read(FILE *file);
   void Latex(FILE *file);
   Inset* Clone();
};




/* Latex. Used to insert LaTeX-Code automatically */

class InsetLatex: public Inset {
 private:
   char* contents;
   
 public:
   ~InsetLatex();
   
   InsetLatex(char* string);
   InsetLatex();
   int Ascent(LyXFont font);
   int Descent(LyXFont font);
   int Width(LyXFont font);
   void Draw(LyXFont font, unsigned long  pm, int baseline, float &x);
   void Write(FILE *file);
   void Read(FILE *file);
   void Latex(FILE *file);
   char Deletable();
   Inset* Clone();
};



/* FormulaLatex. Used to insert FormulaLatex-Code automatically */

class InsetFormulaLatex: public Inset {
 private:
   char* contents;
   
 public:
   ~InsetFormulaLatex();
   
   InsetFormulaLatex(char* string);
   InsetFormulaLatex();
   int Ascent(LyXFont font);
   int Descent(LyXFont font);
   int Width(LyXFont font);
   void Draw(LyXFont font, unsigned long  pm, int baseline, float &x);
   LyXFont ConvertFont(LyXFont font);
   void Write(FILE *file);
   void Read(FILE *file);
   void Latex(FILE *file);
   char  Deletable();
   Inset* Clone();

   
};

class InsetLatexDel: public Inset {
 private:
   char* contents;
   
 public:
   ~InsetLatexDel();
   
   InsetLatexDel(char* string);
   InsetLatexDel();
   int Ascent(LyXFont font);
   int Descent(LyXFont font);
   int Width(LyXFont font);
   void Draw(LyXFont font, unsigned long  pm, int baseline, float &x);
   void Write(FILE *file);
   void Read(FILE *file);
   void Latex(FILE *file);
   char Deletable();
   Inset* Clone();
};



class InsetError: public Inset {
 private:
   char* contents;
   
 public:
   ~InsetError();
   
   InsetError(char* string);
   InsetError();
   int Ascent(LyXFont font);
   int Descent(LyXFont font);
   int Width(LyXFont font);
   void Draw(LyXFont font, unsigned long  pm, int baseline, float &x);
   void Write(FILE *file);
   void Read(FILE *file);
   void Latex(FILE *file);
   char AutoDelete();
   void Edit();
   char Editable();
   Inset* Clone();
};











