/* This file is part of
* ======================================================
* 
*           LyX, the High Level Word Processor
* 	 
*	    Copyright (C) 1995 Matthias Ettrich
*
*======================================================*/

#include "definitions.h"
#include "layout.h"
/* #include "string.h" */ 

#define TRUE 1
#define FALSE 0

#define MAX_LAYOUTS 50
#define MAX_CLASSES 5






static LyXTextClass lyxstyle[MAX_CLASSES];


static LyXLayout staticlayouts[MAX_CLASSES][MAX_LAYOUTS];


static number_of_defined_classes = 0;



static void InitLayouts(char tc) {
   lyxstyle[tc].style  = &staticlayouts[tc][0];
   /* new LyXLayout[MAX_LAYOUTS]; */ 
   lyxstyle[tc].number_of_defined_layouts = 0;
   for (int i=0; i < MAX_LAYOUTS; i++) {
      lyxstyle[tc].style[i].fill_top = FALSE;
      lyxstyle[tc].style[i].fill_bottom = FALSE;
      lyxstyle[tc].style[i].newline_allowed = TRUE;
      lyxstyle[tc].style[i].nextnoindent = FALSE;
   }
   
}

char NumberOfClass(const char* textclass) {
   int i = 0;
   
   while (i < number_of_defined_classes &&
	  !StringEqual(textclass, lyxstyle[i].name))
      i++;
   
   if (i >= number_of_defined_classes) {
      i = 0;
   }
   
   return i;
}

LyXLayout *style(char textclass, char layout) {
   if (layout < lyxstyle[textclass].number_of_defined_layouts)
     return &lyxstyle[textclass].style[layout];
   else {
      return &lyxstyle[textclass].style[0];
   }
}


char NumberOfLayout(char textclass, const char* name) {
   int i = 0;
   while (i < lyxstyle[textclass].number_of_defined_layouts &&
	  !StringEqual(name, lyxstyle[textclass].style[i].name))
      i++;
   
   if (i >= lyxstyle[textclass].number_of_defined_layouts) {
      if (StringEqual(name, "dummy"))
      	 i = LYX_DUMMY_LAYOUT;
      else
      	 i = 0;
   }
   
   return i;
}




char* NameOfLayout(char textclass, char layout) {
   if (layout < lyxstyle[textclass].number_of_defined_layouts)
      return lyxstyle[textclass].style[layout].name;
   else
      return "dummy";
}

char* NameOfClass(char number) {
   if (number < number_of_defined_classes)
      return lyxstyle[number].name;
   else
      return "dummy";
}

LyXTextClass * TextClass(char textclass) {
   if (textclass < number_of_defined_classes)
      return &lyxstyle[textclass];
   else
      return &lyxstyle[0];
}


static void SetStandard(char &textclass, char &layout) {
   lyxstyle[textclass].style[layout].bordertype = BORDER_STATIC;
   lyxstyle[textclass].style[layout].name = "Standard";
   lyxstyle[textclass].style[layout].latextype = LATEX_PARAGRAPH;
   lyxstyle[textclass].style[layout].latexname = "dummy";
   
   
   lyxstyle[textclass].style[layout].font = GetLyXFont(LYX_ROMAN_FAMILY, 
				 LYX_MEDIUM_SERIES,
				 LYX_UP_SHAPE,
				 LYX_SIZE_NORMAL,
				 LYX_NO_LATEX);
   lyxstyle[textclass].style[layout].labelfont = GetLyXFont(LYX_ROMAN_FAMILY,
			      LYX_MEDIUM_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_NORMAL, LYX_NO_LATEX);
   
   lyxstyle[textclass].style[layout].leftborder = "";
   lyxstyle[textclass].style[layout].rightborder = "";

   lyxstyle[textclass].style[layout].labelsep = "";
   lyxstyle[textclass].style[layout].parindent = "MM";
   lyxstyle[textclass].style[layout].parskip = 0.4;
   lyxstyle[textclass].style[layout].itemsep = 0;
   
   lyxstyle[textclass].style[layout].topsep = 0;
   lyxstyle[textclass].style[layout].bottomsep = 0;
   lyxstyle[textclass].style[layout].parsep = 0.0;
   
   lyxstyle[textclass].style[layout].align = LYX_ALIGN_BLOCK;
   lyxstyle[textclass].style[layout].alignpossible = LYX_ALIGN_BLOCK | LYX_ALIGN_LEFT | LYX_ALIGN_RIGHT | LYX_ALIGN_CENTER;
   lyxstyle[textclass].style[layout].labeltype = LABEL_NO_LABEL;
   
   lyxstyle[textclass].style[layout].baselinestretch = 1.5;

   lyxstyle[textclass].style[layout].labelstring = "";   
}

static void SetSections(char &textclass, char &layout) {
   /* section */
   layout++;
   lyxstyle[textclass].style[layout].bordertype = BORDER_DYNAMIC;
   lyxstyle[textclass].style[layout].name = "Section";
   lyxstyle[textclass].style[layout].latextype = LATEX_COMMAND;
   lyxstyle[textclass].style[layout].latexname = "section";

   lyxstyle[textclass].style[layout].font = GetLyXFont(LYX_ROMAN_FAMILY, 
				 LYX_BOLD_SERIES,
				 LYX_UP_SHAPE,
				 LYX_SIZE_LARGER, LYX_NO_LATEX);
   lyxstyle[textclass].style[layout].labelfont = GetLyXFont(LYX_ROMAN_FAMILY, 
				      LYX_BOLD_SERIES,
				      LYX_UP_SHAPE,
				      LYX_SIZE_LARGER, LYX_NO_LATEX);
   
   lyxstyle[textclass].style[layout].nextnoindent = TRUE;
   lyxstyle[textclass].style[layout].leftborder = "";
   lyxstyle[textclass].style[layout].rightborder = "";
   
   lyxstyle[textclass].style[layout].labelsep = "   ";
   lyxstyle[textclass].style[layout].labelindent = "";
   lyxstyle[textclass].style[layout].parindent = "";
   lyxstyle[textclass].style[layout].parskip = 0.4;
   lyxstyle[textclass].style[layout].itemsep = 0;
   
   lyxstyle[textclass].style[layout].topsep = 1.3;
   lyxstyle[textclass].style[layout].bottomsep = 0.7;
   lyxstyle[textclass].style[layout].parsep = 0.7;
   
   lyxstyle[textclass].style[layout].align = LYX_ALIGN_BLOCK;
   lyxstyle[textclass].style[layout].alignpossible = LYX_ALIGN_BLOCK | LYX_ALIGN_LEFT;
   lyxstyle[textclass].style[layout].labeltype = LABEL_COUNTER_SECTION;
   
   lyxstyle[textclass].style[layout].baselinestretch = 1.5;
   lyxstyle[textclass].style[layout].labelstring = "";   
   
   /* subsection */
   layout++;
   lyxstyle[textclass].style[layout].bordertype = BORDER_DYNAMIC;
   lyxstyle[textclass].style[layout].name = "Subsection";
   lyxstyle[textclass].style[layout].latextype = LATEX_COMMAND;
   lyxstyle[textclass].style[layout].latexname = "subsection";

   lyxstyle[textclass].style[layout].font = GetLyXFont(LYX_ROMAN_FAMILY,
			      LYX_BOLD_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_LARGE, LYX_NO_LATEX);
   lyxstyle[textclass].style[layout].labelfont = GetLyXFont(LYX_ROMAN_FAMILY, 
			      LYX_BOLD_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_LARGE, LYX_NO_LATEX);

   lyxstyle[textclass].style[layout].nextnoindent = TRUE;
   lyxstyle[textclass].style[layout].leftborder = "";
   lyxstyle[textclass].style[layout].rightborder = "";
   
   lyxstyle[textclass].style[layout].labelsep = "   ";
   lyxstyle[textclass].style[layout].labelindent = "";
   lyxstyle[textclass].style[layout].parindent = "";
   lyxstyle[textclass].style[layout].parskip = 0.4;
   lyxstyle[textclass].style[layout].itemsep = 0;
   
   lyxstyle[textclass].style[layout].topsep = 0.9;
   lyxstyle[textclass].style[layout].bottomsep = 0.5;
   lyxstyle[textclass].style[layout].parsep = 0.5;
   
   lyxstyle[textclass].style[layout].align = LYX_ALIGN_BLOCK;
   lyxstyle[textclass].style[layout].alignpossible = LYX_ALIGN_BLOCK | LYX_ALIGN_LEFT;
   lyxstyle[textclass].style[layout].labeltype = LABEL_COUNTER_SUBSECTION;
   
   lyxstyle[textclass].style[layout].baselinestretch = 1.5;
   lyxstyle[textclass].style[layout].labelstring = "";      
   
   /* subsubsection */
   layout++;
   lyxstyle[textclass].style[layout].bordertype = BORDER_DYNAMIC;
   lyxstyle[textclass].style[layout].name = "Subsubsection";
   lyxstyle[textclass].style[layout].latextype = LATEX_COMMAND;
   lyxstyle[textclass].style[layout].latexname = "subsubsection";

   lyxstyle[textclass].style[layout].font = GetLyXFont(LYX_ROMAN_FAMILY, 
			      LYX_BOLD_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_NORMAL, LYX_NO_LATEX);
   lyxstyle[textclass].style[layout].labelfont = GetLyXFont(LYX_ROMAN_FAMILY,
			      LYX_BOLD_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_NORMAL, LYX_NO_LATEX);

   lyxstyle[textclass].style[layout].nextnoindent = TRUE;
   lyxstyle[textclass].style[layout].leftborder = "";
   lyxstyle[textclass].style[layout].rightborder = "";
   
   lyxstyle[textclass].style[layout].labelsep = "   ";
   lyxstyle[textclass].style[layout].labelindent = "";
   lyxstyle[textclass].style[layout].parindent = "";
   lyxstyle[textclass].style[layout].parskip = 0.4;
   lyxstyle[textclass].style[layout].itemsep = 0;
   
   lyxstyle[textclass].style[layout].topsep = 0.7;
   lyxstyle[textclass].style[layout].bottomsep = 0.4;
   lyxstyle[textclass].style[layout].parsep = 0.4;
   
   lyxstyle[textclass].style[layout].align = LYX_ALIGN_BLOCK;
   lyxstyle[textclass].style[layout].alignpossible = LYX_ALIGN_BLOCK | LYX_ALIGN_LEFT;
   lyxstyle[textclass].style[layout].labeltype = LABEL_COUNTER_SUBSUBSECTION;
   
   lyxstyle[textclass].style[layout].baselinestretch = 1.5;
   lyxstyle[textclass].style[layout].labelstring = "";      

   /* paragraph */
   layout++;
   lyxstyle[textclass].style[layout].bordertype = BORDER_DYNAMIC;
   lyxstyle[textclass].style[layout].name = "Paragraph";
   lyxstyle[textclass].style[layout].latextype = LATEX_COMMAND;
   lyxstyle[textclass].style[layout].latexname = "paragraph";

   lyxstyle[textclass].style[layout].font = GetLyXFont(LYX_ROMAN_FAMILY, 
			      LYX_BOLD_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_NORMAL, LYX_NO_LATEX);
   lyxstyle[textclass].style[layout].labelfont = GetLyXFont(LYX_ROMAN_FAMILY,
			      LYX_BOLD_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_NORMAL, LYX_NO_LATEX);

   lyxstyle[textclass].style[layout].nextnoindent = TRUE;
   lyxstyle[textclass].style[layout].leftborder = "";
   lyxstyle[textclass].style[layout].rightborder = "";
   
   lyxstyle[textclass].style[layout].labelsep = "   ";
   lyxstyle[textclass].style[layout].labelindent = "";
   lyxstyle[textclass].style[layout].parindent = "";
   lyxstyle[textclass].style[layout].parskip = 0.4;
   lyxstyle[textclass].style[layout].itemsep = 0;
   
   lyxstyle[textclass].style[layout].topsep = 0.4;
   lyxstyle[textclass].style[layout].bottomsep = 0;
   lyxstyle[textclass].style[layout].parsep = 0.4;
   
   lyxstyle[textclass].style[layout].align = LYX_ALIGN_BLOCK;
   lyxstyle[textclass].style[layout].alignpossible = LYX_ALIGN_BLOCK | LYX_ALIGN_LEFT;
   lyxstyle[textclass].style[layout].labeltype = LABEL_COUNTER_PARAGRAPH;
   
   lyxstyle[textclass].style[layout].baselinestretch = 1.5;
   lyxstyle[textclass].style[layout].labelstring = "";      

   /* subparagraph */
   layout++;
   lyxstyle[textclass].style[layout].bordertype = BORDER_DYNAMIC;
   lyxstyle[textclass].style[layout].name = "Subparagraph";
   lyxstyle[textclass].style[layout].latextype = LATEX_COMMAND;
   lyxstyle[textclass].style[layout].latexname = "subparagraph";

   lyxstyle[textclass].style[layout].font = GetLyXFont(LYX_ROMAN_FAMILY, 
			      LYX_BOLD_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_NORMAL, LYX_NO_LATEX);
   lyxstyle[textclass].style[layout].labelfont = GetLyXFont(LYX_ROMAN_FAMILY,
			      LYX_BOLD_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_NORMAL, LYX_NO_LATEX);

   lyxstyle[textclass].style[layout].nextnoindent = FALSE;
   lyxstyle[textclass].style[layout].leftborder = "MM";
   lyxstyle[textclass].style[layout].rightborder = "";
   
   lyxstyle[textclass].style[layout].labelsep = "   ";
   lyxstyle[textclass].style[layout].labelindent = "";
   lyxstyle[textclass].style[layout].parindent = "";
   lyxstyle[textclass].style[layout].parskip = 0.4;
   lyxstyle[textclass].style[layout].itemsep = 0;
   
   lyxstyle[textclass].style[layout].topsep = 0.4;
   lyxstyle[textclass].style[layout].bottomsep = 0;
   lyxstyle[textclass].style[layout].parsep = 0.4;
   
   lyxstyle[textclass].style[layout].align = LYX_ALIGN_BLOCK;
   lyxstyle[textclass].style[layout].alignpossible = LYX_ALIGN_BLOCK | LYX_ALIGN_LEFT;
   lyxstyle[textclass].style[layout].labeltype = LABEL_COUNTER_SUBPARAGRAPH;
   
   lyxstyle[textclass].style[layout].baselinestretch = 1.5;
   lyxstyle[textclass].style[layout].labelstring = "";      

}

static void SetSectionBibliography(char &textclass, char &layout) {
   /* bibliography */
   layout++;
   lyxstyle[textclass].style[layout].bordertype = BORDER_FIRST_DYNAMIC;
   lyxstyle[textclass].style[layout].name = "Bibliography";
   lyxstyle[textclass].style[layout].latextype = LATEX_ITEM_ENVIRONEMENT;
   lyxstyle[textclass].style[layout].latexname = "lyxsectionbibliography";

   lyxstyle[textclass].style[layout].font = GetLyXFont(LYX_ROMAN_FAMILY, 
			      LYX_MEDIUM_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_NORMAL, LYX_NO_LATEX);
   lyxstyle[textclass].style[layout].labelfont = GetLyXFont(LYX_ROMAN_FAMILY, 
			      LYX_BOLD_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_NORMAL, LYX_NO_LATEX);

   lyxstyle[textclass].style[layout].nextnoindent = TRUE;
   lyxstyle[textclass].style[layout].leftborder = "MM";
   lyxstyle[textclass].style[layout].rightborder = "";
   
   lyxstyle[textclass].style[layout].labelsep = "";
   lyxstyle[textclass].style[layout].labelindent = "";
   lyxstyle[textclass].style[layout].parindent = "";
   lyxstyle[textclass].style[layout].parskip = 0.4;
   lyxstyle[textclass].style[layout].itemsep = 0;
   
   lyxstyle[textclass].style[layout].topsep = 0.7;
   lyxstyle[textclass].style[layout].bottomsep = 0.7;
   lyxstyle[textclass].style[layout].parsep = 0.5;
   
   lyxstyle[textclass].style[layout].align = LYX_ALIGN_BLOCK;
   lyxstyle[textclass].style[layout].alignpossible = LYX_ALIGN_BLOCK | LYX_ALIGN_LEFT;
   lyxstyle[textclass].style[layout].labeltype = LABEL_TOP_ENVIRONMENT;
   
   lyxstyle[textclass].style[layout].labelstring = "Reference";

   lyxstyle[textclass].style[layout].baselinestretch = 1.5;


}
static void SetChapterBibliography(char &textclass, char &layout) {
   /* bibliography */
   layout++;
   lyxstyle[textclass].style[layout].bordertype = BORDER_FIRST_DYNAMIC;
   lyxstyle[textclass].style[layout].name = "Bibliography";
   lyxstyle[textclass].style[layout].latextype = LATEX_ITEM_ENVIRONEMENT;
   lyxstyle[textclass].style[layout].latexname = "lyxchapterbibliography";

   lyxstyle[textclass].style[layout].font = GetLyXFont(LYX_ROMAN_FAMILY, 
			      LYX_MEDIUM_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_NORMAL, LYX_NO_LATEX);
   lyxstyle[textclass].style[layout].labelfont = GetLyXFont(LYX_ROMAN_FAMILY, 
			      LYX_BOLD_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_NORMAL, LYX_NO_LATEX);

   lyxstyle[textclass].style[layout].nextnoindent = TRUE;
   lyxstyle[textclass].style[layout].leftborder = "MM";
   lyxstyle[textclass].style[layout].rightborder = "";
   
   lyxstyle[textclass].style[layout].labelsep = "";
   lyxstyle[textclass].style[layout].labelindent = "";
   lyxstyle[textclass].style[layout].parindent = "";
   lyxstyle[textclass].style[layout].parskip = 0.4;
   lyxstyle[textclass].style[layout].itemsep = 0;
   
   lyxstyle[textclass].style[layout].topsep = 0.7;
   lyxstyle[textclass].style[layout].bottomsep = 0.7;
   lyxstyle[textclass].style[layout].parsep = 0.5;
   
   lyxstyle[textclass].style[layout].align = LYX_ALIGN_BLOCK;
   lyxstyle[textclass].style[layout].alignpossible = LYX_ALIGN_BLOCK | LYX_ALIGN_LEFT;
   lyxstyle[textclass].style[layout].labeltype = LABEL_TOP_ENVIRONMENT;
   
   lyxstyle[textclass].style[layout].labelstring = "Bibliography";

   lyxstyle[textclass].style[layout].baselinestretch = 1.5;


}

static void SetCommon(char &textclass, char &layout) {
   /* quotation */
   layout++;
   lyxstyle[textclass].style[layout].bordertype = BORDER_STATIC;
   lyxstyle[textclass].style[layout].name = "Quotation";
   lyxstyle[textclass].style[layout].latextype = LATEX_ENVIRONEMENT;
   lyxstyle[textclass].style[layout].latexname = "quotation";

   lyxstyle[textclass].style[layout].font = GetLyXFont(LYX_ROMAN_FAMILY, 
			      LYX_MEDIUM_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_NORMAL, LYX_NO_LATEX);
   lyxstyle[textclass].style[layout].labelfont = GetLyXFont(LYX_ROMAN_FAMILY, 
			      LYX_MEDIUM_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_NORMAL, LYX_NO_LATEX);


   lyxstyle[textclass].style[layout].nextnoindent = TRUE;
   lyxstyle[textclass].style[layout].leftborder = "MMM";
   lyxstyle[textclass].style[layout].rightborder = "MMM";
   
   lyxstyle[textclass].style[layout].labelsep = "";
   lyxstyle[textclass].style[layout].labelindent = "";
   lyxstyle[textclass].style[layout].parindent = "MM";
   lyxstyle[textclass].style[layout].parskip = 0;
   lyxstyle[textclass].style[layout].itemsep = 0;
   
   lyxstyle[textclass].style[layout].topsep = 0.5;
   lyxstyle[textclass].style[layout].bottomsep = 0.5;
   lyxstyle[textclass].style[layout].parsep = 0;
   
   lyxstyle[textclass].style[layout].align = LYX_ALIGN_BLOCK;
   lyxstyle[textclass].style[layout].alignpossible = LYX_ALIGN_BLOCK | LYX_ALIGN_LEFT | LYX_ALIGN_RIGHT | LYX_ALIGN_CENTER;
   lyxstyle[textclass].style[layout].labeltype = LABEL_NO_LABEL;
   
   lyxstyle[textclass].style[layout].baselinestretch = 1.5;
   lyxstyle[textclass].style[layout].labelstring = "";      
   
   /* quote*/ 
   layout++;
   lyxstyle[textclass].style[layout].bordertype = BORDER_STATIC;
   lyxstyle[textclass].style[layout].name = "Quote";
   lyxstyle[textclass].style[layout].latextype = LATEX_ENVIRONEMENT;
   lyxstyle[textclass].style[layout].latexname = "quote";

   lyxstyle[textclass].style[layout].font = GetLyXFont(LYX_ROMAN_FAMILY, 
			      LYX_MEDIUM_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_NORMAL, LYX_NO_LATEX);
   lyxstyle[textclass].style[layout].labelfont = GetLyXFont(LYX_ROMAN_FAMILY, 
			      LYX_MEDIUM_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_NORMAL, LYX_NO_LATEX);

   lyxstyle[textclass].style[layout].nextnoindent = TRUE;
   lyxstyle[textclass].style[layout].leftborder = "MMM";
   lyxstyle[textclass].style[layout].rightborder = "MMM";
   
   lyxstyle[textclass].style[layout].labelsep = "";
   lyxstyle[textclass].style[layout].labelindent = "";
   lyxstyle[textclass].style[layout].parindent = "";
   lyxstyle[textclass].style[layout].parskip = 0.0;
   lyxstyle[textclass].style[layout].itemsep = 0;
   
   lyxstyle[textclass].style[layout].topsep = 0.5;
   lyxstyle[textclass].style[layout].bottomsep = 0.5;
   lyxstyle[textclass].style[layout].parsep = 0;
   
   lyxstyle[textclass].style[layout].align = LYX_ALIGN_BLOCK;
   lyxstyle[textclass].style[layout].alignpossible = LYX_ALIGN_BLOCK | LYX_ALIGN_LEFT | LYX_ALIGN_RIGHT | LYX_ALIGN_CENTER;
   lyxstyle[textclass].style[layout].labeltype = LABEL_NO_LABEL;
   
   lyxstyle[textclass].style[layout].baselinestretch = 1.5;
   lyxstyle[textclass].style[layout].labelstring = "";   
   
   /* list   */
   layout++;
   lyxstyle[textclass].style[layout].bordertype = BORDER_MANUAL;
   lyxstyle[textclass].style[layout].name = "List";
   lyxstyle[textclass].style[layout].latextype = LATEX_LIST_ENVIRONEMENT;
   lyxstyle[textclass].style[layout].latexname = "lyxlist";

   lyxstyle[textclass].style[layout].font = GetLyXFont(LYX_ROMAN_FAMILY,
			      LYX_MEDIUM_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_NORMAL, LYX_NO_LATEX);
   lyxstyle[textclass].style[layout].labelfont = GetLyXFont(LYX_ROMAN_FAMILY, 
			      LYX_MEDIUM_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_NORMAL, LYX_NO_LATEX);

   lyxstyle[textclass].style[layout].nextnoindent = TRUE;
   lyxstyle[textclass].style[layout].leftborder = "";
   lyxstyle[textclass].style[layout].rightborder = "";
   
   lyxstyle[textclass].style[layout].labelsep = "   ";
   lyxstyle[textclass].style[layout].labelindent = "";
   lyxstyle[textclass].style[layout].parindent = "";
   lyxstyle[textclass].style[layout].parskip = 0.4;
   lyxstyle[textclass].style[layout].itemsep = 0;
   
   lyxstyle[textclass].style[layout].topsep = 0.7;
   lyxstyle[textclass].style[layout].bottomsep = 0.7;
   lyxstyle[textclass].style[layout].parsep = 0.5;
   
   lyxstyle[textclass].style[layout].align = LYX_ALIGN_BLOCK;
   lyxstyle[textclass].style[layout].alignpossible = LYX_ALIGN_BLOCK | LYX_ALIGN_LEFT;
   lyxstyle[textclass].style[layout].labeltype = LABEL_MANUAL;
   
   lyxstyle[textclass].style[layout].labelstring = "00.00.0000";
   
   lyxstyle[textclass].style[layout].baselinestretch = 1.5;

   
   
   /* description   */
   layout++;
   lyxstyle[textclass].style[layout].bordertype = BORDER_FIRST_DYNAMIC;
   lyxstyle[textclass].style[layout].name = "Description";
   lyxstyle[textclass].style[layout].latextype = LATEX_ITEM_ENVIRONEMENT;
   lyxstyle[textclass].style[layout].latexname = "description";

   lyxstyle[textclass].style[layout].font = GetLyXFont(LYX_ROMAN_FAMILY, 
			      LYX_MEDIUM_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_NORMAL, LYX_NO_LATEX);
   lyxstyle[textclass].style[layout].labelfont = GetLyXFont(LYX_ROMAN_FAMILY, 
			      LYX_BOLD_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_NORMAL, LYX_NO_LATEX);

   lyxstyle[textclass].style[layout].nextnoindent = TRUE;
   lyxstyle[textclass].style[layout].leftborder = "MM";
   lyxstyle[textclass].style[layout].rightborder = "";
   
   lyxstyle[textclass].style[layout].labelsep = "   ";
   lyxstyle[textclass].style[layout].labelindent = "";
   lyxstyle[textclass].style[layout].parindent = "";
   lyxstyle[textclass].style[layout].parskip = 0.4;
   lyxstyle[textclass].style[layout].itemsep = 0.2;

   lyxstyle[textclass].style[layout].topsep = 0.7;
   lyxstyle[textclass].style[layout].bottomsep = 0.7;
   lyxstyle[textclass].style[layout].parsep = 0.3;
   
   lyxstyle[textclass].style[layout].align = LYX_ALIGN_BLOCK;
   lyxstyle[textclass].style[layout].alignpossible = LYX_ALIGN_BLOCK | LYX_ALIGN_LEFT;
   lyxstyle[textclass].style[layout].labeltype = LABEL_MANUAL;
   
   lyxstyle[textclass].style[layout].labelstring = "MM";
   
   lyxstyle[textclass].style[layout].baselinestretch = 1.5;

   
   /* enumerate  */
   layout++;
   lyxstyle[textclass].style[layout].bordertype = BORDER_STATIC;
   lyxstyle[textclass].style[layout].name = "Enumerate";
   lyxstyle[textclass].style[layout].latextype = LATEX_ITEM_ENVIRONEMENT;
   lyxstyle[textclass].style[layout].latexname = "enumerate";
   
   lyxstyle[textclass].style[layout].font = GetLyXFont(LYX_ROMAN_FAMILY, 
				 LYX_MEDIUM_SERIES,
				 LYX_UP_SHAPE,
				 LYX_SIZE_NORMAL, LYX_NO_LATEX);
   lyxstyle[textclass].style[layout].labelfont = GetLyXFont(LYX_ROMAN_FAMILY, 
				      LYX_MEDIUM_SERIES,
				      LYX_UP_SHAPE,
				      LYX_SIZE_NORMAL, LYX_NO_LATEX);

   lyxstyle[textclass].style[layout].nextnoindent = TRUE;
   lyxstyle[textclass].style[layout].leftborder = "MMN";
   lyxstyle[textclass].style[layout].rightborder = "";
   
   lyxstyle[textclass].style[layout].labelsep = "  ";
   lyxstyle[textclass].style[layout].labelindent = "";
   lyxstyle[textclass].style[layout].parindent = "";
   lyxstyle[textclass].style[layout].parskip = 0.0;
   lyxstyle[textclass].style[layout].itemsep = 0.2;
   
   lyxstyle[textclass].style[layout].topsep = 0.7;
   lyxstyle[textclass].style[layout].bottomsep = 0.7;
   lyxstyle[textclass].style[layout].parsep = 0.3;
   
   lyxstyle[textclass].style[layout].align = LYX_ALIGN_BLOCK;
   lyxstyle[textclass].style[layout].alignpossible = LYX_ALIGN_BLOCK | LYX_ALIGN_LEFT;
   lyxstyle[textclass].style[layout].labeltype = LABEL_COUNTER_ENUMI;
   
   lyxstyle[textclass].style[layout].labelstring = "";
   
   lyxstyle[textclass].style[layout].baselinestretch = 1.5;

   
   /* itemize  */
   layout++;
   lyxstyle[textclass].style[layout].bordertype = BORDER_STATIC;
   lyxstyle[textclass].style[layout].name = "Itemize";
   lyxstyle[textclass].style[layout].latextype = LATEX_ITEM_ENVIRONEMENT;
   lyxstyle[textclass].style[layout].latexname = "itemize";
   
   lyxstyle[textclass].style[layout].font = GetLyXFont(LYX_ROMAN_FAMILY, 
				 LYX_MEDIUM_SERIES,
				 LYX_UP_SHAPE,
				 LYX_SIZE_NORMAL, LYX_NO_LATEX);
   lyxstyle[textclass].style[layout].labelfont = GetLyXFont(LYX_ROMAN_FAMILY, 
				      LYX_MEDIUM_SERIES,
				      LYX_UP_SHAPE,
				      LYX_SIZE_NORMAL, LYX_NO_LATEX);

   lyxstyle[textclass].style[layout].nextnoindent = TRUE;
   lyxstyle[textclass].style[layout].leftborder = "MMN";
   lyxstyle[textclass].style[layout].rightborder = "";
   
   lyxstyle[textclass].style[layout].labelsep = "  ";
   lyxstyle[textclass].style[layout].labelindent = "";
   lyxstyle[textclass].style[layout].parindent = "";
   lyxstyle[textclass].style[layout].parskip = 0;
   lyxstyle[textclass].style[layout].itemsep = 0.2;
   
   lyxstyle[textclass].style[layout].topsep = 0.7;
   lyxstyle[textclass].style[layout].bottomsep = 0.7;
   lyxstyle[textclass].style[layout].parsep = 0.3;
   
   lyxstyle[textclass].style[layout].align = LYX_ALIGN_BLOCK;
   lyxstyle[textclass].style[layout].alignpossible = LYX_ALIGN_BLOCK | LYX_ALIGN_LEFT;
   lyxstyle[textclass].style[layout].labeltype = LABEL_STATIC;
   
   lyxstyle[textclass].style[layout].labelstring = "*";
   
   lyxstyle[textclass].style[layout].baselinestretch = 1.5;


   /* verse  */
   layout++;
   lyxstyle[textclass].style[layout].bordertype = BORDER_FIRST_DYNAMIC;
   /* first dynamic because of the LaTeX implemenation.
    * In LyX this does not make sense. */
   lyxstyle[textclass].style[layout].name = "Verse";
   lyxstyle[textclass].style[layout].latextype = LATEX_ENVIRONEMENT;
   lyxstyle[textclass].style[layout].latexname = "verse";
   
   lyxstyle[textclass].style[layout].font = GetLyXFont(LYX_ROMAN_FAMILY, 
						       LYX_MEDIUM_SERIES,
						       LYX_UP_SHAPE,
						       LYX_SIZE_NORMAL, LYX_NO_LATEX);
   lyxstyle[textclass].style[layout].labelfont = GetLyXFont(LYX_ROMAN_FAMILY, 
							    LYX_MEDIUM_SERIES,
							    LYX_UP_SHAPE,
							    LYX_SIZE_NORMAL, LYX_NO_LATEX);
   
   lyxstyle[textclass].style[layout].nextnoindent = TRUE;
   lyxstyle[textclass].style[layout].leftborder = "MMMM";
   lyxstyle[textclass].style[layout].rightborder = "";
   
   lyxstyle[textclass].style[layout].labelsep = "";
   lyxstyle[textclass].style[layout].labelindent = "MMMM";
   lyxstyle[textclass].style[layout].parindent = "";
   lyxstyle[textclass].style[layout].parskip = 0.4;
   lyxstyle[textclass].style[layout].itemsep = 0;

   lyxstyle[textclass].style[layout].topsep = 0.7;
   lyxstyle[textclass].style[layout].bottomsep = 0.7;
   lyxstyle[textclass].style[layout].parsep = 0.5;
   
   lyxstyle[textclass].style[layout].align = LYX_ALIGN_BLOCK;
   lyxstyle[textclass].style[layout].alignpossible = LYX_ALIGN_BLOCK | LYX_ALIGN_LEFT;
   lyxstyle[textclass].style[layout].labeltype = LABEL_NO_LABEL;
   
   lyxstyle[textclass].style[layout].labelstring = "";
   
   lyxstyle[textclass].style[layout].baselinestretch = 1.5;

   
   /* right address */
   layout++;
   lyxstyle[textclass].style[layout].bordertype = BORDER_RIGHT_ADDRESS_BOX;
   lyxstyle[textclass].style[layout].name = "Right Address";
   lyxstyle[textclass].style[layout].latextype = LATEX_COMMAND;
   lyxstyle[textclass].style[layout].latexname = "lyxrightaddress";
   
   lyxstyle[textclass].style[layout].font = GetLyXFont(LYX_ROMAN_FAMILY, 
				 LYX_MEDIUM_SERIES,
				 LYX_UP_SHAPE,
				 LYX_SIZE_NORMAL, LYX_NO_LATEX);
   lyxstyle[textclass].style[layout].labelfont = GetLyXFont(LYX_ROMAN_FAMILY, 
				      LYX_MEDIUM_SERIES,
				      LYX_UP_SHAPE,
				      LYX_SIZE_NORMAL, LYX_NO_LATEX);

   lyxstyle[textclass].style[layout].leftborder = "";
   lyxstyle[textclass].style[layout].rightborder = "";
   
   lyxstyle[textclass].style[layout].labelsep = "";
   lyxstyle[textclass].style[layout].labelindent = "";
   lyxstyle[textclass].style[layout].parindent = "";
   lyxstyle[textclass].style[layout].parskip = 0.4;
   lyxstyle[textclass].style[layout].itemsep = 0;
   
   lyxstyle[textclass].style[layout].topsep = 0;
   lyxstyle[textclass].style[layout].bottomsep = 1.5;
   lyxstyle[textclass].style[layout].parsep = 1.5;
   
   lyxstyle[textclass].style[layout].align = LYX_ALIGN_LEFT;
   lyxstyle[textclass].style[layout].alignpossible = LYX_ALIGN_LEFT;
   lyxstyle[textclass].style[layout].labeltype = LABEL_NO_LABEL;
   
   lyxstyle[textclass].style[layout].labelstring = "";
   
   lyxstyle[textclass].style[layout].baselinestretch = 1.5;
   
   /* address */
   layout++;
   lyxstyle[textclass].style[layout].bordertype = BORDER_STATIC;
   lyxstyle[textclass].style[layout].name = "Address";
   lyxstyle[textclass].style[layout].latextype = LATEX_COMMAND;
   lyxstyle[textclass].style[layout].latexname = "lyxaddress";
   
   lyxstyle[textclass].style[layout].font = GetLyXFont(LYX_ROMAN_FAMILY, 
				 LYX_MEDIUM_SERIES,
				 LYX_UP_SHAPE,
				 LYX_SIZE_NORMAL, LYX_NO_LATEX);
   lyxstyle[textclass].style[layout].labelfont = GetLyXFont(LYX_ROMAN_FAMILY, 
				      LYX_MEDIUM_SERIES,
				      LYX_UP_SHAPE,
				      LYX_SIZE_NORMAL, LYX_NO_LATEX);

   lyxstyle[textclass].style[layout].leftborder = "";
   lyxstyle[textclass].style[layout].rightborder = "";
   
   lyxstyle[textclass].style[layout].labelsep = "";
   lyxstyle[textclass].style[layout].labelindent = "";
   lyxstyle[textclass].style[layout].parindent = "";
   lyxstyle[textclass].style[layout].parskip = 0.4;
   lyxstyle[textclass].style[layout].itemsep = 0;
   
   lyxstyle[textclass].style[layout].topsep = 0;
   lyxstyle[textclass].style[layout].bottomsep = 1.5;
   lyxstyle[textclass].style[layout].parsep = 1.5;
   
   lyxstyle[textclass].style[layout].align = LYX_ALIGN_LEFT;
   lyxstyle[textclass].style[layout].alignpossible = LYX_ALIGN_BLOCK | LYX_ALIGN_LEFT | LYX_ALIGN_RIGHT | LYX_ALIGN_CENTER;
   lyxstyle[textclass].style[layout].labeltype = LABEL_NO_LABEL;
   
   lyxstyle[textclass].style[layout].labelstring = "";
   
   lyxstyle[textclass].style[layout].baselinestretch = 1.5;


   /* formula */ 
   layout++;
   lyxstyle[textclass].style[layout].bordertype = BORDER_STATIC;
   lyxstyle[textclass].style[layout].name = "Formula";
   lyxstyle[textclass].style[layout].latextype = LATEX_COMMAND;
   lyxstyle[textclass].style[layout].latexname = "lyxformula";
   
   lyxstyle[textclass].style[layout].font = GetLyXFont(LYX_ROMAN_FAMILY, 
				 LYX_MEDIUM_SERIES,
				 LYX_UP_SHAPE,
				 LYX_SIZE_NORMAL, LYX_FORMULA_LATEX);
   lyxstyle[textclass].style[layout].labelfont = GetLyXFont(LYX_ROMAN_FAMILY, 
				      LYX_MEDIUM_SERIES,
				      LYX_UP_SHAPE,
				      LYX_SIZE_NORMAL, LYX_FORMULA_LATEX);

   lyxstyle[textclass].style[layout].leftborder = "";
   lyxstyle[textclass].style[layout].rightborder = "";
   
   lyxstyle[textclass].style[layout].labelsep = "";
   lyxstyle[textclass].style[layout].labelindent = "";
   lyxstyle[textclass].style[layout].parindent = "";
   lyxstyle[textclass].style[layout].parskip = 0.4;
   lyxstyle[textclass].style[layout].itemsep = 0.3;

   lyxstyle[textclass].style[layout].topsep = 0.7;
   lyxstyle[textclass].style[layout].bottomsep = 0.7;
   lyxstyle[textclass].style[layout].parsep = 0.7;
   
   lyxstyle[textclass].style[layout].align = LYX_ALIGN_FORMULA;
   lyxstyle[textclass].style[layout].alignpossible = LYX_ALIGN_FORMULA;
   lyxstyle[textclass].style[layout].labeltype = LABEL_NO_LABEL;
   
   lyxstyle[textclass].style[layout].labelstring = "";
   
   lyxstyle[textclass].style[layout].baselinestretch = 1.5;



    /* title */
   layout++;
   lyxstyle[textclass].style[layout].bordertype = BORDER_STATIC;
   lyxstyle[textclass].style[layout].name = "Title";
   lyxstyle[textclass].style[layout].latextype = LATEX_COMMAND;
   lyxstyle[textclass].style[layout].latexname = "lyxtitle";
   
   lyxstyle[textclass].style[layout].font = GetLyXFont(LYX_SANS_FAMILY, 
				 LYX_BOLD_SERIES,
				 LYX_UP_SHAPE,
				 LYX_SIZE_LARGEST, LYX_NO_LATEX);
   lyxstyle[textclass].style[layout].labelfont = GetLyXFont(LYX_SANS_FAMILY, 
				 LYX_BOLD_SERIES,
				 LYX_UP_SHAPE,
				 LYX_SIZE_LARGEST, LYX_NO_LATEX);

   lyxstyle[textclass].style[layout].leftborder = "";
   lyxstyle[textclass].style[layout].rightborder = "";
   
   lyxstyle[textclass].style[layout].nextnoindent = TRUE;
   lyxstyle[textclass].style[layout].labelsep = "";
   lyxstyle[textclass].style[layout].labelindent = "";
   lyxstyle[textclass].style[layout].parindent = "";
   lyxstyle[textclass].style[layout].parskip = 0.4;
   lyxstyle[textclass].style[layout].itemsep = 0;
   
   lyxstyle[textclass].style[layout].topsep = 0;
   lyxstyle[textclass].style[layout].bottomsep = 0;
   lyxstyle[textclass].style[layout].parsep = 0;
   lyxstyle[textclass].style[layout].align = LYX_ALIGN_CENTER;
   lyxstyle[textclass].style[layout].alignpossible = LYX_ALIGN_BLOCK | LYX_ALIGN_LEFT | LYX_ALIGN_RIGHT | LYX_ALIGN_CENTER;
   
   lyxstyle[textclass].style[layout].labeltype = LABEL_NO_LABEL;
   lyxstyle[textclass].style[layout].labelstring = "";

   lyxstyle[textclass].style[layout].baselinestretch = 1.5;
   
   lyxstyle[textclass].style[layout].fill_top = 1;
   lyxstyle[textclass].style[layout].fill_bottom = 1;

}

static void SetCaption(char &textclass, char &layout) {
   /* caption */
   layout++;
   lyxstyle[textclass].style[layout].bordertype = BORDER_FIRST_DYNAMIC;
   lyxstyle[textclass].style[layout].name = "Caption";
   lyxstyle[textclass].style[layout].latextype = LATEX_COMMAND;
   lyxstyle[textclass].style[layout].latexname = "caption";

   lyxstyle[textclass].style[layout].font = GetLyXFont(LYX_ROMAN_FAMILY, 
				    LYX_MEDIUM_SERIES,
				    LYX_UP_SHAPE,
				    LYX_SIZE_NORMAL, LYX_NO_LATEX);
   lyxstyle[textclass].style[layout].labelfont = GetLyXFont(LYX_ROMAN_FAMILY, 
					 LYX_MEDIUM_SERIES,
					 LYX_UP_SHAPE,
					 LYX_SIZE_NORMAL, LYX_NO_LATEX);

   lyxstyle[textclass].style[layout].leftborder = "";
   lyxstyle[textclass].style[layout].rightborder = "";
   
   lyxstyle[textclass].style[layout].labelsep = "  ";
   lyxstyle[textclass].style[layout].labelindent = "";
   lyxstyle[textclass].style[layout].parindent = "";
   lyxstyle[textclass].style[layout].parskip = 0.4;
   lyxstyle[textclass].style[layout].itemsep = 0;
   
   lyxstyle[textclass].style[layout].topsep = 0.5;
   lyxstyle[textclass].style[layout].bottomsep = 0;
   lyxstyle[textclass].style[layout].parsep = 0;
   
   lyxstyle[textclass].style[layout].align = LYX_ALIGN_BLOCK;
   lyxstyle[textclass].style[layout].alignpossible = LYX_ALIGN_BLOCK | LYX_ALIGN_LEFT;
   lyxstyle[textclass].style[layout].labeltype = LABEL_SENSITIVE;
   lyxstyle[textclass].style[layout].labelstring = "Caption:";
   
   lyxstyle[textclass].style[layout].baselinestretch = 1.5;

}
static void SetSectionsStar(char &textclass, char &layout) {
   /* section*  */ 
   layout++;
   lyxstyle[textclass].style[layout].bordertype = BORDER_STATIC;
   lyxstyle[textclass].style[layout].name = "Section*";
   lyxstyle[textclass].style[layout].latextype = LATEX_COMMAND;
   lyxstyle[textclass].style[layout].latexname = "section*";

   lyxstyle[textclass].style[layout].font = GetLyXFont(LYX_ROMAN_FAMILY, 
			      LYX_BOLD_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_LARGER, LYX_NO_LATEX);
   lyxstyle[textclass].style[layout].labelfont = GetLyXFont(LYX_ROMAN_FAMILY, 
			      LYX_BOLD_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_LARGER, LYX_NO_LATEX);
   
   lyxstyle[textclass].style[layout].nextnoindent = TRUE;
   lyxstyle[textclass].style[layout].leftborder = "";
   lyxstyle[textclass].style[layout].rightborder = "";
   
   lyxstyle[textclass].style[layout].labelsep = "   ";
   lyxstyle[textclass].style[layout].labelindent = "";
   lyxstyle[textclass].style[layout].parindent = "";
   lyxstyle[textclass].style[layout].parskip = 0.4;
   lyxstyle[textclass].style[layout].itemsep = 0;
   
   lyxstyle[textclass].style[layout].topsep = 1.3;
   lyxstyle[textclass].style[layout].bottomsep = 0.7;
   lyxstyle[textclass].style[layout].parsep = 0.7;
   
   lyxstyle[textclass].style[layout].align = LYX_ALIGN_BLOCK;
   lyxstyle[textclass].style[layout].alignpossible = LYX_ALIGN_BLOCK | LYX_ALIGN_LEFT|LYX_ALIGN_RIGHT|LYX_ALIGN_CENTER;
   lyxstyle[textclass].style[layout].labeltype = LABEL_NO_LABEL;
   
   lyxstyle[textclass].style[layout].baselinestretch = 1.5;
   lyxstyle[textclass].style[layout].labelstring = "";      
   
    /* subsection* */
   layout++;
   lyxstyle[textclass].style[layout].bordertype = BORDER_STATIC;
   lyxstyle[textclass].style[layout].name = "Subsection*";
   lyxstyle[textclass].style[layout].latextype = LATEX_COMMAND;
   lyxstyle[textclass].style[layout].latexname = "subsection*";

   lyxstyle[textclass].style[layout].font = GetLyXFont(LYX_ROMAN_FAMILY, 
			      LYX_BOLD_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_LARGE, LYX_NO_LATEX);
   lyxstyle[textclass].style[layout].labelfont = GetLyXFont(LYX_ROMAN_FAMILY, 
			      LYX_BOLD_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_LARGE, LYX_NO_LATEX);

   lyxstyle[textclass].style[layout].nextnoindent = TRUE;
   lyxstyle[textclass].style[layout].leftborder = "";
   lyxstyle[textclass].style[layout].rightborder = "";
   
   lyxstyle[textclass].style[layout].labelsep = "   ";
   lyxstyle[textclass].style[layout].labelindent = "";
   lyxstyle[textclass].style[layout].parindent = "";
   lyxstyle[textclass].style[layout].parskip = 0.4;
   lyxstyle[textclass].style[layout].itemsep = 0;
   
   lyxstyle[textclass].style[layout].topsep = 1;
   lyxstyle[textclass].style[layout].bottomsep = 0.5;
   lyxstyle[textclass].style[layout].parsep = 0.5;
   
   lyxstyle[textclass].style[layout].align = LYX_ALIGN_BLOCK;
   lyxstyle[textclass].style[layout].alignpossible = LYX_ALIGN_BLOCK | LYX_ALIGN_LEFT|LYX_ALIGN_RIGHT|LYX_ALIGN_CENTER;
   lyxstyle[textclass].style[layout].labeltype = LABEL_NO_LABEL;
   
   lyxstyle[textclass].style[layout].baselinestretch = 1.5;
   lyxstyle[textclass].style[layout].labelstring = "";   

   /* subsubsection* */
   layout++;
   lyxstyle[textclass].style[layout].bordertype = BORDER_STATIC;
   lyxstyle[textclass].style[layout].name = "Subsubsection*";
   lyxstyle[textclass].style[layout].latextype = LATEX_COMMAND;
   lyxstyle[textclass].style[layout].latexname = "subsubsection*";

   lyxstyle[textclass].style[layout].font = GetLyXFont(LYX_ROMAN_FAMILY, 
			      LYX_BOLD_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_NORMAL, LYX_NO_LATEX);
   lyxstyle[textclass].style[layout].labelfont = GetLyXFont(LYX_ROMAN_FAMILY, 
			      LYX_BOLD_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_NORMAL, LYX_NO_LATEX);

   lyxstyle[textclass].style[layout].nextnoindent = TRUE;
   lyxstyle[textclass].style[layout].leftborder = "";
   lyxstyle[textclass].style[layout].rightborder = "";
   
   lyxstyle[textclass].style[layout].labelsep = "   ";
   lyxstyle[textclass].style[layout].labelindent = "";
   lyxstyle[textclass].style[layout].parindent = "";
   lyxstyle[textclass].style[layout].parskip = 0.4;
   lyxstyle[textclass].style[layout].itemsep = 0;
   
   lyxstyle[textclass].style[layout].topsep = 0.7;
   lyxstyle[textclass].style[layout].bottomsep = 0.4;
   lyxstyle[textclass].style[layout].parsep = 0.4;
   
   lyxstyle[textclass].style[layout].align = LYX_ALIGN_BLOCK;
   lyxstyle[textclass].style[layout].alignpossible = LYX_ALIGN_BLOCK | LYX_ALIGN_LEFT|LYX_ALIGN_RIGHT|LYX_ALIGN_CENTER;
   lyxstyle[textclass].style[layout].labeltype = LABEL_NO_LABEL;
   
   lyxstyle[textclass].style[layout].baselinestretch = 1.5;
   lyxstyle[textclass].style[layout].labelstring = "";   
  
}
static void SetLasts(char &textclass, char &layout) {
   /* LaTeX */ 
   layout++;
   lyxstyle[textclass].style[layout].bordertype = BORDER_STATIC;
   lyxstyle[textclass].style[layout].name = "LaTeX";
   lyxstyle[textclass].style[layout].newline_allowed = 0;
   
   lyxstyle[textclass].style[layout].latextype = LATEX_PARAGRAPH;
   lyxstyle[textclass].style[layout].latexname = "dummy";
   
   
   lyxstyle[textclass].style[layout].font = GetLyXFont(LYX_ROMAN_FAMILY, 
				 LYX_MEDIUM_SERIES,
				 LYX_UP_SHAPE,
				 LYX_SIZE_NORMAL,
				 LYX_NO_LATEX);
   lyxstyle[textclass].style[layout].labelfont = GetLyXFont(LYX_ROMAN_FAMILY,
			      LYX_MEDIUM_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_NORMAL, LYX_NO_LATEX);
   
   lyxstyle[textclass].style[layout].leftborder = "";
   lyxstyle[textclass].style[layout].rightborder = "";
   
   lyxstyle[textclass].style[layout].labelsep = "";
   lyxstyle[textclass].style[layout].parindent = "MM";
   lyxstyle[textclass].style[layout].parskip = 0.4;
   lyxstyle[textclass].style[layout].itemsep = 0;
   
   lyxstyle[textclass].style[layout].topsep = 0;
   lyxstyle[textclass].style[layout].bottomsep = 0;
   lyxstyle[textclass].style[layout].parsep = 0.5;
   
   lyxstyle[textclass].style[layout].align = LYX_ALIGN_BLOCK;
   lyxstyle[textclass].style[layout].alignpossible = LYX_ALIGN_BLOCK | LYX_ALIGN_LEFT | LYX_ALIGN_RIGHT | LYX_ALIGN_CENTER;
   lyxstyle[textclass].style[layout].labeltype = LABEL_NO_LABEL;
   
   lyxstyle[textclass].style[layout].baselinestretch = 1.5;

   lyxstyle[textclass].style[layout].labelstring = "";   
   
   /* lyxcode*/ 
   layout++;
   lyxstyle[textclass].style[layout].bordertype = BORDER_STATIC;
   lyxstyle[textclass].style[layout].name = "LyX-Code";
   lyxstyle[textclass].style[layout].latextype = LATEX_ENVIRONEMENT;
   lyxstyle[textclass].style[layout].latexname = "lyxcode";

   lyxstyle[textclass].style[layout].font = GetLyXFont(LYX_TYPEWRITER_FAMILY, 
			      LYX_MEDIUM_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_NORMAL, LYX_NO_LATEX);
   lyxstyle[textclass].style[layout].labelfont = GetLyXFont(LYX_TYPEWRITER_FAMILY, 
			      LYX_MEDIUM_SERIES,
			      LYX_UP_SHAPE,
			      LYX_SIZE_NORMAL, LYX_NO_LATEX);

   lyxstyle[textclass].style[layout].nextnoindent = TRUE;
   lyxstyle[textclass].style[layout].leftborder = "MMM";
   lyxstyle[textclass].style[layout].rightborder = "MMM";
   
   lyxstyle[textclass].style[layout].labelsep = "";
   lyxstyle[textclass].style[layout].labelindent = "";
   lyxstyle[textclass].style[layout].parindent = "";
   lyxstyle[textclass].style[layout].parskip = 0.0;
   lyxstyle[textclass].style[layout].itemsep = 0;
   
   lyxstyle[textclass].style[layout].topsep = 0.5;
   lyxstyle[textclass].style[layout].bottomsep = 0.5;
   lyxstyle[textclass].style[layout].parsep = 0;
   
   lyxstyle[textclass].style[layout].align = LYX_ALIGN_LEFT;
   lyxstyle[textclass].style[layout].alignpossible = LYX_ALIGN_BLOCK | LYX_ALIGN_LEFT | LYX_ALIGN_RIGHT | LYX_ALIGN_CENTER;
   lyxstyle[textclass].style[layout].labeltype = LABEL_NO_LABEL;
   
   lyxstyle[textclass].style[layout].baselinestretch = 1.5;
   lyxstyle[textclass].style[layout].labelstring = "";   

}
static void SetChapter(char &textclass, char &layout) {
   /* Chapter*/
   layout++;
   lyxstyle[textclass].style[layout].bordertype = BORDER_STATIC;
   lyxstyle[textclass].style[layout].name = "Chapter";
   lyxstyle[textclass].style[layout].latextype = LATEX_COMMAND;
   lyxstyle[textclass].style[layout].latexname = "chapter";

   lyxstyle[textclass].style[layout].font = GetLyXFont(LYX_ROMAN_FAMILY, 
				 LYX_BOLD_SERIES,
				 LYX_UP_SHAPE,
				 LYX_SIZE_HUGE, LYX_NO_LATEX);
   lyxstyle[textclass].style[layout].labelfont = GetLyXFont(LYX_ROMAN_FAMILY, 
				      LYX_BOLD_SERIES,
				      LYX_UP_SHAPE,
				      LYX_SIZE_HUGE, LYX_NO_LATEX);
   
   lyxstyle[textclass].style[layout].nextnoindent = TRUE;
   lyxstyle[textclass].style[layout].leftborder = "";
   lyxstyle[textclass].style[layout].rightborder = "";
   
   lyxstyle[textclass].style[layout].labelsep = "";
   lyxstyle[textclass].style[layout].labelindent = "";
   lyxstyle[textclass].style[layout].parindent = "";
   lyxstyle[textclass].style[layout].parskip = 0.4;
   lyxstyle[textclass].style[layout].itemsep = 0;
   
   lyxstyle[textclass].style[layout].topsep = 4;
   lyxstyle[textclass].style[layout].bottomsep = 1.4;
   lyxstyle[textclass].style[layout].parsep = 4;
   
   lyxstyle[textclass].style[layout].align = LYX_ALIGN_BLOCK;
   lyxstyle[textclass].style[layout].alignpossible = LYX_ALIGN_BLOCK | LYX_ALIGN_LEFT;
   lyxstyle[textclass].style[layout].labeltype = LABEL_COUNTER_CHAPTER;
   
   lyxstyle[textclass].style[layout].baselinestretch = 1.5;
   lyxstyle[textclass].style[layout].labelstring = "";   
}

void LyXSetStyle() {
   
   char textclass = 0;
   char layout = 0;
   

   /* first the class article */ 
   
   lyxstyle[textclass].name = "article";
   InitLayouts(textclass);
   
   /* set the class defaults */ 
   lyxstyle[textclass].options = "";
   lyxstyle[textclass].columns = 1;
   lyxstyle[textclass].sides = 1;
   lyxstyle[textclass].pagestyle = PAGESTYLE_PLAIN;
   lyxstyle[textclass].maxcounter = LABEL_COUNTER_SECTION;

   lyxstyle[textclass].top_label_layout = "Section";
   
   layout = 0;
   SetStandard(textclass, layout);
   SetSections(textclass, layout);
   SetCommon(textclass, layout);
   SetSectionBibliography(textclass, layout);
   SetCaption(textclass, layout);
   SetSectionsStar(textclass, layout);
   SetLasts(textclass, layout);
   layout++;
   lyxstyle[textclass].number_of_defined_layouts = layout;
   textclass++;

  /* then the class report  */ 
   
   lyxstyle[textclass].name = "report";
   InitLayouts(textclass);

   /* set the class defaults */ 
   lyxstyle[textclass].options = "";
   lyxstyle[textclass].columns = 1;
   lyxstyle[textclass].sides = 1;
   lyxstyle[textclass].pagestyle = PAGESTYLE_PLAIN;
   lyxstyle[textclass].maxcounter = LABEL_COUNTER_CHAPTER;

   lyxstyle[textclass].top_label_layout = "Chapter";
   
   layout = 0;
   SetStandard(textclass, layout);
   SetChapter(textclass, layout);
   SetSections(textclass, layout);
   SetCommon(textclass, layout);
   SetChapterBibliography(textclass, layout);
   SetCaption(textclass, layout);
   SetSectionsStar(textclass, layout);
   SetLasts(textclass, layout);
   layout++;
   lyxstyle[textclass].number_of_defined_layouts = layout;
   textclass++;

 
  /* then the class book */ 
   
   lyxstyle[textclass].name = "book";
   InitLayouts(textclass);

   /* set the class defaults */ 
   lyxstyle[textclass].options = "";
   lyxstyle[textclass].columns = 1;
   lyxstyle[textclass].sides = 2;
   lyxstyle[textclass].pagestyle = PAGESTYLE_HEADINGS;
   lyxstyle[textclass].maxcounter = LABEL_COUNTER_CHAPTER;
   
   lyxstyle[textclass].top_label_layout = "Chapter";

   layout = 0;
   SetStandard(textclass, layout);
   SetChapter(textclass, layout);
   SetSections(textclass, layout);
   SetCommon(textclass, layout);
   SetChapterBibliography(textclass, layout);
   SetCaption(textclass, layout);
   SetSectionsStar(textclass, layout);
   SetLasts(textclass, layout);
   layout++;
   lyxstyle[textclass].number_of_defined_layouts = layout;
   textclass++;
   number_of_defined_classes = textclass;
}
