/* This file is part of
* ======================================================
* 
*           LyX, the High Level Word Processor
* 	 
*	    Copyright (C) 1995 Matthias Ettrich
*
*======================================================*/
#include "file.h"
#include "lyx.h"
#include "spellchecker.h"
#include <ctype.h>
#include <unistd.h>
#include <signal.h>
#define CONTROL_X_MODE 1

extern FD_form_main *fd_form_main;
extern FD_form_title *fd_form_title;
extern FD_form_paragraph *fd_form_paragraph;
extern FD_form_search *fd_form_search;
extern FD_form_character *fd_form_character;
extern FD_form_document *fd_form_document;
extern FD_form_quotes *fd_form_quotes;
extern FD_form_preamble *fd_form_preamble;
extern FD_form_table *fd_form_table;
extern FD_form_print *fd_form_print;
extern FD_form_figure *fd_form_figure;
extern FD_form_screen *fd_form_screen;
extern FD_form_toc *fd_form_toc;

extern char* roman_font_name;
extern char* sans_font_name;
extern char* typewriter_font_name;


static int work_area_height = 0;
static int work_area_width = 0;

char delete_mapped_to_backspace = 0;
static long current_scrollbar_value = 0;
static int current_layout = 0;
static char current_textclass = 0;
static char keyboard_mode = 0;


LyXRC lyxrc;

char *document_path = NULL;


Buffer *buffer[NUMBER_OF_BUFFERS];
int current_buffer = 0;
int old_current_buffer = 0;
void InsertCorrectQuote();

char work_area_active = False;	       /* boolean flag  */
char scrolling = False;	       /* boolean flag  */

char updatetimer = 0;
char input_prohibited = False;   /* wether the work area should get callbacks */ 

char selection_possible = False;		       /* just a boolean flag */
/* the selection possible is needed, that only motion events are 
* used, where the bottom press event was on the drawing area too */


/* the cursor */
Cursor xterm_cursor;

/* some function prototypes */ 
int WorkAreaButtonPress(FL_OBJECT *ob, Window win, int w, int h, XEvent *ev, void *d);
int WorkAreaButtonRelease(FL_OBJECT *ob, Window win, int w, int h, XEvent *ev, void *d);
int WorkAreaKeyPress(FL_OBJECT *ob, Window win, int w, int h, XEvent *ev, void *d);
int WorkAreaMotionNotify(FL_OBJECT *ob, Window win, int w, int h, XEvent *ev, void *d);
void ResizeCurrentBufferPseudoExpose();
void InitTextParameters(LyXTextParameters &p);
void GotoError();
void OpenStuff();
void HyphenationPoint();
void HFill();
void Newline();
void ProtectedBlank();
void CopyCB();
void LyXBell() {
   XBell(fl_display, 20);
}
int RunLaTeX();
void MenuWrite(int buf);
void CloseBuffer(); 
void MenuPreview();
void MenuPreviewPS();
void MenuLayoutCharacter();
void MenuLayoutParagraph();
void MenuLayoutDocument();
void MenuLayoutQuotes();
void MenuLayoutPreamble();

void error_handler(int err_sig);

static char minibuffer_text[200] = "Welcome to LyX!";

int MinibufferExpose(FL_OBJECT *ob, Window win, int w, int h, XEvent *ev, void *d) {
   static int width = 0;
   static int height = 0;
   static FL_OBJECT *ob2 = NULL;
   static Window win2 = 0;
   static Font fixed_font =  XLoadQueryFont(fl_display, "fixed")->fid;
   static GC gc = LyXGetCopyGC();
   if (w && h) {
      width = w;
      height = h;
      ob2 = ob;
      win2 = win;
   }
   if (!win2)
      return 0;
   
   XFillRectangle(fl_display, win2, LyXGetLightedGC(), 0, 0, width, height);
   XFlush(fl_display);
   
   int i=0;
   for (i=0;minibuffer_text[i]; i++);
   LyXFont font = GetLyXFont(LYX_SANS_FAMILY, 
			     LYX_MEDIUM_SERIES,
			     LYX_UP_SHAPE,
			     LYX_SIZE_SMALL,
			     LYX_NO_LATEX);
   XSetFont(fl_display, gc, fixed_font);
   
   XDrawString(fl_display,
	       win2,
	       gc,
	       5,
	       2* height/3,
	       minibuffer_text, i);
   XFlush(fl_display);
   return 0;
}

void SetMinibuffer(const char* s1, const char* s2, const char* s3) {
   sprintf(minibuffer_text,"%s %s %s", s1, s2, s3);
   MinibufferExpose(NULL, 0, 0, 0, NULL, NULL);
   fl_set_timer(fd_form_main->timer_minibuffer, 4);
}

void InitMinibuffer() {
   char* s="";
   keyboard_mode = 0;
   if (buffer[current_buffer]->text){
      if (!buffer[current_buffer]->lyx_current)
      	 s = "(unsaved changes)";
      sprintf(minibuffer_text,"current buffer: %s %s, Textclass: %s", 
	      buffer[current_buffer]->filename, 
	      s, 
	      NameOfClass(buffer[current_buffer]->parameters.textclass));
   }
   else
      sprintf(minibuffer_text,"* no buffer * ");
   MinibufferExpose(NULL, 0, 0, 0, NULL, NULL);
}

void MinibufferTimerCB(FL_OBJECT *ob, long data) {
   InitMinibuffer();
}


void UpdateLayoutChoice() {
   /* update the layout display */ 
   if (!buffer[current_buffer]->text)
      return;
   int i;
   
   if (current_textclass != buffer[current_buffer]->parameters.textclass) {
      fl_clear_choice(fd_form_main->choice_layouts);
      for (i = 0; !StringEqual(NameOfLayout(buffer[current_buffer]->parameters.textclass, i),"dummy"); i++) {
	 fl_addto_choice(fd_form_main->choice_layouts, NameOfLayout(buffer[current_buffer]->parameters.textclass,i));
      }
      current_textclass = buffer[current_buffer]->parameters.textclass;
      current_layout = 0;
   }
   
   char layout = buffer[current_buffer]->text->cursor.par->GetLayout();
   
   if (layout !=  current_layout){
      fl_set_choice(fd_form_main->choice_layouts, layout + 1);
      current_layout = layout;
      
   }
}

void ResetKeyboardMode() {
   if (keyboard_mode) {
      InitMinibuffer();
   }
}


void ProhibitInput() {
   input_prohibited = True;
   if (buffer[current_buffer]->screen)
     buffer[current_buffer]->screen->HideCursor();

   static Cursor cursor;
   static char cursor_undefined = 1;
   
   if (cursor_undefined)
     cursor = XCreateFontCursor(fl_display, XC_watch);
   
   
   /* set the cursor to the watch for for all forms and the canvas */ 
   XDefineCursor(fl_display, fd_form_main->form_main->window, cursor);
   if (fd_form_paragraph->form_paragraph->visible)
     XDefineCursor(fl_display, fd_form_paragraph->form_paragraph->window, cursor);
   XDefineCursor(fl_display, fl_get_canvas_id(fd_form_main->work_area), cursor); 
   XDefineCursor(fl_display, fl_get_canvas_id(fd_form_main->minibuffer), cursor);


   XFlush(fl_display);
   
   /* remove the canvxas handler */ 
   fl_remove_canvas_handler(fd_form_main->work_area, KeyPress, WorkAreaKeyPress);
   fl_remove_canvas_handler(fd_form_main->work_area, ButtonPress, WorkAreaButtonPress);
   fl_remove_canvas_handler(fd_form_main->work_area, ButtonRelease, WorkAreaButtonRelease);
   fl_remove_canvas_handler(fd_form_main->work_area, MotionNotify, WorkAreaMotionNotify);
   
   if (buffer[current_buffer]->screen)
     buffer[current_buffer]->screen->HideCursor();
   
}

void AllowInput() {
   input_prohibited = False;
   if (buffer[current_buffer]->screen && work_area_active)
     buffer[current_buffer]->screen->ShowCursor();

   /* reset the cursor from the watch for all forms and the canvas */
   
   XUndefineCursor(fl_display, fd_form_main->form_main->window);
   if (fd_form_paragraph->form_paragraph->visible)
     XUndefineCursor(fl_display, fd_form_paragraph->form_paragraph->window);
   XUndefineCursor(fl_display, fl_get_canvas_id(fd_form_main->work_area));
   XDefineCursor(fl_display, fl_get_canvas_id(fd_form_main->work_area), 
		 xterm_cursor);
   XDefineCursor(fl_display, fl_get_canvas_id(fd_form_main->minibuffer),
		 xterm_cursor);

   
   
   /* re-install the canvas handler */ 
   fl_add_canvas_handler(fd_form_main->work_area, KeyPress, WorkAreaKeyPress, fd_form_main);
   fl_add_canvas_handler(fd_form_main->work_area, ButtonPress, WorkAreaButtonPress, fd_form_main);
   fl_add_canvas_handler(fd_form_main->work_area, ButtonRelease, WorkAreaButtonRelease, fd_form_main);
   fl_add_canvas_handler(fd_form_main->work_area, MotionNotify, WorkAreaMotionNotify, fd_form_main);

   if (buffer[current_buffer]->screen && work_area_active)
     buffer[current_buffer]->screen->ShowCursor();

}



void WriteAlert(const char* s1, const char* s2, const char* s3){
   ProhibitInput();
   SetMinibuffer(s1, s2, s3);
   fl_show_alert(s1, s2,
		s3, 0);
   AllowInput();
}


void FreeUpdateTimer() {
   /* a real free timer would be better but I don't know 
    * how to do this with xforms */
   updatetimer = 0;
}

void SetUpdateTimer() {
   fl_set_timer(fd_form_main->timer_update, 0.5);
   updatetimer = 1;
}


void BeforeChange() {
   buffer[current_buffer]->screen->ToggleSelection();
   buffer[current_buffer]->text->ClearSelection();
   FreeUpdateTimer();
}



void UpdateScrollbar() {
   /* if the text is smaller then the working area, the scrollbar
    * maximum must be the working area height. No scrolling will 
    * be possible */
   static long max2 = 0;
   static long height2 = 0;

   long cbth = 0;
   long cbsf = 0;
   
   if (buffer[current_buffer]->text)
     cbth = buffer[current_buffer]->text->height;
   if (buffer[current_buffer]->screen)
     cbsf = buffer[current_buffer]->screen->first;
   if (max2 == cbth &&
       height2 == work_area_height &&
       current_scrollbar_value == cbsf)
     return;			       /* no update necessary */
   max2 = cbth;
   height2 = work_area_height;
   current_scrollbar_value = cbsf;
   
   long maximum_height = work_area_height * 3/4;
   long value = 0;
   
   if (buffer[current_buffer]->text)
      maximum_height += buffer[current_buffer]->text->height;
   if (buffer[current_buffer]->screen)
      value = buffer[current_buffer]->screen->first;
   
   /* set the scrollbar */
   double hfloat = work_area_height;
   double maxfloat = maximum_height;
   
   fl_set_slider_value(fd_form_main->scrollbar, value);
   fl_set_slider_bounds(fd_form_main->scrollbar, 0, maximum_height - work_area_height);
   /* fl_set_slider_step(fd_form_main->scrollbar, 1); */
   if (maxfloat>0)
      fl_set_slider_size(fd_form_main->scrollbar, hfloat/maxfloat);
   else
      fl_set_slider_size(fd_form_main->scrollbar, hfloat);
   fl_set_slider_precision(fd_form_main->scrollbar, 0);
}


void Update(signed char f) {
   UpdateLayoutChoice();

   buffer[current_buffer]->text->FullRebreak();
   buffer[current_buffer]->screen->Update();
   buffer[current_buffer]->screen->FitCursor();
   UpdateScrollbar();
       
   if (f==1 || f==-1) {
      if (buffer[current_buffer]->lyx_current) {
	 buffer[current_buffer]->lyx_current = 0;
	 buffer[current_buffer]->bak_current = 0;
	 buffer[current_buffer]->dvi_current = 0;
	 fl_set_timer(fd_form_main->timer_minibuffer, 4);
      }
      else {
	 buffer[current_buffer]->lyx_current = 0;
	 buffer[current_buffer]->bak_current = 0;
	 buffer[current_buffer]->dvi_current = 0;
      }
   }
   if (f >= 0 && work_area_active)
     buffer[current_buffer]->screen->ShowCursor();
}

void SmallUpdate(signed char f) {
   buffer[current_buffer]->screen->SmallUpdate();
   buffer[current_buffer]->screen->FitCursor();
   UpdateScrollbar();
   
   if (f==1 || f==-1) {
      if (buffer[current_buffer]->lyx_current) {
	 buffer[current_buffer]->lyx_current = 0;
	 buffer[current_buffer]->bak_current = 0;
	 buffer[current_buffer]->dvi_current = 0;
	 fl_set_timer(fd_form_main->timer_minibuffer, 4);
      }
      else {
	 buffer[current_buffer]->lyx_current = 0;
	 buffer[current_buffer]->bak_current = 0;
	 buffer[current_buffer]->dvi_current = 0;
      }
   }
   if (f >= 0 && work_area_active)
     buffer[current_buffer]->screen->ShowCursor();
}

int ResizeCurrentBuffer(FL_OBJECT *ob, Window win, int w, int h, XEvent *ev, void *d) {
     /* if there is no text loaded, I do not have to resize anything */ 
   if (!buffer[current_buffer]->parameters.paragraph) {
      XFillRectangle(fl_display, win, fl_textgc, 0, 0, w, h);
      UpdateScrollbar();
      return 0;
   }

   LyXParagraph *par = NULL;
   LyXParagraph *selstartpar = NULL;
   LyXParagraph *selendpar = NULL;
   int pos = 0;
   int selstartpos = 0;
   int selendpos = 0;
   int selection = 0;
   int mark_set = 0;

   ProhibitInput();

   SetMinibuffer("Resizing buffer...","","");   

   if (buffer[current_buffer]->text) {
      par = buffer[current_buffer]->text->cursor.par;
      pos = buffer[current_buffer]->text->cursor.pos;
      selstartpar = buffer[current_buffer]->text->sel_start_cursor.par;
      selstartpos = buffer[current_buffer]->text->sel_start_cursor.pos;
      selendpar = buffer[current_buffer]->text->sel_end_cursor.par;
      selendpos = buffer[current_buffer]->text->sel_end_cursor.pos;
      selection = buffer[current_buffer]->text->selection;
      mark_set = buffer[current_buffer]->text->mark_set;
      delete(buffer[current_buffer]->text);
   }
   
   buffer[current_buffer]->text = new LyXText(w,
					      &buffer[current_buffer]->parameters);
   if (buffer[current_buffer]->screen)
      delete(buffer[current_buffer]->screen);
   buffer[current_buffer]->screen = new LyXScreen(win, h, w, buffer[current_buffer]->text);
   
   if (par) {
      buffer[current_buffer]->text->selection = True;    /* at this point just
						  * to avoid the Delete-
						  * Empty-Paragraph
						  * Mechanism when
						  * setting the cursor */
      buffer[current_buffer]->text->mark_set = mark_set;
      if (selection) {
	 buffer[current_buffer]->text->SetCursor(selstartpar, selstartpos);
	 buffer[current_buffer]->text->sel_cursor = buffer[current_buffer]->text->cursor;
	 buffer[current_buffer]->text->SetCursor(selendpar, selendpos);
	 buffer[current_buffer]->text->SetSelection();
	 buffer[current_buffer]->text->SetCursor(par, pos);
      }
      else {
	 buffer[current_buffer]->text->SetCursor(par, pos);
	 buffer[current_buffer]->text->sel_cursor = buffer[current_buffer]->text->cursor;
	 buffer[current_buffer]->text->selection = False;
      }
      
   }
   buffer[current_buffer]->screen->first  = 
   buffer[current_buffer]->screen->TopCursorVisible(); /* this will scroll the screen that 
						* the cursor becomes visible */ 
   UpdateScrollbar();
   buffer[current_buffer]->screen->Redraw();
   
   InitMinibuffer();
   AllowInput();
   return 0;
}


int WorkAreaExpose(FL_OBJECT *ob, Window win, int w, int h, XEvent *ev, void *d) {
   static FL_OBJECT *ob2 = NULL;
   static Window win2 = 0;
   static XEvent *ev2 = NULL;
   static void *d2 = NULL;
   
   /* FD_form_main *ui = (FD_form_main*) d; */ 
   
   if (ob == NULL && !buffer[current_buffer]->screen) {
      /* resize current buffer */ 
      ResizeCurrentBuffer(ob2,win2,work_area_width,work_area_height,ev2,d2);
   }
   else if (w != work_area_width || h != work_area_height) {
      work_area_width = w;
      work_area_height = h;
      ob2 = ob;
      win2 = win;
      ev2 = ev;
      d2 = d;
      /* resize */ 
      /* all buffers will need resize */ 
      for (int i =0; i < NUMBER_OF_BUFFERS; i++) {
	 if (buffer[i]->screen)
	    delete(buffer[i]->screen);
	 buffer[i]->screen = NULL;
      }
      /* resize current buffer */ 
      ResizeCurrentBuffer(ob,win,w,h,ev,d);
   }
   else {
      /* redraw */
      if (buffer[current_buffer]->screen) 
      	buffer[current_buffer]->screen->Redraw(); 
      else {
	 XFillRectangle(fl_display, win, fl_textgc, 0, 0, w, h);
      }
   }
   return 0;
}

void ResizeCurrentBufferPseudoExpose() {
   FL_OBJECT *ob2 = NULL;
   Window win2 = 0;
   XEvent *ev2 = NULL;
   void *d2 = NULL;
   if (buffer[current_buffer]->screen)
     UpdateScrollbar();
   /* resize current buffer through Pseudo-Expose*/
   WorkAreaExpose(ob2,win2,work_area_width,work_area_height,ev2,d2);

   UpdateLayoutChoice();
}


void RedoCurrentBuffer() {
   if (buffer[current_buffer]->text){
      if (buffer[current_buffer]->screen)
      	 delete(buffer[current_buffer]->screen);
      buffer[current_buffer]->screen = NULL;
      ResizeCurrentBufferPseudoExpose();
   }
}

void MenuSearch() {
   fl_show_form(fd_form_search->form_search,
		FL_PLACE_CENTER | FL_FREE_SIZE, FL_FULLBORDER,
		"Search & Replace");
}

void MenuQuit() {
   ProhibitInput();
   int i;
   for (i=0; i < NUMBER_OF_BUFFERS; i++) {
	 if (buffer[i]->parameters.paragraph && !buffer[i]->lyx_current) {
	    if (fl_show_question("Unsaved changes in buffer:",
				 buffer[i]->filename,
				 "Save buffer?"))
	       MenuWrite(i);
	 }
	 
      }
   if (fl_show_question("", "Quit LyX?",""))
      exit(0);
   AllowInput();
}

void MenuPrint() {
   const char* filename = NULL;
   
   if (!buffer[current_buffer]->text) {
      filename = "No_File";
   }
   else
      filename = buffer[current_buffer]->filename;
   
   
   /* calculate ps filename */ 
   int dot=0;
   int path = 0;
   int n=0;
   int n2=0;
   int i=0;
   
   /* find the end of the string */ 
   for (n=0; filename[n]; n++);
   
   
   /* go back to the first dot */ 
   for (n2=n; n2 && filename[n2]!='.'
	&& filename[n2]!='/'; n2--);
   
   /* when you find a dot, use this as extension */ 
   if (filename[n2]=='.')
      dot = n2;
   else
      dot = n;
   
   for (path=dot; path && filename[path]!='/';path--);
   if (filename[path]=='/')
      path++;
   
   
   char *ps = new char[dot+10];
   
   
   for (i=path; i<dot; i++) 
      ps[i-path] = filename[i];
   ps[i-path] = '.';
   ps[i+1-path] = 'p';
   ps[i+2-path] = 's';
   ps[i+3-path] = '\0';

   
   if (lyxrc.printer)
      fl_set_input(fd_form_print->input_printer, lyxrc.printer);
   else
      fl_set_input(fd_form_print->input_printer, "lp");
   
   fl_set_input(fd_form_print->input_file, ps);
   
   
   /* set theother default entries */ 
   fl_set_button(fd_form_print->radio_order_normal, 1);
   fl_set_button(fd_form_print->radio_all_pages, 1);
   fl_set_button(fd_form_print->radio_printer, 1);
   fl_set_button(fd_form_print->radio_file, 0);
   fl_set_button(fd_form_print->radio_odd_pages, 0);
   fl_set_button(fd_form_print->radio_even_pages, 0);
   fl_set_button(fd_form_print->radio_order_revers, 0);

   fl_show_form(fd_form_print->form_print,
		FL_PLACE_CENTER | FL_FREE_SIZE, FL_FULLBORDER,
		"Print");
   delete(ps);

}

void MenuRunLaTeX() {
   
   int ret;
   ret = RunLaTeX();
   
   if (ret == 0) {
      /* fl_show_message("","Compilation successfull!",""); */ 
   }
   else if (ret > 0) {
      char * s = new char [50];
      if (ret == 1)
      	 sprintf(s, "%d error detected",ret);
      else
      	 sprintf(s, "%d errors detected",ret);
      WriteAlert("LaTeX Error!", s, "");
      delete(s);
   }
}

void AutoSave() {
   char *fname;
   int n = 0;
   int i=0;
   
   if (buffer[current_buffer]->text) {
      
      if (!buffer[current_buffer]->bak_current) {
	 SetMinibuffer("Auto save current buffer...","","");
	 /* create autosave filename */
	 for (n=0; buffer[current_buffer]->filename[n]; n++);
	 
	 fname = new char[n+10];
	 
	 for (i=0; buffer[current_buffer]->filename[i]; i++)
      	    fname[i] = buffer[current_buffer]->filename[i];
	 
	 fname[i] = '.';
	 fname[i+1] = 'b';
	 fname[i+2] = 'a';
	 fname[i+3] = 'k';   
	 fname[i+4] = '\0';
	 
	 /* make a backup */ 
	 char *s = new char[n+5];
	 for (i=0; fname[i]; i++) 
      	    s[i] = fname[i];
	 s[i] = '~';
	 s[i+1] = '\0';
	 rename(fname, s);
	 delete(s);
	 
	 buffer[current_buffer]->parameters.cursor = buffer[current_buffer]->text->cursor;
	 
	 WriteFile(fname, 
		   buffer[current_buffer]->parameters);
	 buffer[current_buffer]->bak_current = 1;
	 delete(fname);
	 SetMinibuffer("Auto save current buffer done!","","");
      }
      else {
	 SetMinibuffer("Auto save not neccessary!","","");
      }
   }
   
   if (lyxrc.autosave)
      fl_set_timer(fd_form_main->timer_autosave, lyxrc.autosave);
}

void MenuLoadAutoSave() {
   char *fname;
   int n = 0;
   int i=0;

   if (lyxrc.autosave)
      fl_set_timer(fd_form_main->timer_autosave, lyxrc.autosave);

   if (!buffer[current_buffer]->text)
      return;
   
   /* create autosave filename */
   for (n=0; buffer[current_buffer]->filename[n]; n++);
   
   fname = new char[n+10];
   
   for (i=0; buffer[current_buffer]->filename[i]; i++)
      fname[i] = buffer[current_buffer]->filename[i];
   
   fname[i] = '.';
   fname[i+1] = 'b';
   fname[i+2] = 'a';
   fname[i+3] = 'k';   
   fname[i+4] = '\0';
   
   SetMinibuffer("Loading autosave file ",fname,"...");   
   
   if (!OpenAllFile(fname) && !OpenAllFile(buffer[current_buffer]->filename)) {
      WriteAlert("Operation not possible!", "No autosave file",
		 fname);
   } else {
      
      InitTextParameters(buffer[current_buffer]->parameters);
      
      LyXParagraph *par = buffer[current_buffer]->parameters.paragraph;
      LyXParagraph *tmppar = par;
      while(par){
	 tmppar = par->next;
	 delete(par);
	 par = tmppar;
      }
      
      ReadFile(buffer[current_buffer]->parameters);
      buffer[current_buffer]->lyx_current = 0;
      buffer[current_buffer]->bak_current = 0;
      buffer[current_buffer]->dvi_current = 0;
      CloseFile();
      
      buffer[current_buffer]->text->cursor = buffer[current_buffer]->parameters.cursor;
      buffer[current_buffer]->text->ClearSelection();
      
      
      /* use the backup as new autosave file */
      rename(fname, ".lyxtmp");
      char *s = new char[n+5];
      for (i=0; fname[i]; i++) 
      	 s[i] = fname[i];
      s[i] = '~';
      s[i+1] = '\0';
      rename(s, fname);
      rename(".lyxtmp", s);
      delete(s);
      if (buffer[current_buffer]->screen) {
	 delete(buffer[current_buffer]->screen);
	 buffer[current_buffer]->screen = NULL;
      }
      ResizeCurrentBufferPseudoExpose();
      SetMinibuffer("Autosave file loaded.","","");   
   }
   
   delete(fname);
}

void MenuOpen() {
   const char *fname;
   char *s;
   int i=0;
   ProhibitInput();
   fname = fl_show_file_selector("File To Load", document_path,"*.lyx","");
   /* shorten the filename if it begins with the path */
   AllowInput();
   
   if (!fname)
      return;
   
   char *path = new char[500];
   getcwd(path, 500);
   for (i=0; fname[i] == path[i]; i++);
   
   if (i && path[i] == '\0' && fname[i]=='/')
     fname += i + 1;
   delete(path);
   
   
   if (document_path){
     delete(document_path);
     document_path = NULL;
   }
   if (fname == NULL) return;

   SetMinibuffer("Loading file",fname,"...");   
   
   if (!OpenFile(fname)) {
      WriteAlert("Error!", "Cannot open specified file:",
		    fname);
   } else {
      
      /* file already open? */
      for (i=0; i < NUMBER_OF_BUFFERS && 
	   !StringEqual(buffer[i]->filename,
			fname);
	   i++);
      
      if (i<NUMBER_OF_BUFFERS) {
	 CloseFile();
	 WriteAlert("Error!", "Buffer already exists:",
		    fname);
      }
      else {
	 /* find a free buffer */ 
	 i=0;
	 while (i < NUMBER_OF_BUFFERS - 1 && buffer[i]->parameters.paragraph)
	    i++;
	 
	 int a =0;
	 for (a=0;fname[a];a++);
	 s = new char[a+5];
	 for (a=0;fname[a];a++)
	   s[a]=fname[a];
	 s[a]='\0';
	 
	 buffer[i]->filename = s;
	 fl_addto_menu(fd_form_main->menu_buffer, buffer[i]->filename);
	 buffer[i]->menu_position = fl_get_menu_maxitems(fd_form_main->menu_buffer);
	 InitTextParameters(buffer[i]->parameters);
	 ReadFile(buffer[i]->parameters);
	 buffer[i]->lyx_current = 1;
	 buffer[i]->bak_current = 1;
	 buffer[i]->dvi_current = 0;
	 old_current_buffer = current_buffer;
	 current_buffer = i;
	 
	 CloseFile();
      
	 ResizeCurrentBufferPseudoExpose();
      }
   }
}

void InsertAsciiFile(char flag) {
   const char *fname;
   LyXParagraph *tmppar;
   
   if (!buffer[current_buffer]->screen)
     return;
   
   ProhibitInput();
   fname = fl_show_file_selector("File To Insert","","*","");
   AllowInput();
   if (fname == NULL) return;
   if (!OpenAllFile(fname)) {
     WriteAlert("Error!", "Cannot open specified file:",
		fname);
      
   } else {
      tmppar = new LyXParagraph();
      ReadSimpleWholeFile(tmppar);
      /* set the end of the string */
      tmppar->InsertChar(tmppar->last-1,'\0');
      CloseFile();
      /* insert the string */
      if (work_area_active)
      	buffer[current_buffer]->screen->HideCursor();
      
      /* clear the selection */
      BeforeChange();
      if (!flag)
      	buffer[current_buffer]->text->InsertStringA(tmppar->text);
      else
      	buffer[current_buffer]->text->InsertStringB(tmppar->text);
      delete(tmppar);
      Update(1);
   }
}

void MenuNew() {
   const char *fname;

   char *s;
   int i;
   ProhibitInput();
   fname = fl_show_input("Insert Filename:", "newfile.lyx");
   AllowInput();
   if (fname == NULL) return;
   

   /* make sure the filename ends with .lyx */
   for (i=0;fname[i];i++);
   s = new char[i+5];
   for (i=0;fname[i];i++)
     s[i]=fname[i];
   s[i]='\0';
   if (!IsLyXFilename(s)) {
      s[i]='.';i++;
      s[i]='l';i++;
      s[i]='y';i++;
      s[i]='x';i++;
      s[i]='\0';
  }
  
  /* check wether the file exists */
  if (OpenFile(s)) {
    CloseFile();
    WriteAlert("Error!", "File exists:",
		  s);
    delete(s);
  }
  else {
     /* file already open? */
     for (i=0; i < NUMBER_OF_BUFFERS && 
	  !StringEqual(buffer[i]->filename,
		       s);
	  i++);
     
     if (i<NUMBER_OF_BUFFERS) {
	CloseFile();
	WriteAlert("Error!", "Buffer already exists:",
		      s);
     }
     else {
	/* the template stuff */
	ProhibitInput();
	int i;
	if (document_path)
	  delete(document_path);
	document_path = StringCopy(fl_get_directory());
	fname = fl_show_file_selector("Choose Template",lyxrc.template_path,"*.lyx","");
	AllowInput();

	/* find a free buffer */ 
	i=0;
	while (i < NUMBER_OF_BUFFERS - 1 && buffer[i]->parameters.paragraph)
	  i++;

	InitTextParameters(buffer[i]->parameters);	

	if (fname && OpenFile(fname)){
	   ReadFile(buffer[i]->parameters);
	   CloseFile();
	}
	else {
	  buffer[i]->parameters.paragraph = new LyXParagraph();
	}

	buffer[i]->filename = s;
	old_current_buffer = current_buffer;
	current_buffer = i;
	buffer[current_buffer]->lyx_current = 0;
	buffer[current_buffer]->bak_current = 0;
	buffer[current_buffer]->dvi_current = 0;
	
	ResizeCurrentBufferPseudoExpose();
	fl_addto_menu(fd_form_main->menu_buffer, buffer[current_buffer]->filename); 
	buffer[current_buffer]->menu_position = fl_get_menu_maxitems(fd_form_main->menu_buffer);
   }
  }      
}

void MenuWrite(int buf) {
   int n, i;
   if (buffer[buf]->parameters.paragraph) {
      SetMinibuffer("Write buffer", buffer[buf]->filename,"...");
      /* make a backup */ 
      for (n=0; buffer[buf]->filename[n]; n++);
      char *s = new char[n+5];
      for (i=0; i<n; i++) 
      	s[i] = buffer[buf]->filename[i];
      s[i] = '~';
      s[i+1] = '\0';
      rename(buffer[buf]->filename, s);
      delete(s);
      
      if (buffer[buf]->text)
      	 buffer[buf]->parameters.cursor = buffer[buf]->text->cursor;
      
      WriteFile(buffer[buf]->filename, 
		buffer[buf]->parameters);
      buffer[buf]->lyx_current = 1;
      SetMinibuffer("Wrote buffer", buffer[buf]->filename,"!");
      
   }
}

void MenuMakeLaTeX() {
   int n, n2, i;
   if (buffer[current_buffer]->text) {
      
      /* get LaTeX-Filename */
      
      /* find the end of the string */ 
      for (n=0; buffer[current_buffer]->filename[n]; n++);
      
      /* go back to the first dot */ 
      for (n2=n; n2 && buffer[current_buffer]->filename[n2]!='.'
	   && buffer[current_buffer]->filename[n2]!='/'; n2--);
      
      /* when you find a dot, use this as extension */ 
      if (buffer[current_buffer]->filename[n2]=='.')
      	n = n2;
      char *s = new char[n+10];
      for (i=0; i<n; i++) 
      	s[i] = buffer[current_buffer]->filename[i];
      s[i] = '.';
      s[i+1] = 't';
      s[i+2] = 'e';
      s[i+3] = 'x';
      s[i+4] = '\0';
      MakeLaTeXFile(s,
		    buffer[current_buffer]->parameters,
		    65);
      delete(s);
      buffer[current_buffer]->dvi_current = 0;
      
   }
   
}
void MenuWriteAs() {
   const char *fname;

   char *s = NULL;
   int i;
   if (!buffer[current_buffer]->text)
      return;
   
   ProhibitInput();
   fname = fl_show_input("Insert New Filename:", buffer[current_buffer]->filename);
   AllowInput();
   if (fname == NULL) return;
   
   /* make sure the filename ends with .lyx */
   for (i=0;fname[i];i++);
   s = new char[i+5];
   for (i=0;fname[i];i++)
     s[i]=fname[i];
   s[i]='\0';
   if (!IsLyXFilename(s)) {
      s[i]='.';i++;
      s[i]='l';i++;
      s[i]='y';i++;
      s[i]='x';i++;
      s[i]='\0';
  }
  
  /* check wether the file exists */
  if (OpenFile(s)) {
    CloseFile();
    WriteAlert("Error!", "File exists:",
		  s);
    delete(s);
  }
  else {
     /* file already open? */
     for (i=0; i < NUMBER_OF_BUFFERS && 
	  !StringEqual(buffer[i]->filename,
		       s);
	  i++);
     
     if (i<NUMBER_OF_BUFFERS) {
	CloseFile();
	WriteAlert("Error!", "Buffer already exists:",
		      s);
     }
     else {
	if (buffer[current_buffer]->filename) {
	   delete (buffer[current_buffer]->filename);
	}
	
	buffer[current_buffer]->filename = s;
	
	buffer[current_buffer]->lyx_current = 0;
	buffer[current_buffer]->dvi_current = 0;
	
	
	/* update the buffer menu menu */
	int pos = buffer[current_buffer]->menu_position;
	
       fl_replace_menu_item(fd_form_main->menu_buffer, 
			   pos,
			   buffer[current_buffer]->filename);
       }
       
     MenuWrite(current_buffer);
  }
}    



int WorkAreaButtonRelease(FL_OBJECT *ob, Window win, int w, int h, XEvent *ev, void *d) {
   ResetKeyboardMode();
   selection_possible = False;
   return 0;
}

int WorkAreaButtonPress(FL_OBJECT *ob, Window win, int w, int h, XEvent *ev, void *d) {
   ResetKeyboardMode();
   work_area_active = True;
   if (!buffer[current_buffer]->screen)
     return 0 ;
   selection_possible = True;
   buffer[current_buffer]->screen->HideCursor(); 
   /* clear the selection */ 
   buffer[current_buffer]->screen->ToggleSelection();
   buffer[current_buffer]->text->ClearSelection();
   buffer[current_buffer]->text->FullRebreak();
   buffer[current_buffer]->screen->Update();
   UpdateScrollbar();
   buffer[current_buffer]->text->SetCursorFromCoordinates(ev->xbutton.x,
							  ev->xbutton.y +
							  buffer[current_buffer]->screen->first);
   buffer[current_buffer]->text->sel_cursor = buffer[current_buffer]->text->cursor;
   buffer[current_buffer]->text->cursor.x_fix = buffer[current_buffer]->text->cursor.x;
   
   /* i will use the middle button to edit insets or toggle footnotes */ 
   if (ev->xbutton.button == 2) {
      buffer[current_buffer]->text->ToggleFootnote();
      Update(0);

   }
	 

   /* the right button will select a word */
   
   if (ev->xbutton.button == 3) {
      buffer[current_buffer]->text->SelectWord();
      buffer[current_buffer]->screen->ToggleSelection();
   }
   
   Update(0); /* this will fit the cursor on the screen
	       * if necessary */ 
   
   return 0;
}



static void CursorPrevious() {
   long y = buffer[current_buffer]->screen->first;
   long cursory = buffer[current_buffer]->text->cursor.y - y;
   Row* row = buffer[current_buffer]->text->GetRowNearY(y);
   y = y - work_area_height + row->height;
   buffer[current_buffer]->screen->Draw(y);
   buffer[current_buffer]->text->SetCursorFromCoordinates(buffer[current_buffer]->text->cursor.x,
							  cursory + y);
}
static void CursorNext() {
   long y = buffer[current_buffer]->screen->first;
   long cursory = buffer[current_buffer]->text->cursor.y - y;
   y += work_area_height;
   buffer[current_buffer]->text->GetRowNearY(y);
   buffer[current_buffer]->screen->Draw(y);
   buffer[current_buffer]->text->SetCursorFromCoordinates(buffer[current_buffer]->text->cursor.x_fix,
							  cursory + y);
}

int WorkAreaKeyPress(FL_OBJECT *ob, Window win, int w, int h, XEvent *ev, void *d) {
   static char s_r[10];
   const char * string_return;
   XKeyEvent *keyevent = &ev->xkey;
   KeySym keysym_return;
   int tmp;
   selection_possible = False;
   XLookupString(keyevent, &s_r[0], 10, &keysym_return, NULL);
   string_return = XKeysymToString(keysym_return);
   work_area_active = True;
   
   if (buffer[current_buffer]->screen)
      buffer[current_buffer]->screen->HideCursor();
   
   if (keyboard_mode) {
      SetMinibuffer("C-x", string_return,"");
      switch (keysym_return) { 
       case XK_u: MenuLoadAutoSave();break;
       case XK_a: AutoSave();break;
       case XK_f: MenuOpen();break;
       case XK_d: MenuNew();break;
       case XK_s: MenuWrite(current_buffer); break;
       case XK_w: MenuWriteAs(); break;
       case XK_c: MenuQuit(); break;
       case XK_k: CloseBuffer();break;
       case XK_r: MenuRunLaTeX();break;
       case XK_t: 
	 TocUpdateCB(ob, 0);
	 fl_show_form(fd_form_toc->form_toc,
		      FL_PLACE_CENTER | FL_FREE_SIZE, FL_FULLBORDER,
		      "Table Of Contents");
	 break;
       case XK_g: MenuPreviewPS();break;
       case XK_p: MenuPreview();break;
       case XK_b:
		  if (buffer[current_buffer]->text){	  
	   BeforeChange();
	   Update(-2);
	 }
		  tmp = current_buffer;
		  current_buffer = old_current_buffer;
		  old_current_buffer = tmp;
		  ResizeCurrentBufferPseudoExpose();
		  InitMinibuffer();
		  break;
       default: 
	LyXBell(); 
	SetMinibuffer("Unknown command: C-x", string_return,"");
	break;
      }
      keyboard_mode = 0;
      return 0;
   }
   
   if (!buffer[current_buffer]->screen) {
      if ((ev->xkey.state & ControlMask)
	  && keysym_return == XK_x){
	 keyboard_mode = CONTROL_X_MODE;
	 SetMinibuffer("C-x","","");
      }
      return 0;
   }
   
   if (keysym_return >= XK_F1 && keysym_return <= XK_F35) {
      int tmplayoutnumber = keysym_return - XK_F1;
      
      if (fl_get_choice(fd_form_main->choice_layouts) - 1 != tmplayoutnumber) {
	 Update(-2);
	 fl_set_choice(fd_form_main->choice_layouts, tmplayoutnumber + 1);
	 buffer[current_buffer]->text->SetLayout(tmplayoutnumber);
      }
      Update(1);
   }


   /* cursor movements */ 
   if (keysym_return == XK_Right || 
       keysym_return == XK_Left || 
       keysym_return == XK_Up || 
       keysym_return == XK_Down ||
       keysym_return == XK_Prior ||
       keysym_return == XK_Next ||
       keysym_return == XK_Home ||
       keysym_return == XK_End 
       ||(
	  (ev->xkey.state & Mod1Mask) &&
	  (keysym_return == XK_f ||
	   keysym_return == XK_b ||
	   keysym_return == XK_v ||
	   keysym_return == XK_V ||
	   keysym_return == XK_F ||
	   keysym_return == XK_B)
	       )
       ||(
	  (ev->xkey.state & ControlMask) &&
	  (keysym_return == XK_f ||
	   keysym_return == XK_b ||
	   keysym_return == XK_v ||
	   keysym_return == XK_V ||
	   keysym_return == XK_a ||
	   keysym_return == XK_e ||
	   keysym_return == XK_A ||
	   keysym_return == XK_E ||
	   keysym_return == XK_n ||
	   keysym_return == XK_N ||
	   keysym_return == XK_p ||
	   keysym_return == XK_P ||
	   keysym_return == XK_F ||
	   keysym_return == XK_B)
	       )
       ) {
      if (!(ev->xkey.state & ShiftMask) && !buffer[current_buffer]->text->mark_set) {
	 BeforeChange();
      }
      Update(-2);
      if (!(ev->xkey.state & ControlMask)) {
	 switch (keysym_return) {
	  case XK_Right: buffer[current_buffer]->text->CursorRight(); break;
	  case XK_Left: buffer[current_buffer]->text->CursorLeft(); break;
	  case XK_Up: buffer[current_buffer]->text->CursorUp(); break;
	  case XK_Down: buffer[current_buffer]->text->CursorDown(); break;
	  case XK_Prior: case XK_v: case XK_V: CursorPrevious(); break;
	  case XK_Next: CursorNext(); break;
	  case XK_Home: buffer[current_buffer]->text->CursorHome(); break;
	  case XK_End: buffer[current_buffer]->text->CursorEnd(); break;
	  case XK_f:case XK_F: buffer[current_buffer]->text->CursorRightOneWord(); break;
	  case XK_b:case XK_B: buffer[current_buffer]->text->CursorLeftOneWord(); break;
	 }
      }
      else {
	 switch (keysym_return) {
	  case XK_Right: buffer[current_buffer]->text->CursorRightOneWord(); break;
	  case XK_Left: buffer[current_buffer]->text->CursorLeftOneWord(); break;
	  case XK_Up: case XK_p:case XK_P: buffer[current_buffer]->text->CursorUp(); break;
	  case XK_Down: case XK_n: case XK_N: buffer[current_buffer]->text->CursorDown(); break;
	  case XK_Prior: CursorPrevious(); break;
	  case XK_Next: case XK_v: case XK_V: CursorNext(); break;
	  case XK_Home: buffer[current_buffer]->text->CursorTop(); break;
	  case XK_End: buffer[current_buffer]->text->CursorBottom(); break;
	  case XK_a: case XK_A: buffer[current_buffer]->text->CursorHome(); break;
	  case XK_e: case XK_E: buffer[current_buffer]->text->CursorEnd(); break;
	  case XK_f: case XK_F: buffer[current_buffer]->text->CursorRight(); break;
	  case XK_b: case XK_B: buffer[current_buffer]->text->CursorLeft(); break;
	 }
      }
      if ((ev->xkey.state & ShiftMask) || buffer[current_buffer]->text->mark_set) {
	 buffer[current_buffer]->text->SetSelection();
	 buffer[current_buffer]->screen->ToggleToggle();
	 Update(0);
      }
      else {
	 Update(0);
	 buffer[current_buffer]->text->sel_cursor = buffer[current_buffer]->text->cursor;
      }
      
   }
   
   
   
   
   
   /* a few shortcuts with control */

   else if (ev->xkey.state & ControlMask){
      switch (keysym_return) {
       case XK_s: MenuSearch(); break;
       case XK_y: case XK_7: PasteCB(ob, 0); break;
       case XK_w: case XK_6: CutCB(ob, 0); break;
       case XK_1: EmphCB(ob, 0); break;
       case XK_2: NounCB(ob, 0); break;
       case XK_g: GotoError(); break;
       case XK_o: OpenStuff(); break;
       case XK_h: HyphenationPoint(); break;
       case XK_i: HFill(); break;
       case XK_0: DepthCB(ob,0); break;
       case XK_3: FreeCB(ob,0); break;
       case XK_8: TexCB(ob, 0);break;
       case XK_9: FormulaCB(ob, 0); break;
       case XK_4: 
	 if (buffer[current_buffer]->text
	     && !buffer[current_buffer]->text->selection
	     && buffer[current_buffer]->text->cursor.par->footnoteflag!=LYX_NO_FOOTNOTE)
	    MeltCB(ob,0);
	 else
	    FootCB(ob, 0); 
	 break;
       case XK_5: 
	 if (buffer[current_buffer]->text
	     && !buffer[current_buffer]->text->selection
	     && buffer[current_buffer]->text->cursor.par->footnoteflag!=LYX_NO_FOOTNOTE)
	    MeltCB(ob,0);
	 else
	    MarginCB(ob, 0); 
	 break;

	 
       case XK_x: 
	 Update(-2); 
	 keyboard_mode = CONTROL_X_MODE; 
	 SetMinibuffer("C-x","","");
	 break;
	 
       case XK_Return:
	 BeforeChange();
	 buffer[current_buffer]->text->InsertChar(LYX_META_NEWLINE);
	 SmallUpdate(1);
	 SetUpdateTimer();
	 break;
	 
       case XK_space:
	 if(buffer[current_buffer]->text->mark_set) {
	    BeforeChange();
	    Update(0);
	    SetMinibuffer("mark removed","","");
	 }
	 else {
	    BeforeChange();
	    buffer[current_buffer]->text->mark_set = 1;
	    Update(0);
	    SetMinibuffer("mark set","","");
	 }
	 buffer[current_buffer]->text->sel_cursor = buffer[current_buffer]->text->cursor;
	 break;
	 
       case XK_d: case XK_D:
	 FreeUpdateTimer();
	 if (!buffer[current_buffer]->text->selection) {
	    buffer[current_buffer]->text->Delete();
	    buffer[current_buffer]->text->sel_cursor = buffer[current_buffer]->text->cursor;
	    SmallUpdate(1);
	 }
	 else {
	    buffer[current_buffer]->text->CutSelection();
	    Update(1);
	    SetMinibuffer("Cut!","","");
	 }
	 SetUpdateTimer();
	 break;
	 
       case XK_l:
	 BeforeChange();
	 Update(-2);
	 if (buffer[current_buffer]->text->cursor.y > work_area_height / 2) {
	    buffer[current_buffer]->screen->Draw(buffer[current_buffer]->text->cursor.y - work_area_height / 2);
	    UpdateScrollbar();
	    SetMinibuffer("Screen centered!","","");
	 }
	 else {
	    LyXBell();
	    SetMinibuffer("Cannot center top of document.","","");
	 }
	 Update(0);
	 break;
	 
       case XK_q: case XK_quotedbl:
	 BeforeChange();
	 buffer[current_buffer]->text->InsertChar('\"');
	 SmallUpdate(1);
	 SetUpdateTimer();
	 break;
      }
   }
   
   /* a few shortcuts with alt */ 
   else if (ev->xkey.state & Mod1Mask){
      switch (keysym_return) {
       case XK_w: CopyCB(); break;
       case XK_space:
	 BeforeChange();
	 buffer[current_buffer]->text->InsertChar(LYX_META_PROTECTED_SEPARATOR);
	 SmallUpdate(1);
	 SetUpdateTimer();
	 break;
      }
   }
   
   
   else {
      if (delete_mapped_to_backspace
	  && keysym_return == XK_Delete)
      	 keysym_return = XK_BackSpace;
      switch (keysym_return) {
       case XK_Delete:
	 FreeUpdateTimer();
	 if (!buffer[current_buffer]->text->selection) {
	    buffer[current_buffer]->text->Delete();
	    buffer[current_buffer]->text->sel_cursor = buffer[current_buffer]->text->cursor;
	    SmallUpdate(1);
	 }
	 else {
	    buffer[current_buffer]->text->CutSelection();
	    Update(1);
	    SetMinibuffer("Cut!","","");
	 }
	 SetUpdateTimer();
	 break;
       case XK_Return:
	 BeforeChange();
	 buffer[current_buffer]->text->BreakParagraph();
	 SmallUpdate(1);
	 SetUpdateTimer();
	 buffer[current_buffer]->text->sel_cursor = buffer[current_buffer]->text->cursor;
	 break;
       case XK_BackSpace:
	 BeforeChange();
	 buffer[current_buffer]->text->Backspace();
	 SmallUpdate(1);
	 SetUpdateTimer();
	 buffer[current_buffer]->text->sel_cursor = buffer[current_buffer]->text->cursor;
       default:
	 if(s_r[0] && keysym_return < 0xF000){
	    BeforeChange();
	    /* the quotes */ 
	    if (s_r[0]=='\"')
	       InsertCorrectQuote();
	    else
	       buffer[current_buffer]->text->InsertChar(s_r[0]);
	    
	    SmallUpdate(1);
	    SetUpdateTimer();
	    /* printf("%c  \n",string_return[0]);*/ 
	    buffer[current_buffer]->text->sel_cursor = buffer[current_buffer]->text->cursor;
	 }
      }
   }
   return 0;
}

int WorkAreaMotionNotify(FL_OBJECT *ob, Window win, int w, int h, XEvent *ev, void *d) {
   if (!buffer[current_buffer]->screen)
     return 0;
   
   /* only use motion with button 1 */
   if (ev->xmotion.state & Button3MotionMask
       || ev->xmotion.state & Button2MotionMask)
      return 0; 
    
   
   /* the selection possible is needed, that only motion events are 
    * used, where the bottom press event was on the drawing area too */
   if (selection_possible) {
      
      buffer[current_buffer]->screen->HideCursor();
      /* buffer[current_buffer]->screen->ToggleSelection(); */

      buffer[current_buffer]->text->SetCursorFromCoordinates(ev->xbutton.x,
						     ev->xbutton.y +
						     buffer[current_buffer]->screen->first);
      if (!buffer[current_buffer]->text->selection)
      	Update(-2);       /* maybe an empty line was deleted  */
      buffer[current_buffer]->text->SetSelection();
      buffer[current_buffer]->screen->ToggleToggle();
      if (buffer[current_buffer]->screen->FitCursor())
      	UpdateScrollbar(); 
      buffer[current_buffer]->screen->ShowCursor();
   } 
   return 0;
}
int WorkAreaEnterNotify(FL_OBJECT *ob, Window win, int w, int h, XEvent *ev, void *d) {
   ResetKeyboardMode();
   work_area_active = True;
   if (buffer[current_buffer]->screen && !input_prohibited)
     buffer[current_buffer]->screen->ShowCursor();
   return 0;
}
int WorkAreaLeaveNotify(FL_OBJECT *ob, Window win, int w, int h, XEvent *ev, void *d) {
   ResetKeyboardMode();
   work_area_active = False;
   if (buffer[current_buffer]->screen) {
      buffer[current_buffer]->screen->HideCursor();
      buffer[current_buffer]->text->FullRebreak();
      buffer[current_buffer]->screen->Update();
      UpdateScrollbar();
   }
   return 0;
}




void CursorToggleCB(FL_OBJECT *ob, long data) {
   if (work_area_active && buffer[current_buffer]->screen)
     buffer[current_buffer]->screen->CursorToggle();
   fl_set_timer(ob, 0.4);
}



void InitTextParameters(LyXTextParameters &p) {
   p = lyxrc.parameters;
   p.language = StringCopy(p.language);
   p.fonts = StringCopy(p.fonts);
   p.inputenc = StringCopy(p.inputenc);
   p.preamble = StringCopy(p.preamble);
   p.options = StringCopy(p.options);
}

void CloseBuffer() {
   int i;

   /* This doesn't make sense if there is no document */
   if (!buffer[current_buffer]->parameters.paragraph)
     return;
						    

   /*doit save buffer etc */
   i = current_buffer;
   if (buffer[i]->parameters.paragraph && !buffer[i]->lyx_current) {
      ProhibitInput();
      if (fl_show_question("Unsaved changes in buffer:",
			   buffer[i]->filename,
			   "Save buffer?"))
      	MenuWrite(i);
      AllowInput();
   }
   /* find the item number in the menu */
   int pos = buffer[current_buffer]->menu_position;
   
   fl_delete_menu_item(fd_form_main->menu_buffer,
		       pos);

   for (i=0; i < NUMBER_OF_BUFFERS; i++) {
      if (buffer[i]->menu_position > pos)
      	buffer[i]->menu_position--;
   }
   
   if (buffer[current_buffer]->filename) {
      delete(buffer[current_buffer]->filename);
      buffer[current_buffer]->filename = NULL;
   }
   
   LyXParagraph *par = buffer[current_buffer]->parameters.paragraph;
   LyXParagraph *tmppar = par;
   while(par){
      tmppar = par->next;
      delete(par);
      par = tmppar;
   }
   buffer[current_buffer]->parameters.paragraph = NULL;
   
   if (buffer[current_buffer]->text) {
      delete(buffer[current_buffer]->text);
      buffer[current_buffer]->text = NULL;
   }
   if (buffer[current_buffer]->screen) {
      delete(buffer[current_buffer]->screen);
      buffer[current_buffer]->screen = NULL;
   }
   buffer[current_buffer]->menu_position = 0;
   
   buffer[current_buffer]->lyx_current = 0;
   buffer[current_buffer]->dvi_current = 0;
   buffer[current_buffer]->bak_current = 0;

   delete(buffer[current_buffer]->parameters.language);
   buffer[current_buffer]->parameters.language=NULL;
   delete(buffer[current_buffer]->parameters.fonts);
   buffer[current_buffer]->parameters.fonts=NULL;
   delete(buffer[current_buffer]->parameters.inputenc);
   buffer[current_buffer]->parameters.inputenc=NULL;
   delete(buffer[current_buffer]->parameters.preamble);
   buffer[current_buffer]->parameters.preamble=NULL;
   delete(buffer[current_buffer]->parameters.options);
   buffer[current_buffer]->parameters.options=NULL;
   
   ResizeCurrentBufferPseudoExpose();
}


void init(int argc, char **argv) {
   int i, a;

   lyxrc.parameters.paragraph_separation = LYX_PARSEP_INDENT;
/*   lyxrc.parameters.quotes_language = GERMAN; */ 
   lyxrc.parameters.quotes_language = ENGLISH;
   lyxrc.parameters.quotes_times = 2;
/*  lyxrc.parameters.papersize = PAPER_A4PAPER; */ 
   lyxrc.parameters.papersize = PAPER_USLETTER;
   lyxrc.parameters.fontsize = 10;
   lyxrc.parameters.baseline = 1.0;
   lyxrc.parameters.secnumdepth = 3;
   lyxrc.parameters.tocdepth = 3;
/*    lyxrc.parameters.language = "german"; */ 
   lyxrc.parameters.language = StringCopy("default");
   lyxrc.parameters.fonts = StringCopy("default");
   lyxrc.parameters.inputenc = StringCopy("default");
   lyxrc.parameters.preamble = new char; 
   lyxrc.parameters.preamble[0] = 0;
   lyxrc.parameters.options = new char; 
   lyxrc.parameters.textclass = 0;
   lyxrc.parameters.options[0] = 0;
   lyxrc.parameters.no_epsfig = 0;
   
   lyxrc.parameters.sides = 1;
   lyxrc.parameters.columns = 1;
   lyxrc.parameters.orientation = ORIENTATION_PORTRAIT;
   lyxrc.parameters.pagestyle = PAGESTYLE_PLAIN;
   
   lyxrc.printer = StringCopy("lp");
   lyxrc.print_command = "dvips";
   lyxrc.xdvi_extra_options = "";
   lyxrc.template_path = "";
   
   lyxrc.default_zoom = 14;
   lyxrc.autosave = 120;
   lyxrc.parameters.cursor.par = NULL;
   lyxrc.parameters.cursor.pos = 0;
   

   /* load the layouts in the layout choice  */
   LyXSetStyle();
   
   /* put the text classes in the coice */ 
   for (i = 0; !StringEqual(NameOfClass(i),"dummy"); i++) {
      fl_addto_choice(fd_form_document->choice_class, NameOfClass(i));
   }

   
   /* define the slider bounds */ 
   fl_set_slider_bounds(fd_form_paragraph->slider_spaces_top,0,20);
   fl_set_slider_bounds(fd_form_paragraph->slider_spaces_bottom,0,20);
   fl_set_slider_bounds(fd_form_document->slider_baselinestretch,0.75,2);
   fl_set_slider_bounds(fd_form_document->slider_secnumdepth,-1,5);
   fl_set_slider_bounds(fd_form_document->slider_tocdepth,-1,5);
   fl_set_slider_bounds(fd_form_table->slider_rows,1,50);
   fl_set_slider_bounds(fd_form_table->slider_columns,1,50);
   fl_set_slider_value(fd_form_table->slider_rows, 5);
   fl_set_slider_value(fd_form_table->slider_columns, 5);
   fl_set_slider_precision(fd_form_document->slider_secnumdepth, 0);
   fl_set_slider_precision(fd_form_document->slider_tocdepth, 0);
   fl_set_slider_precision(fd_form_table->slider_rows, 0);
   fl_set_slider_precision(fd_form_table->slider_columns, 0);
   
   fl_set_button(fd_form_figure->radio_postscript, 1);
   fl_set_button(fd_form_figure->radio_latex, 0);
   
   fl_add_canvas_handler(fd_form_main->work_area, Expose, WorkAreaExpose, fd_form_main);
   fl_add_canvas_handler(fd_form_main->work_area, KeyPress, WorkAreaKeyPress, fd_form_main);
   fl_add_canvas_handler(fd_form_main->work_area, ButtonPress, WorkAreaButtonPress, fd_form_main);
   fl_add_canvas_handler(fd_form_main->work_area, ButtonRelease, WorkAreaButtonRelease, fd_form_main);
   fl_add_canvas_handler(fd_form_main->work_area, MotionNotify, WorkAreaMotionNotify, fd_form_main);
   fl_add_canvas_handler(fd_form_main->work_area, EnterNotify, WorkAreaEnterNotify, fd_form_main);
   fl_add_canvas_handler(fd_form_main->work_area, LeaveNotify, WorkAreaLeaveNotify, fd_form_main);
   
   fl_add_canvas_handler(fd_form_main->minibuffer, Expose, MinibufferExpose, fd_form_main);
   

   /* activate the timer for the cursor */ 
   fl_set_timer(fd_form_main->timer_cursor, 0.4);
   
   if (lyxrc.autosave)
      fl_set_timer(fd_form_main->timer_autosave, lyxrc.autosave);
   
   /* set a nice cursor for the drawing area */ 
   xterm_cursor = XCreateFontCursor(fl_display ,XC_xterm);
   XDefineCursor(fl_display, fl_get_canvas_id(fd_form_main->work_area), 
		 xterm_cursor);


   
   /* read the rc file */ 
   char *cpwd = new char[500];
   getcwd(cpwd, 500);
   const char* home = getenv("HOME");
   chdir(home);
   ReadRCFile(".lyxrc", lyxrc);
   chdir(cpwd);
   delete(cpwd);

   SetZoom(lyxrc.default_zoom);
   
   /* init all buffers */
   for (i=0; i < NUMBER_OF_BUFFERS; i++) {
      buffer[i] = new Buffer();
      buffer[i]->filename = NULL;
      buffer[i]->parameters.paragraph = NULL;
      buffer[i]->text = NULL;
      buffer[i]->screen = NULL;
      buffer[i]->lyx_current = 0;
      buffer[i]->bak_current = 0;
      buffer[i]->dvi_current = 0;
      buffer[i]->menu_position = 0;
      InitTextParameters(buffer[i]->parameters);
   }
   current_buffer = 0;
   old_current_buffer = current_buffer;

   
   /* set the layout choice */ 
   for (i = 0; !StringEqual(NameOfLayout(lyxrc.parameters.textclass, i),"dummy"); i++) {
      fl_addto_choice(fd_form_main->choice_layouts, NameOfLayout(lyxrc.parameters.textclass,i));
   } 
   
   /* install the signal handlers */ 
   signal(SIGHUP, error_handler);
   signal(SIGFPE, error_handler);
   signal(SIGSEGV, error_handler);

   
   /* load the files specified in the command line */ 
   
   for (int argi = 1; argi < argc; argi++) {
      if (!OpenFile(argv[argi])) {
	 fprintf(stderr, "Cannot open specified file %s \n", argv[argi]);
      } else {
	 /* find a free buffer */ 
	 i=0;
	 while (i < NUMBER_OF_BUFFERS - 1 && buffer[i]->parameters.paragraph)
	   i++;
	 for (a=0;argv[argi][a];a++);
	 buffer[i]->filename = new char[a+5];
	 for (a=0;argv[argi][a];a++)
	   buffer[i]->filename[a] = argv[argi][a];
	 buffer[i]->filename[a] = '\0';
	 InitTextParameters(buffer[i]->parameters);
	 ReadFile(buffer[i]->parameters);
	 buffer[i]->lyx_current = 1;
	 buffer[i]->bak_current = 0;
	 buffer[i]->dvi_current = 0;
	 old_current_buffer = current_buffer;
	 current_buffer = i;
	 CloseFile();
	 fl_addto_menu(fd_form_main->menu_buffer, buffer[current_buffer]->filename);
	 buffer[current_buffer]->menu_position = fl_get_menu_maxitems(fd_form_main->menu_buffer);
      }
   }
}

   
/* callbacks for form form_main */
void FootCB(FL_OBJECT *ob, long data)
{
   if (buffer[current_buffer]->text) {
      SetMinibuffer("Inserting Footnote...","","");
      buffer[current_buffer]->screen->HideCursor();
      Update(-2);
      buffer[current_buffer]->text->InsertFootnoteEnvironement(LYX_FOOTNOTE);
      Update(1);
   }
}

void LayoutsCB(FL_OBJECT *ob, long data) {
   if (!buffer[current_buffer]->screen)
     return;
   buffer[current_buffer]->screen->HideCursor();
   Update(-2);
   int tmplayout = fl_get_choice(ob) - 1;
   
   if (current_layout != tmplayout) {
      current_layout = tmplayout;
      buffer[current_buffer]->text->SetLayout(current_layout);
   }
   Update(1);
}

void WorkAreaCB(FL_OBJECT *ob, long data)
{
  /* fill-in code for callback */
}

void ScrollCB(FL_OBJECT *ob, long data)
{
   current_scrollbar_value = (long)fl_get_slider_value(ob);
   if (current_scrollbar_value < 0)
      current_scrollbar_value = 0;
   ResetKeyboardMode();
   
   if (!buffer[current_buffer]->screen)
     return;
   
   buffer[current_buffer]->screen->Draw(current_scrollbar_value);
}


int RunLaTeX() {
   int n, n2, i;
   int a=0;
   int dot = 0;
   int path = 0;
   int errors = -1;
   if (buffer[current_buffer]->text) {
      ProhibitInput();

      SetMinibuffer("Running LaTeX...","","");   
      /* get LaTeX-Filename */
      
      /* find the end of the string */ 
      for (n=0; buffer[current_buffer]->filename[n]; n++);

      
      /* go back to the first dot */ 
      for (n2=n; n2 && buffer[current_buffer]->filename[n2]!='.'
	   && buffer[current_buffer]->filename[n2]!='/'; n2--);
      
      /* when you find a dot, use this as extension */ 
      if (buffer[current_buffer]->filename[n2]=='.')
      	dot = n2;

      char *s = new char[dot+10];
      char *s4 = new char[dot+10];
      
      for (path=dot; path && buffer[current_buffer]->filename[path]!='/';path--);
      if (buffer[current_buffer]->filename[path]=='/')
      	path++;
      
      for (i=0; i<path; i++)
      	s4[i] = buffer[current_buffer]->filename[i];
      s4[path] = '\0';
      
      for (i=path; i<dot; i++) 
      	s[i-path] = buffer[current_buffer]->filename[i];
      s[i-path] = '.';
      s[i+1-path] = 't';
      s[i+2-path] = 'e';
      s[i+3-path] = 'x';
      s[i+4-path] = '\0';
      dot = i;
      char *s2 = new char[n+100]; /* used to print command strings into */
      char *cpwd = new char[500];
      getcwd(cpwd, 500);

      
      /*Remove all errors */

      LyXParagraph *par = buffer[current_buffer]->parameters.paragraph;
      a = 0;
      while (par) {
	 a += par->AutoDeleteInsets();
	 par = par->next;
      }
      
      
          
      if (s4[0])
      	chdir(s4);
      delete (s4);
      
      if (!buffer[current_buffer]->dvi_current) {
	 
	 MakeLaTeXFile(s,
		       buffer[current_buffer]->parameters,
		       0);
	 
	 buffer[current_buffer]->dvi_current = 0;
	 

	}
      
	/* new stuff */ 
	sprintf(s2, "latex %s", s);

	system(s2);
	buffer[current_buffer]->dvi_current = 1;
   
      
      /* scanning errors */ 
      s[dot-path] = '.';
      s[dot+1-path] = 'l';
      s[dot+2-path] = 'o';
      s[dot+3-path] = 'g';
      s[dot+4-path] = '\0';


      SetMinibuffer("Scanning log-file ...","","");   

      errors = ScanLogFile(s);
      chdir(cpwd);
      delete(cpwd);
      
      SetMinibuffer("Done.","","");
      
      if (a || errors > 0)
      	 RedoCurrentBuffer();
      
      if (errors < 0) {
	 WriteAlert("LaTeX did not work!", "Missing log file:",
		    s);
	 buffer[current_buffer]->dvi_current = 0;
      }
      
      delete(s);
      delete(s2);

      AllowInput();
   }
   
   return errors;
}

/* 1 means preview, 0 means print */ 
void RunScript(char flag, char* command) {
   int n, n2, i;
   int ret = 0;
   int dot=0;
   int path=0;
   if (buffer[current_buffer]->text) {
      
      /* get DVI-Filename */
      
      /* find the end of the string */ 
      for (n=0; buffer[current_buffer]->filename[n]; n++);

      
      /* go back to the first dot */ 
      for (n2=n; n2 && buffer[current_buffer]->filename[n2]!='.'
	   && buffer[current_buffer]->filename[n2]!='/'; n2--);
      
      /* when you find a dot, use this as extension */ 
      if (buffer[current_buffer]->filename[n2]=='.')
      	dot = n2;

      char *s = new char[dot+10];
      char *s4 = new char[dot+10];
      
      for (path=dot; path && buffer[current_buffer]->filename[path]!='/';path--);
      if (buffer[current_buffer]->filename[path]=='/')
      	path++;
      
      for (i=0; i<path; i++)
      	s4[i] = buffer[current_buffer]->filename[i];
      s4[path] = '\0';
      
      for (i=path; i<dot; i++) 
      	s[i-path] = buffer[current_buffer]->filename[i];
      s[i-path] = '.';
      s[i+1-path] = 'd';
      s[i+2-path] = 'v';
      s[i+3-path] = 'i';
      s[i+4-path] = '\0';
      dot = i;
      char *s2 = new char[n+100]; /* used to print command strings into */
      char *cpwd = new char[500];
      getcwd(cpwd, 500);
      
      if (!buffer[current_buffer]->dvi_current)
      	ret = RunLaTeX();
      
      if (s4[0])
      	chdir(s4);
      delete (s4);
      
      if (!OpenAllFile(s)) {
	 if (flag)
	    WriteAlert("Error!", "Cannot preview: No LaTeX-Output (dvi)","");
	 else
	    WriteAlert("Error!", "Cannot print: No LaTeX-Output (dvi)","");
      }
      else {
	 CloseFile();
      }
      
      if (ret > 0) {
	 char *s3 = new char [50];
	 if (ret == 1)
	    sprintf(s3, "%d error detected",ret);
	 else
	    sprintf(s3, "%d errors detected",ret);
	 WriteAlert("LaTeX Error!", s3, "");
	 delete(s3);
      }

      
      if (flag==3) {
      SetMinibuffer("Executing command:", command,"");
      	 sprintf(s2, "%s ", command);
      }
      else {
	 SetMinibuffer("Executing command:", command, s);   

	 if (flag == 2)
      	    sprintf(s2, "%s %s ", command, s);
	 else
      	    sprintf(s2, "%s %s &", command, s);
      }
      
      system(s2); 
      
      chdir(cpwd);
      delete(cpwd);
      
      delete(s2);	
      
      delete(s);
   }
   
   
}

void MenuPreviewPS() {
   
   if (!buffer[current_buffer]->text)
      return;

   ProhibitInput();
   /* calculate ps filename */ 
   char* filename = buffer[current_buffer]->filename;
   
   int dot=0;
   int path = 0;
   int n=0;
   int n2=0;
   int i=0;
   
   /* find the end of the string */ 
   for (n=0; filename[n]; n++);
   
   /* go back to the first dot */ 
   for (n2=n; n2 && filename[n2]!='.'
	&& filename[n2]!='/'; n2--);
   
   /* when you find a dot, use this as extension */ 
   if (filename[n2]=='.')
      dot = n2;
   else
      dot = n;
   
   for (path=dot; path && filename[path]!='/';path--);
   if (filename[path]=='/')
      path++;
   

   char *ps = new char[dot+15];
   
   
   for (i=path; i<dot; i++) 
      ps[i-path] = filename[i];
   ps[i-path] = '.';
   ps[i+1-path] = 'p';
   ps[i+2-path] = 's';
   ps[i+3-path] = '_';
   ps[i+4-path] = 't';
   ps[i+5-path] = 'm';
   ps[i+6-path] = 'p';
   ps[i+7-path] = '\0';

   char* paper = "a4";
   
   switch (buffer[current_buffer]->parameters.papersize) {
    case PAPER_USLETTER:
      paper = "letter";
      break;
    case PAPER_A5PAPER:
      paper = "a5";
      break;
    case PAPER_B5PAPER:
      paper = "b5";
      break;
    case PAPER_EXECUTIVEPAPER:
      paper = "foolscap";
      break;
    case PAPER_LEGALPAPER:
      paper = "legal";
      break;
   }

   
   char* orientationflag = "";
   if (buffer[current_buffer]->parameters.orientation == ORIENTATION_LANDSCAPE)
      orientationflag = "-t landscape";
   
   char *command = new char[n + 100];
   
   sprintf(command, "dvips -o %s -t %s %s", ps, paper, orientationflag);
   
   RunScript(2, command);
   sprintf(command, "ghostview -swap %s &", ps);
   RunScript(3, command);
   AllowInput();
   delete(command);
   delete(ps);
}


void MenuPreview() {
   if (!buffer[current_buffer]->text)
      return;
   
   char* command = new char[50];
   
   char* paper = "a4";
   
   switch (buffer[current_buffer]->parameters.papersize) {
    case PAPER_USLETTER:
      paper = "us";
      break;
    case PAPER_A5PAPER:
      paper = "a5";
      break;
    case PAPER_B5PAPER:
      paper = "b5";
      break;
    case PAPER_EXECUTIVEPAPER:
      paper = "foolscap";
      break;
    case PAPER_LEGALPAPER:
      paper = "legal";
      break;
   }
   
   char* rotate ="";
   
   if (buffer[current_buffer]->parameters.orientation == ORIENTATION_LANDSCAPE)
      rotate ="r";
   
   sprintf(command, "xdvi -paper %s%s %s", paper, rotate, 
	   lyxrc.xdvi_extra_options);
   
   RunScript(1, command);
   
   delete(command);
}




void FileCB(FL_OBJECT *ob, long data) {
   switch (fl_get_menu(ob)) {
    case 1: MenuOpen(); break;
    case 2: MenuNew(); break;
    case 3: MenuWrite(current_buffer); break;
    case 4: MenuWriteAs(); break;
    case 5: MenuRunLaTeX(); break;
    case 6: MenuPreview(); break;
    case 7: MenuPreviewPS(); break;
    case 8: MenuPrint(); break;
    case 9: MenuMakeLaTeX(); break;
    case 10: InsertAsciiFile(0); break;
    case 11: InsertAsciiFile(1); break;
    case 12: CloseBuffer(); break;
    case 13: MenuQuit(); break;
   }
   AllowInput();
}

void AllFloats(char flag) {
   if (!buffer[current_buffer]->text)
      return;

   if (!flag && buffer[current_buffer]->text->cursor.par->footnoteflag != LYX_NO_FOOTNOTE) {
      OpenStuff();
   }
   else
      BeforeChange();
   LyXParagraph *par = buffer[current_buffer]->parameters.paragraph;
   while (par) {
      if (flag) {
	 if (par->footnoteflag == LYX_CLOSED_FOOTNOTE)
	    par->footnoteflag = LYX_OPEN_FOOTNOTE;
      }
      else  {
	 if (par->footnoteflag == LYX_OPEN_FOOTNOTE)
	    par->footnoteflag = LYX_CLOSED_FOOTNOTE;
      }
      par = par->next;
   }
   RedoCurrentBuffer();
}


   
void EditCB(FL_OBJECT *ob, long data) { 
   switch (fl_get_menu(ob)) {
    case 1: MenuLoadAutoSave(); break;
    case 2: AutoSave(); break;
    case 3: CutCB(ob,data); break;
    case 4: CopyCB(); break;
    case 5: PasteCB(ob,data); break;
    case 6: MenuSearch(); break;
    case 7: GotoError(); break;
    case 8: OpenStuff(); break;
    case 9: AllFloats(1); 
      SetMinibuffer("All floats are open!","",""); 
      break; /* open all floats */ 
    case 10: AllFloats(0); 
      SetMinibuffer("All floats are closed!","",""); 
      break; /* close all floats */ 
    case 11:
      fl_set_input(fd_form_screen->input_roman, roman_font_name);
      fl_set_input(fd_form_screen->input_sans, sans_font_name);
      fl_set_input(fd_form_screen->input_typewriter, typewriter_font_name);
      char* tmpstring = new char[50];
      sprintf(tmpstring,"%d", lyxrc.default_zoom);
      fl_set_input(fd_form_screen->intinput_size, tmpstring);
      delete(tmpstring);
      fl_show_form(fd_form_screen->form_screen,
		   FL_PLACE_CENTER | FL_FREE_SIZE, FL_FULLBORDER,
		   "Screen Options");
      break;
    case 12: ShowSpellChecker(buffer[current_buffer]->parameters.language); break;
    case 13:
      TocUpdateCB(ob, data);
      fl_show_form(fd_form_toc->form_toc,
		   FL_PLACE_CENTER | FL_FREE_SIZE, FL_FULLBORDER,
		   "Table Of Contents");
      break;
	 
   }
}


void MenuLayoutCharacter() {
   fl_show_form(fd_form_character->form_character,
		FL_PLACE_CENTER | FL_FREE_SIZE,FL_FULLBORDER,
		"Character Layout");
}
void MenuLayoutParagraph() {
      if (buffer[current_buffer]->text) {
	 fl_set_input(fd_form_paragraph->input_labelwidth,
		      buffer[current_buffer]->text->cursor.par->GetLabelWidthString());
	 fl_set_button(fd_form_paragraph->radio_align_right, 0);
	 fl_set_button(fd_form_paragraph->radio_align_left, 0);
	 fl_set_button(fd_form_paragraph->radio_align_center, 0);
	 fl_set_button(fd_form_paragraph->radio_align_block, 0);
	 int align = buffer[current_buffer]->text->cursor.par->GetAlign();
	 if (align == LYX_ALIGN_LAYOUT)
	   align = style(buffer[current_buffer]->parameters.textclass, buffer[current_buffer]->text->cursor.par->GetLayout())->align;
	 switch (align) {
	  case LYX_ALIGN_RIGHT: fl_set_button(fd_form_paragraph->radio_align_right, 1); break;
	  case LYX_ALIGN_LEFT: fl_set_button(fd_form_paragraph->radio_align_left, 1); break;
	  case LYX_ALIGN_CENTER: fl_set_button(fd_form_paragraph->radio_align_center, 1); break;
	  default: fl_set_button(fd_form_paragraph->radio_align_block, 1); break;
	 }
	 fl_set_button(fd_form_paragraph->check_lines_top,
		       buffer[current_buffer]->text->cursor.par->FirstPhysicalPar()->line_top);
	 fl_set_button(fd_form_paragraph->check_lines_bottom,
		       buffer[current_buffer]->text->cursor.par->FirstPhysicalPar()->line_bottom);
	 fl_set_button(fd_form_paragraph->check_pagebreaks_top,
		       buffer[current_buffer]->text->cursor.par->FirstPhysicalPar()->pagebreak_top);
	 fl_set_button(fd_form_paragraph->check_pagebreaks_bottom,
		       buffer[current_buffer]->text->cursor.par->FirstPhysicalPar()->pagebreak_bottom);
	 fl_set_button(fd_form_paragraph->check_noindent,
		       buffer[current_buffer]->text->cursor.par->FirstPhysicalPar()->noindent);
	 fl_set_button(fd_form_paragraph->check_fill_top,
		       buffer[current_buffer]->text->cursor.par->FirstPhysicalPar()->fill_top);
	 fl_set_button(fd_form_paragraph->check_fill_bottom,
		       buffer[current_buffer]->text->cursor.par->FirstPhysicalPar()->fill_bottom);
	 fl_set_slider_value(fd_form_paragraph->slider_spaces_top,
		       buffer[current_buffer]->text->cursor.par->FirstPhysicalPar()->added_space_top);
	 fl_set_slider_value(fd_form_paragraph->slider_spaces_bottom,
		       buffer[current_buffer]->text->cursor.par->FirstPhysicalPar()->added_space_bottom);
	 fl_set_button(fd_form_paragraph->check_noindent,
		       buffer[current_buffer]->text->cursor.par->FirstPhysicalPar()->noindent);
      }
      fl_show_form(fd_form_paragraph->form_paragraph,
		   FL_PLACE_CENTER | FL_FREE_SIZE,FL_FULLBORDER,
		   "Paragraph Layout");
}

void MenuLayoutDocument() {
   fl_set_choice_text(fd_form_document->choice_class, 
		      NameOfClass(buffer[current_buffer]->parameters.textclass));
   fl_set_choice_text(fd_form_document->choice_language, 
		      buffer[current_buffer]->parameters.language);
   fl_set_choice_text(fd_form_document->choice_fonts, 
		      buffer[current_buffer]->parameters.fonts);
   fl_set_choice_text(fd_form_document->choice_inputenc, 
		      buffer[current_buffer]->parameters.inputenc);
   fl_set_choice(fd_form_document->choice_fontsize, 
		 buffer[current_buffer]->parameters.fontsize - 9);
   fl_set_choice(fd_form_document->choice_papersize,
		 buffer[current_buffer]->parameters.papersize + 1);
   fl_set_choice(fd_form_document->choice_pagestyle,
		 buffer[current_buffer]->parameters.pagestyle + 1);
   fl_set_button(fd_form_document->check_no_epsfig,
		 buffer[current_buffer]->parameters.no_epsfig);
   fl_set_button(fd_form_document->radio_indent, 0);
   fl_set_button(fd_form_document->radio_skip, 0);
   if (buffer[current_buffer]->parameters.paragraph_separation == LYX_PARSEP_INDENT)
      fl_set_button(fd_form_document->radio_indent, 1);
   else
      fl_set_button(fd_form_document->radio_skip, 1);
   fl_set_button(fd_form_document->radio_portrait, 0);
   fl_set_button(fd_form_document->radio_landscape, 0);
   if (buffer[current_buffer]->parameters.orientation == ORIENTATION_LANDSCAPE)
      fl_set_button(fd_form_document->radio_landscape, 1);
   else
      fl_set_button(fd_form_document->radio_portrait, 1);
   fl_set_button(fd_form_document->radio_sides_one, 0);
   fl_set_button(fd_form_document->radio_sides_two, 0);
   if (buffer[current_buffer]->parameters.sides == 2)
      fl_set_button(fd_form_document->radio_sides_two, 1);
   else
      fl_set_button(fd_form_document->radio_sides_one, 1);
   fl_set_button(fd_form_document->radio_columns_one, 0);
   fl_set_button(fd_form_document->radio_columns_two, 0);
   if (buffer[current_buffer]->parameters.columns == 2)
      fl_set_button(fd_form_document->radio_columns_two, 1);
   else
      fl_set_button(fd_form_document->radio_columns_one, 1);
   fl_set_slider_value(fd_form_document->slider_baselinestretch, 
		       buffer[current_buffer]->parameters.baseline);
   fl_set_slider_value(fd_form_document->slider_secnumdepth, 
		       buffer[current_buffer]->parameters.secnumdepth);
   fl_set_slider_value(fd_form_document->slider_tocdepth, 
		       buffer[current_buffer]->parameters.tocdepth);
   fl_set_input(fd_form_document->input_extra,
		buffer[current_buffer]->parameters.options);
   
   fl_show_form(fd_form_document->form_document,
		FL_PLACE_CENTER | FL_FREE_SIZE,FL_FULLBORDER,
		"Document Layout");
}

void MenuLayoutQuotes() {
     if (buffer[current_buffer]->text){
       fl_set_choice(fd_form_quotes->choice_quotes_language,
		     buffer[current_buffer]->parameters.quotes_language + 1);
       fl_set_button(fd_form_quotes->radio_single, 0);
       fl_set_button(fd_form_quotes->radio_double, 0);
       
       if (buffer[current_buffer]->parameters.quotes_times == 1)
	 fl_set_button(fd_form_quotes->radio_single, 1);
       else
	 fl_set_button(fd_form_quotes->radio_double, 1);
     }
     fl_show_form(fd_form_quotes->form_quotes,
			FL_PLACE_CENTER | FL_FREE_SIZE,FL_FULLBORDER,
			"Quotes");
}

void MenuLayoutPreamble() {
   if (buffer[current_buffer]->text){
      fl_set_input(fd_form_preamble->input_preamble,
		     buffer[current_buffer]->parameters.preamble);
   }
   fl_show_form(fd_form_preamble->form_preamble,
		FL_PLACE_CENTER | FL_FREE_SIZE,FL_FULLBORDER,
		"LaTeX Preamble");
}

void LayoutCB(FL_OBJECT *ob, long data){
   switch (fl_get_menu(ob)) {
    case 1: MenuLayoutCharacter(); break;
    case 2: MenuLayoutParagraph(); break;
    case 3: MenuLayoutDocument(); break;
    case 4: MenuLayoutQuotes(); break;
    case 5: EmphCB(ob, data); break;
    case 6: NounCB(ob, data); break;
    case 7: TexCB(ob, data); break;
    case 8: FormulaCB(ob, data); break;
    case 9: DepthCB(ob, data); break;
    case 10: MenuLayoutPreamble(); break;
   }
   
}


/* only usable from special cb!!! */ 
void InsertLaTeXTable(char* table) {
   buffer[current_buffer]->text->BreakParagraph();
   Update(-1);
   
   if (buffer[current_buffer]->text->cursor.par->Last()) {
      buffer[current_buffer]->text->CursorLeft();
      
      buffer[current_buffer]->text->BreakParagraph();
      Update(-1);
   }
   
   buffer[current_buffer]->text->SetLayout(NumberOfLayout(buffer[current_buffer]->parameters.textclass, "LaTeX"));
   
   buffer[current_buffer]->text->SetParagraph(0, 0,
						0, 0,
						0, 0,
						0, 0,
						LYX_ALIGN_LAYOUT, 
						NULL,
						0);
   Update(-1);
   
   buffer[current_buffer]->text->current_font.latex = LYX_NO_LATEX;
   
   InsetLatexDel *new_inset = new InsetLatexDel(table);
   buffer[current_buffer]->text->InsertInset(new_inset);
   Update(-1);
   
   buffer[current_buffer]->text->CursorRight();
}


void SpecialCB(FL_OBJECT *ob, long data) {
   if (!buffer[current_buffer]->text)
     return;
   buffer[current_buffer]->screen->HideCursor();
   BeforeChange();
   Update(-2);
   switch (fl_get_menu(ob)) {
    case 1: FigureCB(ob, data); break;
    case 2: TableCB(ob, data); break;
    case 3:
      buffer[current_buffer]->text->InsertFootnoteEnvironement(LYX_FIG);
      break;
    case 4:
      buffer[current_buffer]->text->InsertFootnoteEnvironement(LYX_TAB);
      break;
    case 5: FootCB(ob, data); break;
    case 6: MarginCB(ob, data); break;
    case 7: MeltCB(ob, data); break;
    case 8: InsertLaTeXTable("\\tableofcontents"); break;
    case 9: InsertLaTeXTable("\\listoffigures"); break;
    case 10: InsertLaTeXTable("\\listoftables"); break;
    case 11: HFill(); break;
    case 12: HyphenationPoint(); break; 
    case 13: ProtectedBlank(); break; 
    case 14: Newline(); break; 
   }
   Update(1);
}


void BufferCB(FL_OBJECT *ob, long data){
   
   /* find the buffer-number in the buffer list  */ 
   int i;
   int pos = fl_get_menu(ob);
   for (i=0; i < NUMBER_OF_BUFFERS &&
	buffer[i]->menu_position != pos;
	i++);
   
   if (i>=NUMBER_OF_BUFFERS)
     return;

   old_current_buffer = current_buffer;
   current_buffer = i;
   ResizeCurrentBufferPseudoExpose();
}

void HelpCB(FL_OBJECT *ob, long data) {
   switch (fl_get_menu(ob)) {
    case 1: 
      ProhibitInput();
      fl_show_message("","LyX is Copyright (C) 1995 by Matthias Ettrich\n(ettrich@informatik.uni-tuebingen.de)\n\nThis program is free software; you can redistribute it and/or modify it under the\nterms of the GNU General Public License as published by the Free Software\nFoundation; either version 2 of the License, or (at your option) any later version.","");
      AllowInput();
      break;
    case 2: 
      ProhibitInput();
      fl_show_message("","LyX is distributed in the hope that it will be useful, but WITHOUT ANY\nWARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS\n FOR A PARTICULAR PURPOSE. See the GNU General Public License for more\ndetails. You should have received a copy of the GNU General Public License\nalong with this program; if not, write to the Free Software Foundation,\n Inc., 675 Mass Ave, Cambridge, MA 02139, USA.","");
      AllowInput();
      break;
    case 3: 
      ProhibitInput();
      fl_show_message("","This is LyX 0.7p4d3 BETA of Tue, Nov 2 1995","");
      AllowInput();
      break;
   }
}

void OpenStuff() {
   if (buffer[current_buffer]->text)  {
     SetMinibuffer("Open/Close...","","");
      buffer[current_buffer]->screen->HideCursor();
      BeforeChange();
      Update(-2);
      buffer[current_buffer]->text->ToggleFootnote();
      Update(0);
   }
}

void HyphenationPoint() {
   if (buffer[current_buffer]->text)  {
      buffer[current_buffer]->screen->HideCursor();
      Update(-2);
      InsetLatexDel *new_inset = new InsetLatexDel("\\-");
      buffer[current_buffer]->text->InsertInset(new_inset);
      Update(0);
   }
}
void Newline() {
   if (buffer[current_buffer]->text)  {
      buffer[current_buffer]->screen->HideCursor();
      Update(-2);
      buffer[current_buffer]->text->InsertChar(LYX_META_NEWLINE);
      Update(0);
   }
}
void ProtectedBlank() {
   if (buffer[current_buffer]->text)  {
      buffer[current_buffer]->screen->HideCursor();
      Update(-2);
      buffer[current_buffer]->text->InsertChar(LYX_META_PROTECTED_SEPARATOR);
      Update(0);
   }
}
void HFill() {
   if (buffer[current_buffer]->text)  {
      buffer[current_buffer]->screen->HideCursor();
      Update(-2);
      buffer[current_buffer]->text->InsertChar(LYX_META_HFILL);
      Update(0);
   }
}


void EmphCB(FL_OBJECT *ob, long data)
{   
   if (buffer[current_buffer]->text) { 
      SetMinibuffer("Toggle Emphasize!","","");
      buffer[current_buffer]->screen->HideCursor();
      Update(-2);
      buffer[current_buffer]->text->ToggleEmphasize();
      Update(1);
   }
}

void NounCB(FL_OBJECT *ob, long data)
{
   if (buffer[current_buffer]->text) {
      SetMinibuffer("Toggle Noun!","","");
      buffer[current_buffer]->screen->HideCursor();
      Update(-2);
      buffer[current_buffer]->text->ToggleNoun();
      Update(1);
   }
}

void MarginCB(FL_OBJECT *ob, long data)
{
   if (buffer[current_buffer]->text) {
      SetMinibuffer("Inserting marginpar...","","");
      buffer[current_buffer]->screen->HideCursor();
      Update(-2);
      buffer[current_buffer]->text->InsertFootnoteEnvironement(LYX_MARGIN);
      Update(1);
   }
}

void FigureCB(FL_OBJECT *ob, long data) {
   fl_show_form(fd_form_figure->form_figure,
		FL_PLACE_CENTER | FL_FREE_SIZE, FL_FULLBORDER,
		"Figure");
}



void TableCB(FL_OBJECT *ob, long data){
   fl_show_form(fd_form_table->form_table,
		FL_PLACE_CENTER | FL_FREE_SIZE, FL_FULLBORDER,
		"Table");
}


void CopyCB() {
   if (buffer[current_buffer]->text) {
      buffer[current_buffer]->screen->HideCursor();
      Update(-2);
      buffer[current_buffer]->text->CutSelection();
      /* paste */ 
      buffer[current_buffer]->text->PasteSelection();
      Update(1);
      SetMinibuffer("Copy!","","");
   }
}

void CutCB(FL_OBJECT *ob, long data)
{
   if (buffer[current_buffer]->text) {
      buffer[current_buffer]->screen->HideCursor();
      Update(-2);
      buffer[current_buffer]->text->CutSelection();
      Update(1);
      SetMinibuffer("Cut!","","");
   }
}

void PasteCB(FL_OBJECT *ob, long data)
{
   if (buffer[current_buffer]->text) {
      SetMinibuffer("Paste!","","");
      buffer[current_buffer]->screen->HideCursor();
      /* clear the selection */ 
      buffer[current_buffer]->screen->ToggleSelection();
      buffer[current_buffer]->text->ClearSelection();
      Update(-2);
      
      /* paste */ 
      buffer[current_buffer]->text->PasteSelection();
      Update(1);
   }
}

void MeltCB(FL_OBJECT *ob, long data)
{
   if (buffer[current_buffer]->text) {
      SetMinibuffer("Melt!","","");
      buffer[current_buffer]->screen->HideCursor();
      BeforeChange();
      Update(-2);
      buffer[current_buffer]->text->MeltFootnoteEnvironement();
      Update(1);
   }
}

void TexCB(FL_OBJECT *ob, long data)
{
   if (buffer[current_buffer]->text) {
      SetMinibuffer("Toggle TeX!","","");
      buffer[current_buffer]->screen->HideCursor();
      Update(-2);
      buffer[current_buffer]->text->ToggleLaTeX();
      Update(1);
   }
}

void FormulaCB(FL_OBJECT *ob, long data)
{
   if (buffer[current_buffer]->text) {
      SetMinibuffer("Toggle Math!","","");
      buffer[current_buffer]->screen->HideCursor();
      Update(-2);
      buffer[current_buffer]->text->ToggleFormLaTeX();
      Update(1);
   }
}

void MathCB(FL_OBJECT *ob, long data){
   InsetFormulaLatex *new_inset = NULL;
 if (buffer[current_buffer]->text)  {
    buffer[current_buffer]->screen->HideCursor();
    BeforeChange();
    Update(-2);
    buffer[current_buffer]->text->current_font.latex = LYX_FORMULA_LATEX;
    
    switch (fl_get_menu(ob)) {
     case 1: /* frac */
       new_inset = new InsetFormulaLatex("\\frac{");
       buffer[current_buffer]->text->InsertInset(new_inset);
       SmallUpdate(-1);
       new_inset = new InsetFormulaLatex("}{");
       buffer[current_buffer]->text->InsertInset(new_inset);
       SmallUpdate(-1);
       new_inset = new InsetFormulaLatex("}");
       buffer[current_buffer]->text->InsertInset(new_inset);
       Update(-1);
       buffer[current_buffer]->text->CursorLeft();
       buffer[current_buffer]->text->CursorLeft();
       break;
     case 2: /* sqrt */
       new_inset = new InsetFormulaLatex("\\sqrt[");
       buffer[current_buffer]->text->InsertInset(new_inset);
       SmallUpdate(-1);
       new_inset = new InsetFormulaLatex("]{");
       buffer[current_buffer]->text->InsertInset(new_inset);
       SmallUpdate(-1);
       new_inset = new InsetFormulaLatex("}");
       buffer[current_buffer]->text->InsertInset(new_inset);
       Update(-1);
       buffer[current_buffer]->text->CursorLeft();
       break;
     case 3: /* Exponent  */ 
       new_inset = new InsetFormulaLatex("^{");
       buffer[current_buffer]->text->InsertInset(new_inset);
       SmallUpdate(-1);
       new_inset = new InsetFormulaLatex("}");
       buffer[current_buffer]->text->InsertInset(new_inset);
       Update(-1);
       buffer[current_buffer]->text->CursorLeft();
       break;
     case 4: /* Index  */
       new_inset = new InsetFormulaLatex("_{");
       buffer[current_buffer]->text->InsertInset(new_inset);
       SmallUpdate(-1);
       new_inset = new InsetFormulaLatex("}");
       buffer[current_buffer]->text->InsertInset(new_inset);
       Update(-1);
       buffer[current_buffer]->text->CursorLeft();
       break;
     case 5: /* sum */ 
       new_inset = new InsetFormulaLatex("\\sum_{");
       buffer[current_buffer]->text->InsertInset(new_inset);
       SmallUpdate(-1);
       new_inset = new InsetFormulaLatex("}^{");
       buffer[current_buffer]->text->InsertInset(new_inset);
       SmallUpdate(-1);
       new_inset = new InsetFormulaLatex("}");
       buffer[current_buffer]->text->InsertInset(new_inset);
       Update(-1);
       buffer[current_buffer]->text->CursorLeft();
       buffer[current_buffer]->text->CursorLeft();
       break;
     case 6: /* int */
       new_inset = new InsetFormulaLatex("\\int_{");
       buffer[current_buffer]->text->InsertInset(new_inset);
       SmallUpdate(-1);
       new_inset = new InsetFormulaLatex("}^{");
       buffer[current_buffer]->text->InsertInset(new_inset);
       SmallUpdate(-1);
       new_inset = new InsetFormulaLatex("}");
       buffer[current_buffer]->text->InsertInset(new_inset);
       Update(-1);
       buffer[current_buffer]->text->CursorLeft();
       buffer[current_buffer]->text->CursorLeft();
       break;
    }
    Update(0);
 }
}

void DepthCB(FL_OBJECT *ob, long data)
{
   if (buffer[current_buffer]->text) {
      buffer[current_buffer]->screen->HideCursor();
      Update(-2);
      buffer[current_buffer]->text->IncDepth();
      Update(1);
      SetMinibuffer("changed environment depth (in possible range, maybe not)", "", "");
   }
}

void FreeCB(FL_OBJECT *ob, long data)
{
  SetMinibuffer("Toggle User style","","");

  LyXFont font = { 
    LYX_NO_FONT_CHANGE,
    LYX_NO_FONT_CHANGE,
    LYX_NO_FONT_CHANGE,
    LYX_NO_FONT_CHANGE,
    LYX_NO_FONT_CHANGE
    };
  
   int pos;
  
  pos = fl_get_choice(fd_form_character->choice_family);
  if (pos > 1)
    font.family = pos - 2;
  
  pos = fl_get_choice(fd_form_character->choice_series);  if (pos > 1)
    font.series = pos - 2;
  
  pos = fl_get_choice(fd_form_character->choice_shape);  if (pos > 1)
    font.shape = pos - 2;
  
  pos = fl_get_choice(fd_form_character->choice_size);  if (pos > 1)
    font.size = pos - 1; /* heres a -1 because of the
			  * unaccesible tiny font, that I need
			  * for smallcaps in small font */
  
  if (buffer[current_buffer]->text) { 
     buffer[current_buffer]->text->ToggleFree(font);
      Update(1);
    }
}



/* callbacks for form form_title */
void TimerCB(FL_OBJECT *ob, long data)
{
   fl_hide_form(fd_form_title->form_title);
   fl_free_form(fd_form_title->form_title);
   
}

void UpdateTimerCB(FL_OBJECT *ob, long data) {
   if (!buffer[current_buffer]->screen || !updatetimer)
     return;
   buffer[current_buffer]->screen->HideCursor();
   Update(-2);
   /* this update can happen, even when the work area has lost
    * the focus. So supress the cursor in that case */ 
   updatetimer = 0;
}

void AutosaveTimerCB(FL_OBJECT *ob, long data) {
   AutoSave();
}



/* callbacks for form form_paragraph */
void ParagraphApplyCB(FL_OBJECT *ob, long data)
{
   if (!buffer[current_buffer]->text)
     return;
   char line_top, line_bottom;
   char pagebreak_top, pagebreak_bottom;
   float space_top, space_bottom;
   char fill_top, fill_bottom;
   char align;
   const char* labelwidthstring;
   char noindent;
   
   line_top = fl_get_button(fd_form_paragraph->check_lines_top);
   line_bottom = fl_get_button(fd_form_paragraph->check_lines_bottom);
   pagebreak_top = fl_get_button(fd_form_paragraph->check_pagebreaks_top);
   pagebreak_bottom = fl_get_button(fd_form_paragraph->check_pagebreaks_bottom);
   space_top = fl_get_slider_value(fd_form_paragraph->slider_spaces_top);
   space_bottom = fl_get_slider_value(fd_form_paragraph->slider_spaces_bottom);
   fill_top = fl_get_button(fd_form_paragraph->check_fill_top);
   fill_bottom = fl_get_button(fd_form_paragraph->check_fill_bottom);
   
   if (fl_get_button(fd_form_paragraph->radio_align_left))
     align = LYX_ALIGN_LEFT;
   else if (fl_get_button(fd_form_paragraph->radio_align_right))
     align = LYX_ALIGN_RIGHT;
   else if (fl_get_button(fd_form_paragraph->radio_align_center))
     align = LYX_ALIGN_CENTER;
   else 
     align = LYX_ALIGN_BLOCK;
   
   labelwidthstring = fl_get_input(fd_form_paragraph->input_labelwidth);
   noindent = fl_get_button(fd_form_paragraph->check_noindent);
   
   buffer[current_buffer]->text->SetParagraph(line_top, line_bottom,
				      pagebreak_top, pagebreak_bottom,
				      space_top, space_bottom,
				      fill_top, fill_bottom,
				      align, 
				      labelwidthstring,
				      noindent);
   Update(1);
   SetMinibuffer("user defined paragraph layout set","","");
}

void ParagraphCancelCB(FL_OBJECT *ob, long data)
{
   fl_hide_form(fd_form_paragraph->form_paragraph);
}

void ParagraphOKCB(FL_OBJECT *ob, long data)
{
   ParagraphApplyCB(ob, data);
   ParagraphCancelCB(ob, data);
}


/* callbacks for form form_search */
void SearchCancelCB(FL_OBJECT *ob, long data)
{
   fl_hide_form(fd_form_search->form_search);
}

void SearchReplaceCB(FL_OBJECT *ob, long data){
   if (!buffer[current_buffer]->screen)
     return;
   char* replacestring = (char *) fl_get_input(fd_form_search->input_replace);
   buffer[current_buffer]->screen->HideCursor();
   Update(-2);
   /* clear the selection (if there is any) */ 
   buffer[current_buffer]->screen->ToggleSelection();
   buffer[current_buffer]->text->ReplaceSelectionWithString(replacestring);
   
   buffer[current_buffer]->text->SetSelectionOverString(replacestring);
   Update(1);
}

void SearchForwardCB(FL_OBJECT *ob, long data){
   if (!buffer[current_buffer]->screen)
     return;
   
   buffer[current_buffer]->screen->HideCursor();
   Update(-2);
   char* searchstring = (char*) fl_get_input(fd_form_search->input_search);
   
   if (buffer[current_buffer]->text->selection) 
     buffer[current_buffer]->text->cursor = buffer[current_buffer]->text->sel_end_cursor;
   
   if (!searchstring[0] || buffer[current_buffer]->text->SearchForward(searchstring)) {
      Update(-2);
      /* clear the selection (if there is any) */ 
      buffer[current_buffer]->screen->ToggleSelection();
      buffer[current_buffer]->text->ClearSelection();
      /* set the new selection */
      buffer[current_buffer]->text->SetSelectionOverString(searchstring);
      buffer[current_buffer]->screen->ToggleSelection();
   }
   else {
      LyXBell();
   }
   if (work_area_active)
     buffer[current_buffer]->screen->ShowCursor();
}

void SearchBackwardCB(FL_OBJECT *ob, long data){
   if (!buffer[current_buffer]->screen)
     return;
   
   char* searchstring = (char *) fl_get_input(fd_form_search->input_search);
   
   buffer[current_buffer]->screen->HideCursor();
   Update(-2);

   if (buffer[current_buffer]->text->selection) 
     buffer[current_buffer]->text->cursor = buffer[current_buffer]->text->sel_start_cursor;

   if (!searchstring[0] || buffer[current_buffer]->text->SearchBackward(searchstring)) {
      Update(-2);
      /* clear the selection (if there is any) */ 
      buffer[current_buffer]->screen->ToggleSelection();
      buffer[current_buffer]->text->ClearSelection();
      /* set the new selection */
      buffer[current_buffer]->text->SetSelectionOverString(searchstring);
      buffer[current_buffer]->screen->ToggleSelection();
   }
   else {
      LyXBell();
   }
   if (work_area_active)
     buffer[current_buffer]->screen->ShowCursor();
}



/* callbacks for form form_character */

void CharacterApplyCB(FL_OBJECT *ob, long data) {
  

  LyXFont font = { 
    LYX_NO_FONT_CHANGE,
    LYX_NO_FONT_CHANGE,
    LYX_NO_FONT_CHANGE,
    LYX_NO_FONT_CHANGE,
    LYX_NO_FONT_CHANGE
    };
  
   int pos;
  
  pos = fl_get_choice(fd_form_character->choice_family);
  if (pos > 1)
    font.family = pos - 2;
  
  pos = fl_get_choice(fd_form_character->choice_series);  if (pos > 1)
    font.series = pos - 2;
  
  pos = fl_get_choice(fd_form_character->choice_shape);  if (pos > 1)
    font.shape = pos - 2;
  
  pos = fl_get_choice(fd_form_character->choice_size);  if (pos > 1)
    font.size = pos - 1; /* heres a -1 because of the
			  * unaccesible tiny font, that I need
			  * for smallcaps in small font */
  
   if (buffer[current_buffer]->text) { 
      buffer[current_buffer]->text->SetFont(font);
      Update(1);
      SetMinibuffer("user defined character layout set","","");
   }
}

void CharacterCancelCB(FL_OBJECT *ob, long data)
{
   fl_hide_form(fd_form_character->form_character);
}
void CharacterOKCB(FL_OBJECT *ob, long data){
   CharacterApplyCB(ob,data);
   CharacterCancelCB(ob,data);
}



/* callbacks for form form_document */

void ChoiceClassCB(FL_OBJECT *ob, long data) {
   ProhibitInput();
   if (fl_show_question("Should I set the defaults of textclass",
			fl_get_choice_text(ob),
			"for Columns, Sides, Pagestyle and Extra Options?")) {
      
      fl_set_choice(fd_form_document->choice_pagestyle, 
		    TextClass(fl_get_choice(ob) - 1)->pagestyle + 1);
      fl_set_button(fd_form_document->radio_sides_one, 0);
      fl_set_button(fd_form_document->radio_sides_two, 0);
      if (TextClass(fl_get_choice(ob) - 1)->sides == 2)
      	 fl_set_button(fd_form_document->radio_sides_two, 1);
      else
      	 fl_set_button(fd_form_document->radio_sides_one, 1);
      fl_set_button(fd_form_document->radio_columns_one, 0);
      fl_set_button(fd_form_document->radio_columns_two, 0);
      if (TextClass(fl_get_choice(ob) - 1)->columns == 2)
	   fl_set_button(fd_form_document->radio_columns_two, 1);
	 else
	   fl_set_button(fd_form_document->radio_columns_one, 1);
      fl_set_input(fd_form_document->input_extra,
		   TextClass(fl_get_choice(ob) - 1)->options);
   }
   AllowInput();
}




void DocumentApplyCB(FL_OBJECT *ob, long data){
   char redo = 0;
   delete (buffer[current_buffer]->parameters.language);
   delete(buffer[current_buffer]->parameters.fonts);
   delete(buffer[current_buffer]->parameters.inputenc);
   buffer[current_buffer]->parameters.language = StringCopy(fl_get_choice_text(fd_form_document->choice_language));
   buffer[current_buffer]->parameters.fonts = StringCopy(fl_get_choice_text(fd_form_document->choice_fonts));
   buffer[current_buffer]->parameters.inputenc = StringCopy(fl_get_choice_text(fd_form_document->choice_inputenc));
   buffer[current_buffer]->parameters.fontsize = 9 + fl_get_choice(fd_form_document->choice_fontsize);
   buffer[current_buffer]->parameters.papersize = -1 + fl_get_choice(fd_form_document->choice_papersize);
   buffer[current_buffer]->parameters.pagestyle = -1 + fl_get_choice(fd_form_document->choice_pagestyle);
   buffer[current_buffer]->parameters.no_epsfig = fl_get_button(fd_form_document->check_no_epsfig);
   
   if (!buffer[current_buffer]->text)
     return;

   char new_class = fl_get_choice(fd_form_document->choice_class) - 1;
   if (buffer[current_buffer]->parameters.textclass != new_class) {
      redo = 1;
      SetMinibuffer("Converting document to new textclass... ","","");
      int ret = buffer[current_buffer]->text->SwitchLayoutsBetweenClasses(buffer[current_buffer]->parameters.textclass,
									  new_class,
									  buffer[current_buffer]->parameters.paragraph);
      
      if (ret){
	 char *s = new char[50];
	 sprintf(s, "%d paragraph(s) couldn't be converted", ret);
	 WriteAlert("Conversion Errors!",s,"into choosen textclass");
	 delete(s);
      }
      
      buffer[current_buffer]->parameters.textclass = new_class;
   }

   char tmpsep = buffer[current_buffer]->parameters.paragraph_separation;
   if (fl_get_button(fd_form_document->radio_indent))
     buffer[current_buffer]->parameters.paragraph_separation = LYX_PARSEP_INDENT;
   else
      buffer[current_buffer]->parameters.paragraph_separation = LYX_PARSEP_SKIP;
   if (tmpsep != buffer[current_buffer]->parameters.paragraph_separation)
      redo = 1;
   
   if (fl_get_button(fd_form_document->radio_landscape))
      buffer[current_buffer]->parameters.orientation = ORIENTATION_LANDSCAPE;
   else
      buffer[current_buffer]->parameters.orientation = ORIENTATION_PORTRAIT;
   if (fl_get_button(fd_form_document->radio_columns_two))
      buffer[current_buffer]->parameters.columns = 2;
   else
      buffer[current_buffer]->parameters.columns = 1;
   if (fl_get_button(fd_form_document->radio_sides_two))
      buffer[current_buffer]->parameters.sides = 2;
   else
      buffer[current_buffer]->parameters.sides = 1;

   if (buffer[current_buffer]->parameters.baseline != fl_get_slider_value(fd_form_document->slider_baselinestretch))
      redo = 1;
   buffer[current_buffer]->parameters.baseline = fl_get_slider_value(fd_form_document->slider_baselinestretch);

   
   double tmpdouble = fl_get_slider_value(fd_form_document->slider_secnumdepth);
   signed char tmpchar = 0;
   
   if (tmpdouble >= 0)
      tmpchar = (signed char) (tmpdouble + 0.5);
   else
      tmpchar = (signed char) (tmpdouble - 0.5);
      
   
   if (buffer[current_buffer]->parameters.secnumdepth != tmpchar)
      redo = 1;
   buffer[current_buffer]->parameters.secnumdepth = tmpchar;
   
   tmpdouble = fl_get_slider_value(fd_form_document->slider_tocdepth);
   
   if (tmpdouble >= 0)
      tmpchar = (signed char) (tmpdouble + 0.5);
   else
      tmpchar = (signed char) (tmpdouble - 0.5);
   buffer[current_buffer]->parameters.tocdepth = tmpchar;
      
   if (redo)
      RedoCurrentBuffer();
   
   SetMinibuffer("user defined document layout set","","");
   buffer[current_buffer]->lyx_current = 0;
   buffer[current_buffer]->dvi_current = 0;
   buffer[current_buffer]->bak_current = 0;

   if (buffer[current_buffer]->parameters.options)
      delete(buffer[current_buffer]->parameters.options);
   buffer[current_buffer]->parameters.options = StringCopy(fl_get_input(fd_form_document->input_extra));
   
}

void DocumentCancelCB(FL_OBJECT *ob, long data)
{
   fl_hide_form(fd_form_document->form_document);
}


void DocumentOKCB(FL_OBJECT *ob, long data)
{
   DocumentCancelCB(ob,data);
   DocumentApplyCB(ob,data);
}







void GotoError() {
   if (!buffer[current_buffer]->screen)
     return;
   buffer[current_buffer]->screen->HideCursor();
   BeforeChange();
   Update(-2);
   LyXCursor tmp;
   
   if (!buffer[current_buffer]->text->GotoNextError()) {
      if (buffer[current_buffer]->text->cursor.pos 
	  || buffer[current_buffer]->text->cursor.par != buffer[current_buffer]->text->FirstParagraph()) {
	     tmp = buffer[current_buffer]->text->cursor;
	     buffer[current_buffer]->text->cursor.par = buffer[current_buffer]->text->FirstParagraph();
	     buffer[current_buffer]->text->cursor.pos = 0;
	     if (!buffer[current_buffer]->text->GotoNextError()) {
		buffer[current_buffer]->text->cursor = tmp;
		SetMinibuffer("No more errors!","","");
		LyXBell();
	     }
	  }
      else {
	 SetMinibuffer("No more errors!","","");
	 LyXBell();
      }
   }
   Update(0);
   buffer[current_buffer]->text->sel_cursor = buffer[current_buffer]->text->cursor;
}


void InsertCorrectQuote() {
   Inset *new_inset;
   char *s = new char[4];

   switch (buffer[current_buffer]->parameters.quotes_language) {
    case GERMAN:
      s[0]='g';
      break;
    case FRENCH:
      s[0]='f';
      break;
    default:
      s[0]='e';
      break;
   }
   
   
   /* first decide wether left or right */ 
   
   
   char c;
   if  (buffer[current_buffer]->text->cursor.pos )
     c = buffer[current_buffer]->text->cursor.par->GetChar(buffer[current_buffer]->text->cursor.pos -1);
   else c = ' ';
   
   if (c != ' ' &&
       c != '(' &&
       c != '{' &&
       c != '[' &&
       c != '-' &&
       c != ':' &&
       c != LYX_META_HFILL &&
       c != LYX_META_PROTECTED_SEPARATOR &&
       c != LYX_META_NEWLINE) 
     s[1]='r';  /* right quote */
   else
     s[1]='l';     /* left quote */ 
   
   
   if (buffer[current_buffer]->parameters.quotes_times == 1)
     s[2]='s';			       /* single */
   else
     s[2]='d';			       /* double */
   
   s[3]='\0';
   
   new_inset = new InsetQuotes(s);
   
   buffer[current_buffer]->text->InsertInset(new_inset);
   
}

/* callbacks for form form_quotes */
void QuotesApplyCB(FL_OBJECT *ob, long data)
{
  if (buffer[current_buffer]->text){
     SetMinibuffer("user defined quotes set","","");
     buffer[current_buffer]->parameters.quotes_language = fl_get_choice(fd_form_quotes->choice_quotes_language) - 1;
   
     if (fl_get_button(fd_form_quotes->radio_single))   
     	buffer[current_buffer]->parameters.quotes_times = 1;
     else
     	buffer[current_buffer]->parameters.quotes_times = 2;
  }
}
void QuotesCancelCB(FL_OBJECT *ob, long data)
{
  fl_hide_form(fd_form_quotes->form_quotes);
}
void QuotesOKCB(FL_OBJECT *ob, long data)
{
  QuotesApplyCB(ob, data);
  QuotesCancelCB(ob, data);
}




/* callbacks for form form_preamble */
void PreambleCancelCB(FL_OBJECT *ob, long data)
{
  fl_hide_form(fd_form_preamble->form_preamble);
}

void PreambleApplyCB(FL_OBJECT *ob, long data)
{
   if (buffer[current_buffer]->text) {
      if (buffer[current_buffer]->parameters.preamble)
      	 delete(buffer[current_buffer]->parameters.preamble);
      buffer[current_buffer]->parameters.preamble = StringCopy(fl_get_input(fd_form_preamble->input_preamble));
      buffer[current_buffer]->lyx_current = 0;
      buffer[current_buffer]->bak_current = 0;
      buffer[current_buffer]->dvi_current = 0;
      SetMinibuffer("user defined LaTeX preamble set","","");
   }
}

void PreambleOKCB(FL_OBJECT *ob, long data)
{
  PreambleApplyCB(ob, data);
  PreambleCancelCB(ob, data);
}


/* callbacks for form form_table */
void TableApplyCB(FL_OBJECT *ob, long data){
   int xsize,ysize;
   if (!buffer[current_buffer]->screen)
     return;
   SetMinibuffer("Inssert table...","","");

   ysize = (int) (fl_get_slider_value(fd_form_table->slider_rows) + 0.5);
   xsize = (int) (fl_get_slider_value(fd_form_table->slider_columns) + 0.5);
   
   
   buffer[current_buffer]->screen->HideCursor();
   BeforeChange();
   Update(-2);
   
   int i, i2;
   
   buffer[current_buffer]->text->BreakParagraph();
   Update(-1);
   
   if (buffer[current_buffer]->text->cursor.par->Last()) {
      buffer[current_buffer]->text->CursorLeft();
      
      buffer[current_buffer]->text->BreakParagraph();
      Update(-1);
   }
   buffer[current_buffer]->text->current_font.latex = LYX_NO_LATEX;
   
   buffer[current_buffer]->text->SetLayout(NumberOfLayout(buffer[current_buffer]->parameters.textclass, "LaTeX"));

   if (buffer[current_buffer]->text->cursor.par->footnoteflag == LYX_NO_FOOTNOTE) {
      buffer[current_buffer]->text->SetParagraph(0, 0,
						 0, 0,
						 0.3 * buffer[current_buffer]->parameters.baseline,
						 0.3 * buffer[current_buffer]->parameters.baseline,
						 0, 0,
						 LYX_ALIGN_CENTER,
						 NULL,
						 0);
   }
   else
     buffer[current_buffer]->text->SetParagraph(0, 0,
						0, 0,
						0, 0,
						0, 0,
						LYX_ALIGN_CENTER, 
						NULL,
						0);
   Update(-1);
   
   
   InsetLatex *new_inset = new InsetLatex("\\begin{tabular}{");
   buffer[current_buffer]->text->InsertInset(new_inset);
   SmallUpdate(-1);
   
   LyXCursor tmpcursor = buffer[current_buffer]->text->cursor;
   
   buffer[current_buffer]->text->InsertChar('|');
   SmallUpdate(-1);
   
   for (i=0; i<xsize; i++) {
      buffer[current_buffer]->text->InsertChar('c');
      SmallUpdate(-1);
      buffer[current_buffer]->text->InsertChar('|');
      SmallUpdate(-1);
   }
   
   new_inset = new InsetLatex("}");
   buffer[current_buffer]->text->InsertInset(new_inset);
   SmallUpdate(-1);
   
   buffer[current_buffer]->text->InsertChar(LYX_META_NEWLINE);
   SmallUpdate(-1);
   
   new_inset = new InsetLatex("\\hline ");
   buffer[current_buffer]->text->InsertInset(new_inset);
   
   buffer[current_buffer]->text->InsertChar(LYX_META_NEWLINE);
   SmallUpdate(-1);
   
   for (i2=0; i2<ysize;i2++) {
      for (i=1; i<xsize; i++) {
	 new_inset = new InsetLatex("&");
	 buffer[current_buffer]->text->InsertInset(new_inset);
	 SmallUpdate(-1);
      }
      
      new_inset = new InsetLatex("\\\\");
      buffer[current_buffer]->text->InsertInset(new_inset);
      SmallUpdate(-1);
      buffer[current_buffer]->text->InsertChar(LYX_META_NEWLINE);
      SmallUpdate(-1);
      
      new_inset = new InsetLatex("\\hline ");
      buffer[current_buffer]->text->InsertInset(new_inset);
      SmallUpdate(-1);
      
      buffer[current_buffer]->text->InsertChar(LYX_META_NEWLINE);
      SmallUpdate(-1);
      
      if (i2==0) {
	 new_inset = new InsetLatex("\\hline ");
	 buffer[current_buffer]->text->InsertInset(new_inset);
	 SmallUpdate(-1);
	 
	 buffer[current_buffer]->text->InsertChar(LYX_META_NEWLINE);
	 SmallUpdate(-1);
      }
      
   }
   
   new_inset = new InsetLatex("\\end{tabular}");
   buffer[current_buffer]->text->InsertInset(new_inset);
   
   buffer[current_buffer]->text->SetCursor(tmpcursor.par, tmpcursor.pos);
   Update(1);
   SetMinibuffer("Table inserted","","");
}

void TableCancelCB(FL_OBJECT *ob, long data)
{
   fl_hide_form(fd_form_table->form_table);
}

void TableOKCB(FL_OBJECT *ob, long data)
{
   TableApplyCB(ob,data);
   TableCancelCB(ob,data);
}

/* callbacks for form form_print */
void PrintCancelCB(FL_OBJECT *ob, long data){
   fl_hide_form(fd_form_print->form_print);
}

void PrintApplyCB(FL_OBJECT *ob, long data){
   int i;
   if (!buffer[current_buffer]->text)
      return;

   char* pageflag = "";
   if (fl_get_button(fd_form_print->radio_even_pages))
      pageflag = "-B";
   else if (fl_get_button(fd_form_print->radio_odd_pages))
      pageflag = "-A";
   
   char* reverseflag = "";
   if (fl_get_button(fd_form_print->radio_order_revers))
      reverseflag = "-r";
   
   char* orientationflag = "";
   if (buffer[current_buffer]->parameters.orientation == ORIENTATION_LANDSCAPE)
      orientationflag = "-t landscape";
   
   const char* ps_file = fl_get_input(fd_form_print->input_file);
   
   for (i=0; ps_file[i]; i++);
   
   char *command = new char[i+100]; 
   
   
   const char* printer = fl_get_input(fd_form_print->input_printer);
   
   
   
   if (printer) {
      if (lyxrc.printer)
      	 delete(lyxrc.printer);
      lyxrc.printer = StringCopy(printer);
   }
   
   if (fl_get_button(fd_form_print->radio_file)) 
      sprintf(command, "%s %s %s %s -o \"%s\"", lyxrc.print_command, pageflag, reverseflag, orientationflag, ps_file);
   else {
      while (printer[0] == ' ')
      	 printer++;
      if (printer[0])
      	 sprintf(command, "%s %s %s %s -P%s", lyxrc.print_command, pageflag, reverseflag, orientationflag, printer);
      else
      	 sprintf(command, "%s %s %s %s", lyxrc.print_command, pageflag, reverseflag, orientationflag);
   }

   RunScript(0, command);
   delete(command);
}


void PrintOKCB(FL_OBJECT *ob, long data){
   PrintCancelCB(ob, data);  
   PrintApplyCB(ob,data);
}


/* callbacks for form form_figure */
void FigureApplyCB(FL_OBJECT *ob, long data){
   if (buffer[current_buffer]->text) {
      SetMinibuffer("Insert figure...","","");
      buffer[current_buffer]->screen->HideCursor();
      Update(-2);
      BeforeChange();
      
      buffer[current_buffer]->text->BreakParagraph();
      Update(-1);
      
      if (buffer[current_buffer]->text->cursor.par->Last()) {
	 buffer[current_buffer]->text->CursorLeft();
	 
	 buffer[current_buffer]->text->BreakParagraph();
	 Update(-1);
      }
      
      buffer[current_buffer]->text->SetLayout(NumberOfLayout(buffer[current_buffer]->parameters.textclass, "LaTeX"));
      
      if (buffer[current_buffer]->text->cursor.par->footnoteflag == LYX_NO_FOOTNOTE) {
	 buffer[current_buffer]->text->SetParagraph(0, 0,
						    0, 0,
						    0.3 * buffer[current_buffer]->parameters.baseline,
						    0.3 * buffer[current_buffer]->parameters.baseline,
						    0, 0,
						    LYX_ALIGN_CENTER, 
						    NULL,
						    0);
      }
      else
      	buffer[current_buffer]->text->SetParagraph(0, 0,
						   0, 0,
						   0, 0,
						   0, 0,
						   LYX_ALIGN_CENTER, 
						   NULL,
						   0);
      
      Update(-1);
      
      buffer[current_buffer]->text->current_font.latex = LYX_LATEX;
      
      InsetLatex *new_inset = NULL;
      
      if (fl_get_button(fd_form_figure->radio_latex)) {
	 new_inset = new InsetLatex("\\include ");
	 buffer[current_buffer]->text->InsertInset(new_inset);
      }
      else {
	 new_inset = new InsetLatex("\\epsfig{");
	 buffer[current_buffer]->text->InsertInset(new_inset);
	 SmallUpdate(-1);
	 new_inset = new InsetLatex("}");
	 buffer[current_buffer]->text->InsertInset(new_inset);
	 Update(-1);
	 buffer[current_buffer]->text->CursorLeft();
      }
      
      Update(0);
      SetMinibuffer("Figure inserted","","");
   }

}

void FigureCancelCB(FL_OBJECT *ob, long data)
{
   fl_hide_form(fd_form_figure->form_figure);
}

void FigureOKCB(FL_OBJECT *ob, long data)
{
   FigureApplyCB(ob,data);
   FigureCancelCB(ob,data);
}

void error_handler(int err_sig) {
   switch (err_sig) {
    case SIGHUP:
      fprintf(stderr, "\nlyx: SIGHUP signal trapped\n");
      break;
    case SIGFPE:
      fprintf(stderr, "\nlyx: SIGFPE signal trapped\n");
	break;
    case SIGSEGV:
	fprintf(stderr, "\nlyx: SIGSEGV signal trapped\n");
        fprintf(stderr, "(Sorry. If you can repeat this violation and have the current version of lyx,\n please send a bug report. Thanks.)\n");
	break;
    }
   
   
   /* deinstall the signal handlers */ 
   signal(SIGHUP, SIG_DFL);
   signal(SIGFPE, SIG_DFL);
   signal(SIGSEGV, SIG_DFL);

   char *s = NULL;
   int n;
   int i,a;
   int buf;
   for (buf=0; buf < NUMBER_OF_BUFFERS; buf++) {
	 if (buffer[buf]->parameters.paragraph && !buffer[buf]->lyx_current) {
	    
	    for (n=0; buffer[buf]->filename[n]; n++);
	    s = new char[n+5];
	    for (i=0; i<n; i++) 
	       s[i] = buffer[buf]->filename[i];
	    s[i] = '#';
	    s[i+1] = '\0';
	    for (a=i+2; a && s[a-1]!='/'; a--) {
	       s[a] = s[a-1];
	    }
	    s[a]='#';
	    
	    fprintf(stderr, "lyx: attempting to save buffer %s in %s...\n", 
		    buffer[buf]->filename, s);

	    if (buffer[buf]->text)
	       buffer[buf]->parameters.cursor = buffer[buf]->text->cursor;
	    
	    WriteFile(s, 
		      buffer[buf]->parameters);
	    buffer[buf]->lyx_current = 1;
	 
	    delete(s);
	 }
   }
   fprintf(stderr, "bye.\n");
   exit(0);
}

void ScreenApplyCB(FL_OBJECT *ob, long data){

   roman_font_name = (char*) fl_get_input(fd_form_screen->input_roman);
   sans_font_name = (char*) fl_get_input(fd_form_screen->input_sans);
   typewriter_font_name = (char*) fl_get_input(fd_form_screen->input_typewriter);
   
   lyxrc.default_zoom = atoi(fl_get_input(fd_form_screen->intinput_size));
   SetZoom(lyxrc.default_zoom);
   
   /* all buffers will need resize */ 
   for (int i =0; i < NUMBER_OF_BUFFERS; i++) {
      if (buffer[i]->screen)
      	 delete(buffer[i]->screen);
      buffer[i]->screen = NULL;
   }
   RedoCurrentBuffer();
   SetMinibuffer("New screen options set","","");
}

void ScreenCancelCB(FL_OBJECT *ob, long data)
{
   fl_hide_form(fd_form_screen->form_screen);
}


void ScreenOKCB(FL_OBJECT *ob, long data)
{
   ScreenCancelCB(ob,data);
   ScreenApplyCB(ob,data);
}

/* these functions are for the spellchecker */ 
char* NextWord(float &value) {
  if (!buffer[current_buffer]->text){
    value = 1;
    return NULL;
  }
   
  char* string =  buffer[current_buffer]->text->SelectNextWord(value);

  return string;
}

void SelectLastWord(){
   if (!buffer[current_buffer]->text)
      return;
   if (work_area_active)
      buffer[current_buffer]->screen->HideCursor();
   BeforeChange(); 
   buffer[current_buffer]->text->SelectSelectedWord();
   buffer[current_buffer]->screen->ToggleSelection();
   Update(0);
}


void EndOfSpellCheck(){
   if (!buffer[current_buffer]->text)
      return;
   if (work_area_active)
      buffer[current_buffer]->screen->HideCursor();
   BeforeChange(); 
   buffer[current_buffer]->text->SelectSelectedWord();
   buffer[current_buffer]->text->ClearSelection();
   Update(0);
}


void ReplaceWord(const char* replacestring) {
   if (!buffer[current_buffer]->screen)
      return;
   buffer[current_buffer]->screen->HideCursor();
   Update(-2);
   /* clear the selection (if there is any) */ 
   buffer[current_buffer]->screen->ToggleSelection();
   buffer[current_buffer]->text->ReplaceSelectionWithString((char*) replacestring);
   
   buffer[current_buffer]->text->SetSelectionOverString((char*) replacestring);
   Update(1);
}


static int ScrollUpCB(long time) {
   if (!buffer[current_buffer]->screen)
      return 0;
   
   double value= fl_get_slider_value(fd_form_main->scrollbar);
   
   if (value == 0)
      return 0;
   
   float add_value =  (buffer[current_buffer]->text->DefaultHeight()
		       + (float)(time) * (float)(time) * 0.125);
   if (add_value > work_area_height)
      add_value = (float) (work_area_height - buffer[current_buffer]->text->DefaultHeight());
   
   value -= add_value;

   if (value < 0)
      value = 0;
   
   fl_set_slider_value(fd_form_main->scrollbar, value);
   
   ScrollCB(fd_form_main->scrollbar,0); 
   return 0;
}

static int ScrollDownCB(long time) {
   if (!buffer[current_buffer]->screen)
      return 0;
   
   double value= fl_get_slider_value(fd_form_main->scrollbar);
   double min, max;
   fl_get_slider_bounds(fd_form_main->scrollbar, &min, &max);

   if (value == max)
      return 0;
   
   float add_value =  (buffer[current_buffer]->text->DefaultHeight()
		       + (float)(time) * (float)(time) * 0.125);
   if (add_value > work_area_height)
      add_value = (float) (work_area_height - buffer[current_buffer]->text->DefaultHeight());
   
   value += add_value;
   
   if (value > max)
      value = max;
   
   fl_set_slider_value(fd_form_main->scrollbar, value);
   
   ScrollCB(fd_form_main->scrollbar,0); 
   return 0;
}


static void ScrollUpOnePageCB(long time) {
   if (!buffer[current_buffer]->screen)
      return;
   long y = buffer[current_buffer]->screen->first;
   if (!y)
      return;
   Row* row = buffer[current_buffer]->text->GetRowNearY(y);
   y = y - work_area_height + row->height;
   buffer[current_buffer]->screen->Draw(y);
   UpdateScrollbar();
}

static void ScrollDownOnePageCB(long time) {
   if (!buffer[current_buffer]->screen)
      return;
   double min, max;
   fl_get_slider_bounds(fd_form_main->scrollbar, &min, &max);
   long y = buffer[current_buffer]->screen->first;
   if (y > buffer[current_buffer]->text->height - work_area_height)
      return;
   y += work_area_height;
   buffer[current_buffer]->text->GetRowNearY(y);
   buffer[current_buffer]->screen->Draw(y);
   UpdateScrollbar();
}

void DownCB(FL_OBJECT *ob, long data) {
   const XEvent*ev2;
   static unsigned int button = Button1;
   static long time = 0;
   ev2 = fl_last_event();
   if (ev2->type == ButtonPress || ev2->type == ButtonRelease) {
     button = ev2->xbutton.button;
     time = 0;
   }
   
   switch (button) {
   case Button2: ScrollUpOnePageCB(time++); break;
   case Button3: ScrollDownOnePageCB(time++); break;
   default: ScrollDownCB(time++); break;
   }
}

void UpCB(FL_OBJECT *ob, long data) {
   const XEvent*ev2;
   static unsigned int button = Button1;
   static long time = 0;
   ev2 = fl_last_event();
   if (ev2->type == ButtonPress || ev2->type == ButtonRelease) {
     button = ev2->xbutton.button;
     time = 0;
   }
   
   switch (button) {
   case Button3: ScrollUpOnePageCB(time++); break;
   case Button2: ScrollDownOnePageCB(time++); break;
   default: ScrollUpCB(time++); break;
   }
}


struct TocList {
  char counter[6];
  TocList *next;
};

static TocList* toclist = NULL;


void TocSelectCB(FL_OBJECT *ob, long data) {
   if (!buffer[current_buffer]->text) 
      return;
   TocList* tmptoclist = toclist;
   int i = fl_get_browser(ob);
   int a = 0;

   for (a=1; a<i && tmptoclist->next; a++){
     tmptoclist = tmptoclist->next;
   }

   if (!tmptoclist)
     return;
     

   LyXParagraph *par = buffer[current_buffer]->parameters.paragraph;
   while (par && 
	  (
	   par->GetCounter(0) != tmptoclist->counter[0] ||
	   par->GetCounter(1) != tmptoclist->counter[1] ||
	   par->GetCounter(2) != tmptoclist->counter[2] ||
	   par->GetCounter(3) != tmptoclist->counter[3] ||
	   par->GetCounter(4) != tmptoclist->counter[4] ||
	   par->GetCounter(5) != tmptoclist->counter[5]
	   )
	  ) {
      par = par->LastPhysicalPar()->Next();
   }
   
   if (par) {
      BeforeChange();
      buffer[current_buffer]->text->SetCursor(par, 0);
      buffer[current_buffer]->text->sel_cursor = buffer[current_buffer]->text->cursor;
      Update(0);
   }
   else {
      WriteAlert("Error", "Couldn't find this label", "in current document.");
   }
	  
}


void TocCancelCB(FL_OBJECT *ob, long data) {
   fl_hide_form(fd_form_toc->form_toc);
}

void TocUpdateCB(FL_OBJECT *ob, long data){
   static LyXParagraph* stapar = NULL;
   TocList *tmptoclist = NULL;
   
   /* deletet the toclist */ 
   if (toclist){
     while (toclist){
       tmptoclist = toclist->next;
       delete(toclist);
       toclist = tmptoclist;
     }
   }
   toclist = NULL;
   tmptoclist = toclist;


   fl_clear_browser(fd_form_toc->browser_toc);
   if (!buffer[current_buffer]->text) {
      fl_add_browser_line(fd_form_toc->browser_toc, "*** No Document ***");
      return;
   }
   fl_hide_object(fd_form_toc->browser_toc);
   /* get the table of contents */ 
   LyXParagraph *par = buffer[current_buffer]->parameters.paragraph;
   char labeltype;
   char* line = new char[200];
   int i = 0;
   int pos = 0;
   unsigned char c;
   int topline = 0;
   
   if (stapar == par)
      topline = fl_get_browser_topline(fd_form_toc->browser_toc);
   stapar = par;
   
   while (par) {
      labeltype = style(buffer[current_buffer]->parameters.textclass, 
			par->GetLayout())->labeltype;   
      if (labeltype >= LABEL_COUNTER_CHAPTER && labeltype <= LABEL_COUNTER_CHAPTER + buffer[current_buffer]->parameters.tocdepth) {
	 /* inset this into the table of contents */ 
	  /* first indent a little bit */ 

	 for (pos=0; 
	      pos < (labeltype - 
		     TextClass(buffer[current_buffer]->parameters.textclass)->maxcounter) * 4 + 2;
	      pos++)
	    line[pos] = ' ';
	 
	 /* then the labelstring */ 
	 
	 i = 0;
	 if (par->labelstring) {
	    while (pos < 199 && par->labelstring[i]) {
	       line[pos] = par->labelstring[i];
	       i++;
	       pos++;
	    }
	 }
	 
	 line[pos] = ' ';
	 pos++;
	 
	 /* now the contents */ 
	 i = 0;
	 while (pos < 199 && i < par->last) {
	    c = par->GetChar(i);
	    if (isprint(c) || c >= 128) {
	       line[pos] = c;
	       pos++;
	    }
	    i++;
	 }
	 line[pos] = '\0';
	 fl_add_browser_line(fd_form_toc->browser_toc, line);
	 
	 /* make a toclist entry */
	 
	 if (!tmptoclist){
	   tmptoclist = new TocList;
	   toclist = tmptoclist;
	 }
	 else {
	   tmptoclist->next = new TocList;
	   tmptoclist = tmptoclist->next;
	 }

	 tmptoclist->next = NULL;
	 int a = 0;
	 for (a=0; a<6; a++){
	   tmptoclist->counter[a] = par->GetCounter(a);
	 }
      }
      par = par->LastPhysicalPar()->Next();
	 
   }
   delete (line);
   fl_set_browser_topline(fd_form_toc->browser_toc, topline);
   fl_show_object(fd_form_toc->browser_toc);
}

