/* This file is part of
* ======================================================
* 
*           LyX, the High Level Word Processor
* 	 
*	    Copyright (C) 1995 Matthias Ettrich
*
*======================================================*/

#include <forms.h>
#include "lyx.h"

#ifdef TWO_COLOR_ICONS
#include "cut_bw.xpm"
#include "emph_bw.xpm"
#include "fig_bw.xpm"
#include "foot_bw.xpm"
#include "form_bw.xpm"
#include "depth_bw.xpm"
#include "margin_bw.xpm"
#include "melt_bw.xpm"
#include "noun_bw.xpm"
#include "paste_bw.xpm"
#include "free_bw.xpm"
#include "tab_bw.xpm"
#include "tex_bw.xpm"
#else 
#include "cut.xpm"
#include "emph.xpm"
#include "fig.xpm"
#include "foot.xpm"
#include "form.xpm"
#include "depth.xpm"
#include "margin.xpm"
#include "melt.xpm"
#include "noun.xpm"
#include "paste.xpm"
#include "free.xpm"
#include "tab.xpm"
#include "tex.xpm"
#endif

#include "up.xpm"
#include "down.xpm"



FD_form_main *fd_form_main;
FD_form_title *fd_form_title;
FD_form_paragraph *fd_form_paragraph;
FD_form_search *fd_form_search;
FD_form_character *fd_form_character;
FD_form_document *fd_form_document;
FD_form_quotes *fd_form_quotes;
FD_form_preamble *fd_form_preamble;
FD_form_table *fd_form_table;
FD_form_print *fd_form_print;
FD_form_figure *fd_form_figure;
FD_form_screen *fd_form_screen;
FD_form_toc *fd_form_toc;

extern void init(int argc, char **argv);

static char form_has_focus = 0;
extern char work_area_active;


int focus_event(FL_FORM *form, void *xev){
   form_has_focus = (((XEvent *)xev)->type == EnterNotify);
   return 0;
}


int main(int argc, char *argv[])
{
   FL_IOPT cntl;
   cntl.buttonFontSize = FL_MEDIUM_SIZE;
   cntl.browserFontSize = FL_MEDIUM_SIZE;
   cntl.labelFontSize = FL_MEDIUM_SIZE;
   cntl.choiceFontSize = FL_MEDIUM_SIZE;
   cntl.inputFontSize = FL_MEDIUM_SIZE;
   cntl.pupFontSize = FL_MEDIUM_SIZE;   
   cntl.borderWidth = -2;
   fl_set_defaults(FL_PDButtonFontSize, &cntl);  
   fl_set_defaults(FL_PDBrowserFontSize, &cntl);  
   fl_set_defaults(FL_PDLabelFontSize, &cntl);  
   fl_set_defaults(FL_PDChoiceFontSize, &cntl);  
   fl_set_defaults(FL_PDInputFontSize, &cntl);  
   fl_set_defaults(FL_PDPupFontSize, &cntl);   
   fl_set_defaults(FL_PDBorderWidth, &cntl);
   
   fl_setpup_color(FL_MCOL,FL_BLACK);
		   
   fl_initialize(argv[0], "Lyx", 0, 0, &argc, argv);

   fd_form_main = create_form_form_main();
   fd_form_title = create_form_form_title();
   fd_form_paragraph = create_form_form_paragraph();
   fd_form_search = create_form_form_search();
   fd_form_character = create_form_form_character();
   fd_form_document = create_form_form_document();
   fd_form_quotes = create_form_form_quotes();
   fd_form_preamble = create_form_form_preamble();
   fd_form_table = create_form_form_table();
   fd_form_print = create_form_form_print();
   fd_form_figure = create_form_form_figure();
   fd_form_screen = create_form_form_screen();
   fd_form_toc = create_form_form_toc();
   
   /* fill-in form initialization code */

    fl_set_pixmapbutton_data(fd_form_main->icon_foot,foot_xpm);
    fl_set_pixmapbutton_data(fd_form_main->icon_emph,emph_xpm);
    fl_set_pixmapbutton_data(fd_form_main->icon_noun,noun_xpm);
    fl_set_pixmapbutton_data(fd_form_main->icon_margin,margin_xpm);
    fl_set_pixmapbutton_data(fd_form_main->icon_figure,fig_xpm);
    fl_set_pixmapbutton_data(fd_form_main->icon_table,tab_xpm);
    fl_set_pixmapbutton_data(fd_form_main->icon_cut,cut_xpm);
    fl_set_pixmapbutton_data(fd_form_main->icon_paste,paste_xpm);
    fl_set_pixmapbutton_data(fd_form_main->icon_melt,melt_xpm);
    fl_set_pixmapbutton_data(fd_form_main->icon_tex,tex_xpm);
    fl_set_pixmapbutton_data(fd_form_main->icon_formula,form_xpm);
    fl_set_pixmapbutton_data(fd_form_main->icon_depth,depth_xpm);
    fl_set_pixmapbutton_data(fd_form_main->icon_free,free_xpm);          
   
   fl_set_pixmapbutton_data(fd_form_main->button_up,up_xpm);          
   fl_set_pixmapbutton_data(fd_form_main->button_down,down_xpm);          
   
   
   
   
   
   
   
   
   
   fl_addto_menu(fd_form_main->menu_file, 
		 " Open... (C-x C-f) | New... (C-x C-d) | Write (C-x C-s) | Write as... (C-x C-w) | Run LaTeX (C-x r) | Preview (xdvi) (C-x p) | Preview (ghostscript) (C-x g) | Print... | Make (nice) LaTeX-file | Insert Ascii File (Lines) | Insert Ascii File (Paragraphs) | Close Buffer (C-x k) | Quit (C-x C-c) ");
   fl_addto_menu(fd_form_main->menu_edit,
		 " Load autosave file (C-x u) | Force autosave (C-x a) | Cut (C-w) | Copy (M-w) | Paste (C-y) | Search & Replace (C-s) | Goto Error (C-g) | Open/Close (C-o) | Open All Floats | Close All Floats | Screen Options... | Spell Checker... | Table Of Contents...(C-x t)");
   fl_addto_menu(fd_form_main->menu_layout,
		 " Character... | Paragraph... | Document... | Quotes... | Toggle Emphasize Style | Toggle Noun Style | Toggle TeX Style | Toggle Math Style | Change Environment Depth | LaTeX Preamble...");
   fl_addto_menu(fd_form_main->menu_special,
		 " Figure | Table | Figure Float | Table Float | Footnote Float | Marginpar Float | Melt Float | Table of Contents | List of Figures | List of Tables | HFill (C-i) | Hyphenation Point (C-h) | Protected Blank (M-space) | Newline (C-enter) ");
   fl_addto_menu(fd_form_main->menu_math, 
		 " frac | sqrt | ^ (Exponent) | _ (Index) | sum | int ");
   fl_addto_menu(fd_form_main->menu_help, 
		 " Copying... | Warranty... | Version... ");
   

   fl_set_goodies_font(FL_NORMAL_STYLE, FL_MEDIUM_SIZE);
   

   fl_addto_choice(fd_form_document->choice_language,
		   "default|american|austrian|bahasa|brazil|catalan|croatian|czech|danish|dutch|english|esperanto|finnish|francais|french|galician|german|hungarian|italian|magyar|norsk|polish|portuges|romanian|russian|spanish|slovak|slovene|swedish|turkish");

   fl_addto_choice(fd_form_document->choice_fontsize, "10|11|12");

   fl_addto_choice(fd_form_document->choice_fonts, 
		   "default|times|palatino|helvet|avant|newcent|bookman");
   
   fl_addto_choice(fd_form_document->choice_inputenc, "default|latin1|latin2");

   fl_addto_choice(fd_form_document->choice_papersize,
		   " A4 normal Borders | A4 small Borders (only portrait) | A4 very small Borders (only portrait) | A4 very wide borders (only portrait) | US Letter | A5 | B5 | legal | executive ");
   
   fl_addto_choice(fd_form_document->choice_pagestyle,
		   " Plain | Headings | Empty ");


   fl_addto_choice(fd_form_quotes->choice_quotes_language,
		   " German | English | French ");

   fl_addto_choice(fd_form_character->choice_family, 
		   " no change | Roman | Sans Serif | Typewriter ");
   fl_addto_choice(fd_form_character->choice_series, 
		   " no change | normal | bold ");
   fl_addto_choice(fd_form_character->choice_shape, 
		   " no change | up | italic | small caps ");
   fl_addto_choice(fd_form_character->choice_size, 
		   " no change | small | normal | large | larger | largest | huge | giant ");
   
   
   fl_set_timer(fd_form_title->timer_title, 15);
   
   /* show the first form */
   fl_show_form(fd_form_main->form_main,
		FL_PLACE_CENTER | FL_FREE_SIZE,FL_FULLBORDER,
		"LyX");
   fl_show_form(fd_form_title->form_title, 
		FL_PLACE_CENTER, FL_NOBORDER, 
		"Title");
   init(argc, argv);
   

   fl_register_raw_callback(fd_form_main->form_main, EnterWindowMask | LeaveWindowMask, 
			    focus_event);
   
   XEvent ev;
   Window win;
   int tmp;
   
   XGetInputFocus(fl_display, &win, &tmp);
   form_has_focus = (win == fd_form_main->form_main->window);
   
   while(1) {
      /* this is a hack to reduce processor time that lyx needs 
       * I cannot use fl_do_forms, since this seems to be buggy
       * in XForms 0.75 (hanging scrollbar :-( )*/
      
      /* the work_area_active let LyX react faster in the
       * work area, but makes it little unfriendly against other
       * applications :-( 
       * with fd_check_forms this is necessary because of the
       / xform timer events */ 
      
      if (!form_has_focus && !work_area_active)
      	 XPeekEvent(fl_display, &ev);
      fl_check_forms();
   }
   return 0;
}







      
